@@ -5,9 +5,7 @@ from sympy.abc import x, y
 from sympy.assumptions.assume import global_assumptions, Predicate
 from sympy.assumptions.ask import _extract_facts, Q
 from sympy.core import symbols
-from sympy.logic.boolalg import Or
 from sympy.printing import pretty
-from sympy.utilities.pytest import XFAIL
 
 
 def test_equal():
@@ -20,7 +18,7 @@ def test_equal():
 def test_pretty():
     assert pretty(Q.positive(x)) == "Q.positive(x)"
     assert pretty(
-        set([Q.positive, Q.integer])) == "set([Q.integer, Q.positive])"
+        {Q.positive, Q.integer}) == "{Q.integer, Q.positive}"
 
 
 def test_extract_facts():
