@@ -122,22 +122,19 @@ There is a function constructing a loop (or a complete function) like this in
 
 """
 
-from __future__ import print_function, division
+from typing import Any, Dict, List
 
-from functools import total_ordering
-from itertools import chain
 from collections import defaultdict
 from sympy.core import Symbol, Tuple, Dummy
 from sympy.core.basic import Basic
 from sympy.core.expr import Expr
-from sympy.core.compatibility import string_types
 from sympy.core.numbers import Float, Integer, oo
 from sympy.core.relational import Lt, Le, Ge, Gt
 from sympy.core.sympify import _sympify, sympify, SympifyError
-from sympy.logic import true, false
 from sympy.utilities.iterables import iterable
 
 
+
 def _mk_Tuple(args):
     """
     Create a Sympy Tuple object from an iterable, converting Python strings to
@@ -145,14 +142,16 @@ def _mk_Tuple(args):
 
     Parameters
     ==========
+
     args: iterable
         Arguments to :class:`sympy.Tuple`.
 
     Returns
     =======
+
     sympy.Tuple
     """
-    args = [String(arg) if isinstance(arg, string_types) else arg for arg in args]
+    args = [String(arg) if isinstance(arg, str) else arg for arg in args]
     return Tuple(*args)
 
 
@@ -168,12 +167,12 @@ class Token(Basic):
     Subclasses should not need to override the ``__new__()`` method. They may
     define a class or static method named ``_construct_<attr>`` for each
     attribute to process the value passed to ``__new__()``. Attributes listed
-    in the class attribute ``not_in_args`` are not passed to :class:`sympy.Basic`.
+    in the class attribute ``not_in_args`` are not passed to :class:`~.Basic`.
     """
 
-    __slots__ = []
-    defaults = {}
-    not_in_args = []
+    __slots__ = ()
+    defaults = {}  # type: Dict[str, Any]
+    not_in_args = []  # type: List[str]
     indented_args = ['body']
 
     @property
@@ -188,6 +187,7 @@ class Token(Basic):
     @classmethod
     def _construct(cls, attr, arg):
         """ Construct an attribute value from argument passed to ``__new__()``. """
+        # arg may be ``NoneToken()``, so comparation is done using == instead of ``is`` operator
         if arg == None:
             return cls.defaults.get(attr, none)
         else:
@@ -254,7 +254,7 @@ class Token(Basic):
         return tuple([getattr(self, attr) for attr in self.__slots__])
 
     def __hash__(self):
-        return super(Token, self).__hash__()
+        return super().__hash__()
 
     def _joiner(self, k, indent_level):
         return (',\n' + ' '*indent_level) if k in self.indented_args else ', '
@@ -265,7 +265,7 @@ class Token(Basic):
             if isinstance(arg, Token):
                 return printer._print(arg, *args, joiner=self._joiner(k, il), **kwargs)
             else:
-                return printer._print(v, *args, **kwargs)
+                return printer._print(arg, *args, **kwargs)
 
         if isinstance(v, Tuple):
             joined = self._joiner(k, il).join([_print(arg) for arg in v.args])
@@ -298,7 +298,7 @@ class Token(Basic):
                 indented = self._indented(printer, attr, value, *args, **kwargs)
             arg_reprs.append(('{1}' if i == 0 else '{0}={1}').format(attr, indented.lstrip()))
 
-        return "{0}({1})".format(self.__class__.__name__, joiner.join(arg_reprs))
+        return "{}({})".format(self.__class__.__name__, joiner.join(arg_reprs))
 
     _sympystr = _sympyrepr
 
@@ -311,6 +311,7 @@ class Token(Basic):
 
         Parameters
         ==========
+
         exclude : collection of str
             Collection of keywords to exclude.
 
@@ -382,7 +383,7 @@ class NoneToken(Token):
         return ()
 
     def __hash__(self):
-        return super(Token, self).__hash__()
+        return super().__hash__()
 
 
 none = NoneToken()
@@ -404,7 +405,7 @@ class AssignmentBase(Basic):
 
         cls._check_args(lhs, rhs)
 
-        return super(AssignmentBase, cls).__new__(cls, lhs, rhs)
+        return super().__new__(cls, lhs, rhs)
 
     @property
     def lhs(self):
@@ -496,6 +497,7 @@ class AugmentedAssignment(AssignmentBase):
        Symbol for binary operation being applied in the assignment, such as "+",
        "*", etc.
     """
+    binop = None  # type: str
 
     @property
     def op(self):
@@ -597,8 +599,8 @@ class CodeBlock(Basic):
         Return a new CodeBlock with common subexpressions eliminated and
         pulled out as assignments.
 
-    Example
-    =======
+    Examples
+    ========
 
     >>> from sympy import symbols, ccode
     >>> from sympy.codegen.ast import CodeBlock, Assignment
@@ -628,18 +630,17 @@ class CodeBlock(Basic):
         return iter(self.args)
 
     def _sympyrepr(self, printer, *args, **kwargs):
-        from sympy.printing.printer import printer_context
         il = printer._context.get('indent_level', 0)
         joiner = ',\n' + ' '*il
         joined = joiner.join(map(printer._print, self.args))
-        return ('{0}(\n'.format(' '*(il-4) + self.__class__.__name__,) +
+        return ('{}(\n'.format(' '*(il-4) + self.__class__.__name__,) +
                 ' '*il + joined + '\n' + ' '*(il - 4) + ')')
 
     _sympystr = _sympyrepr
 
     @property
     def free_symbols(self):
-        return super(CodeBlock, self).free_symbols - set(self.left_hand_sides)
+        return super().free_symbols - set(self.left_hand_sides)
 
     @classmethod
     def topological_sort(cls, assignments):
@@ -654,8 +655,8 @@ class CodeBlock(Basic):
         This is a class constructor so that the default constructor for
         CodeBlock can error when variables are used before they are assigned.
 
-        Example
-        =======
+        Examples
+        ========
 
         >>> from sympy import symbols
         >>> from sympy.codegen.ast import CodeBlock, Assignment
@@ -828,7 +829,7 @@ class For(Token):
         ))
     ))
     """
-    __slots__ = ['target', 'iterable', 'body']
+    __slots__ = ('target', 'iterable', 'body')
     _construct_target = staticmethod(_sympify)
 
     @classmethod
@@ -872,13 +873,13 @@ class String(Token):
     String('foo')
 
     """
-    __slots__ = ['text']
+    __slots__ = ('text',)
     not_in_args = ['text']
     is_Atom = True
 
     @classmethod
     def _construct_text(cls, text):
-        if not isinstance(text, string_types):
+        if not isinstance(text, str):
             raise TypeError("Argument text is not a string type.")
         return text
 
@@ -913,9 +914,9 @@ class Node(Token):
 
     """
 
-    __slots__ = ['attrs']
+    __slots__ = ('attrs',)
 
-    defaults = {'attrs': Tuple()}
+    defaults = {'attrs': Tuple()}  # type: Dict[str, Any]
 
     _construct_attrs = staticmethod(_mk_Tuple)
 
@@ -979,7 +980,7 @@ class Type(Token):
     .. [1] https://docs.scipy.org/doc/numpy/user/basics.types.html
 
     """
-    __slots__ = ['name']
+    __slots__ = ('name',)
 
     _construct_name = String
 
@@ -1045,7 +1046,6 @@ class Type(Token):
             Absolute tolerance (in addition to ``rtol``).
         limits : dict
             Values given by ``limits.h``, x86/IEEE754 defaults if not given.
-            Default: :attr:`default_limits`.
         type_aliases : dict
             Maps substitutions for Type, e.g. {integer: int64, real: float32}
 
@@ -1082,7 +1082,6 @@ class Type(Token):
         0.123456789012345649
 
         """
-        from sympy.functions.elementary.complexes import im, re
         val = sympify(value)
 
         ten = Integer(10)
@@ -1106,12 +1105,12 @@ class Type(Token):
 
 class IntBaseType(Type):
     """ Integer base type, contains no size information. """
-    __slots__ = ['name']
+    __slots__ = ('name',)
     cast_nocheck = lambda self, i: Integer(int(i))
 
 
 class _SizedIntType(IntBaseType):
-    __slots__ = ['name', 'nbits']
+    __slots__ = ('name', 'nbits',)
 
     _construct_nbits = Integer
 
@@ -1190,7 +1189,7 @@ class FloatType(FloatBaseType):
     ValueError: Maximum value for data type smaller than new value.
     """
 
-    __slots__ = ['name', 'nbits', 'nmant', 'nexp']
+    __slots__ = ('name', 'nbits', 'nmant', 'nexp',)
 
     _construct_nbits = _construct_nmant = _construct_nexp = Integer
 
@@ -1250,6 +1249,10 @@ class FloatType(FloatBaseType):
 
     def cast_nocheck(self, value):
         """ Casts without checking if out of bounds or subnormal. """
+        if value == oo:  # float(oo) or oo
+            return float(oo)
+        elif value == -oo:  # float(-oo) or -oo
+            return float(-oo)
         return Float(str(sympify(value).evalf(self.decimal_dig)), self.decimal_dig)
 
     def _check(self, value):
@@ -1266,14 +1269,14 @@ class ComplexBaseType(FloatBaseType):
         """ Casts without checking if out of bounds or subnormal. """
         from sympy.functions import re, im
         return (
-            super(ComplexBaseType, self).cast_nocheck(re(value)) +
-            super(ComplexBaseType, self).cast_nocheck(im(value))*1j
+            super().cast_nocheck(re(value)) +
+            super().cast_nocheck(im(value))*1j
         )
 
     def _check(self, value):
         from sympy.functions import re, im
-        super(ComplexBaseType, self)._check(re(value))
-        super(ComplexBaseType, self)._check(im(value))
+        super()._check(re(value))
+        super()._check(im(value))
 
 
 class ComplexType(ComplexBaseType, FloatType):
@@ -1335,8 +1338,9 @@ class Attribute(Token):
     >>> a.parameters == (1, 2, 3)
     True
     """
-    __slots__ = ['name', 'parameters']
+    __slots__ = ('name', 'parameters')
     defaults = {'parameters': Tuple()}
+
     _construct_name = String
     _construct_parameters = staticmethod(_mk_Tuple)
 
@@ -1401,11 +1405,10 @@ class Variable(Node):
 
     """
 
-    __slots__ = ['symbol', 'type', 'value'] + Node.__slots__
-    defaults = dict(chain(Node.defaults.items(), {
-        'type': untyped,
-        'value': none
-    }.items()))
+    __slots__ = ('symbol', 'type', 'value') + Node.__slots__
+
+    defaults = Node.defaults.copy()
+    defaults.update({'type': untyped, 'value': none})
 
     _construct_symbol = staticmethod(sympify)
     _construct_value = staticmethod(sympify)
@@ -1465,10 +1468,15 @@ class Variable(Node):
         Examples
         ========
 
-        >>> from sympy.codegen.ast import Variable
+        >>> from sympy.codegen.ast import Variable, NoneToken
         >>> x = Variable('x')
         >>> decl1 = x.as_Declaration()
-        >>> decl1.variable.value == None
+        >>> # value is special NoneToken() which must be tested with == operator
+        >>> decl1.variable.value is None  # won't work
+        False
+        >>> decl1.variable.value == None  # not PEP-8 compliant
+        True
+        >>> decl1.variable.value == NoneToken()  # OK
         True
         >>> decl2 = x.as_Declaration(value=42.0)
         >>> decl2.variable.value == 42
@@ -1507,7 +1515,7 @@ class Pointer(Variable):
     >>> i = Symbol('i', integer=True)
     >>> p = Pointer('x')
     >>> p[i+1]
-    Element(x, indices=((i + 1,),))
+    Element(x, indices=(i + 1,))
 
     """
 
@@ -1537,7 +1545,7 @@ class Element(Token):
     'x[i*l + j*m + k*n + o]'
 
     """
-    __slots__ = ['symbol', 'indices', 'strides', 'offset']
+    __slots__ = ('symbol', 'indices', 'strides', 'offset')
     defaults = {'strides': none, 'offset': none}
     _construct_symbol = staticmethod(sympify)
     _construct_indices = staticmethod(lambda arg: Tuple(*arg))
@@ -1557,14 +1565,19 @@ class Declaration(Token):
     ========
 
     >>> from sympy import Symbol
-    >>> from sympy.codegen.ast import Declaration, Type, Variable, integer, untyped
+    >>> from sympy.codegen.ast import Declaration, Type, Variable, NoneToken, integer, untyped
     >>> z = Declaration('z')
     >>> z.variable.type == untyped
     True
-    >>> z.variable.value == None
+    >>> # value is special NoneToken() which must be tested with == operator
+    >>> z.variable.value is None  # won't work
+    False
+    >>> z.variable.value == None  # not PEP-8 compliant
+    True
+    >>> z.variable.value == NoneToken()  # OK
     True
     """
-    __slots__ = ['variable']
+    __slots__ = ('variable',)
     _construct_variable = Variable
 
 
@@ -1578,7 +1591,7 @@ class While(Token):
     Parameters
     ==========
 
-    condition : expression convertable to Boolean
+    condition : expression convertible to Boolean
     body : CodeBlock or iterable
         When passed an iterable it is used to instantiate a CodeBlock.
 
@@ -1595,7 +1608,7 @@ class While(Token):
     ... ])
 
     """
-    __slots__ = ['condition', 'body']
+    __slots__ = ('condition', 'body')
     _construct_condition = staticmethod(lambda cond: _sympify(cond))
 
     @classmethod
@@ -1616,7 +1629,7 @@ class Scope(Token):
         When passed an iterable it is used to instantiate a CodeBlock.
 
     """
-    __slots__ = ['body']
+    __slots__ = ('body',)
 
     @classmethod
     def _construct_body(cls, itr):
@@ -1649,7 +1662,7 @@ class Stream(Token):
     print("x", file=sys.stderr)
 
     """
-    __slots__ = ['name']
+    __slots__ = ('name',)
     _construct_name = String
 
 stdout = Stream('stdout')
@@ -1675,7 +1688,7 @@ class Print(Token):
 
     """
 
-    __slots__ = ['print_args', 'format_string', 'file']
+    __slots__ = ('print_args', 'format_string', 'file')
     defaults = {'format_string': none, 'file': none}
 
     _construct_print_args = staticmethod(_mk_Tuple)
@@ -1709,7 +1722,7 @@ class FunctionPrototype(Node):
 
     """
 
-    __slots__ = ['return_type', 'name', 'parameters', 'attrs']
+    __slots__ = ('return_type', 'name', 'parameters', 'attrs')
 
     _construct_return_type = Type
     _construct_name = String
@@ -1763,7 +1776,7 @@ class FunctionDefinition(FunctionPrototype):
     }
     """
 
-    __slots__ = FunctionPrototype.__slots__[:-1] + ['body', 'attrs']
+    __slots__ = FunctionPrototype.__slots__[:-1] + ('body', 'attrs')
 
     @classmethod
     def _construct_body(cls, itr):
@@ -1802,7 +1815,7 @@ class FunctionCall(Token, Expr):
     foo(bar, baz)
 
     """
-    __slots__ = ['name', 'function_args']
+    __slots__ = ('name', 'function_args')
 
     _construct_name = String
     _construct_function_args = staticmethod(lambda args: Tuple(*args))
