@@ -1,4 +1,3 @@
-from __future__ import absolute_import
 import numbers as nums
 import decimal
 from sympy import (Rational, Symbol, Float, I, sqrt, cbrt, oo, nan, pi, E,
@@ -7,7 +6,6 @@ from sympy import (Rational, Symbol, Float, I, sqrt, cbrt, oo, nan, pi, E,
                    Number, zoo, log, Mul, Pow, Tuple, latex, Gt, Lt, Ge, Le,
                    AlgebraicNumber, simplify, sin, fibonacci, RealField,
                    sympify, srepr, Dummy, Sum)
-from sympy.core.compatibility import long
 from sympy.core.logic import fuzzy_not
 from sympy.core.numbers import (igcd, ilcm, igcdex, seterr,
     igcd2, igcd_lehmer, mpf_norm, comp, mod_inverse)
@@ -15,12 +13,12 @@ from sympy.core.power import integer_nthroot, isqrt, integer_log
 from sympy.polys.domains.groundtypes import PythonRational
 from sympy.utilities.decorator import conserve_mpmath_dps
 from sympy.utilities.iterables import permutations
-from sympy.utilities.pytest import XFAIL, raises
+from sympy.testing.pytest import XFAIL, raises
 
 from mpmath import mpf
 from mpmath.rational import mpq
 import mpmath
-from sympy import numbers
+from sympy.core import numbers
 t = Symbol('t', real=False)
 
 _ninf = float(-oo)
@@ -277,12 +275,12 @@ def _test_rational_new(cls):
     assert cls(-1) is S.NegativeOne
     # These look odd, but are similar to int():
     assert cls('1') is S.One
-    assert cls(u'-1') is S.NegativeOne
+    assert cls('-1') is S.NegativeOne
 
     i = Integer(10)
     assert _strictly_equal(i, cls('10'))
-    assert _strictly_equal(i, cls(u'10'))
-    assert _strictly_equal(i, cls(long(10)))
+    assert _strictly_equal(i, cls('10'))
+    assert _strictly_equal(i, cls(int(10)))
     assert _strictly_equal(i, cls(i))
 
     raises(TypeError, lambda: cls(Symbol('x')))
@@ -439,7 +437,7 @@ def test_Float():
     mpf = (0, 5404319552844595, -52, 53)
     x_str =  Float((0, '13333333333333', -52, 53))
     x2_str = Float((0, '26666666666666', -53, 54))
-    x_hex = Float((0, long(0x13333333333333), -52, 53))
+    x_hex = Float((0, int(0x13333333333333), -52, 53))
     x_dec = Float(mpf)
     assert x_str == x_hex == x_dec == Float(1.2)
     # x2_str was entered slightly malformed in that the mantissa
@@ -450,13 +448,13 @@ def test_Float():
     assert Float(1.2)._mpf_ == mpf
     assert x2_str._mpf_ == mpf
 
-    assert Float((0, long(0), -123, -1)) is S.NaN
-    assert Float((0, long(0), -456, -2)) is S.Infinity
-    assert Float((1, long(0), -789, -3)) is S.NegativeInfinity
+    assert Float((0, int(0), -123, -1)) is S.NaN
+    assert Float((0, int(0), -456, -2)) is S.Infinity
+    assert Float((1, int(0), -789, -3)) is S.NegativeInfinity
     # if you don't give the full signature, it's not special
-    assert Float((0, long(0), -123)) == Float(0)
-    assert Float((0, long(0), -456)) == Float(0)
-    assert Float((1, long(0), -789)) == Float(0)
+    assert Float((0, int(0), -123)) == Float(0)
+    assert Float((0, int(0), -456)) == Float(0)
+    assert Float((1, int(0), -789)) == Float(0)
 
     raises(ValueError, lambda: Float((0, 7, 1, 3), ''))
 
@@ -546,14 +544,14 @@ def test_Float():
     assert Float(decimal.Decimal('Infinity')) is S.Infinity
     assert Float(decimal.Decimal('-Infinity')) is S.NegativeInfinity
 
-    assert '{0:.3f}'.format(Float(4.236622)) == '4.237'
-    assert '{0:.35f}'.format(Float(pi.n(40), 40)) == \
+    assert '{:.3f}'.format(Float(4.236622)) == '4.237'
+    assert '{:.35f}'.format(Float(pi.n(40), 40)) == \
         '3.14159265358979323846264338327950288'
 
     # unicode
-    assert Float(u'0.73908513321516064100000000') == \
+    assert Float('0.73908513321516064100000000') == \
         Float('0.73908513321516064100000000')
-    assert Float(u'0.73908513321516064100000000', 28) == \
+    assert Float('0.73908513321516064100000000', 28) == \
         Float('0.73908513321516064100000000', 28)
 
     # binary precision
@@ -1248,19 +1246,6 @@ def test_int():
     assert type(int(a)) is type(int(-a)) is int
 
 
-def test_long():
-    a = Rational(5)
-    assert long(a) == 5
-    a = Rational(9, 10)
-    assert long(a) == long(-a) == 0
-    a = Integer(2**100)
-    assert long(a) == a
-    assert long(pi) == 3
-    assert long(E) == 2
-    assert long(GoldenRatio) == 1
-    assert long(TribonacciConstant) == 2
-
-
 def test_real_bug():
     x = Symbol("x")
     assert str(2.0*x*x) in ["(2.0*x)*x", "2.0*x**2", "2.00000000000000*x**2"]
@@ -1631,7 +1616,7 @@ def test_as_content_primitive():
 
 def test_hashing_sympy_integers():
     # Test for issue 5072
-    assert set([Integer(3)]) == set([int(3)])
+    assert {Integer(3)} == {int(3)}
     assert hash(Integer(4)) == hash(int(4))
 
 
@@ -1649,12 +1634,12 @@ def test_mpmath_issues():
     from mpmath.libmp.libmpf import _normalize
     import mpmath.libmp as mlib
     rnd = mlib.round_nearest
-    mpf = (0, long(0), -123, -1, 53, rnd)  # nan
-    assert _normalize(mpf, 53) != (0, long(0), 0, 0)
-    mpf = (0, long(0), -456, -2, 53, rnd)  # +inf
-    assert _normalize(mpf, 53) != (0, long(0), 0, 0)
-    mpf = (1, long(0), -789, -3, 53, rnd)  # -inf
-    assert _normalize(mpf, 53) != (0, long(0), 0, 0)
+    mpf = (0, int(0), -123, -1, 53, rnd)  # nan
+    assert _normalize(mpf, 53) != (0, int(0), 0, 0)
+    mpf = (0, int(0), -456, -2, 53, rnd)  # +inf
+    assert _normalize(mpf, 53) != (0, int(0), 0, 0)
+    mpf = (1, int(0), -789, -3, 53, rnd)  # -inf
+    assert _normalize(mpf, 53) != (0, int(0), 0, 0)
 
     from mpmath.libmp.libmpf import fnan
     assert mlib.mpf_eq(fnan, fnan)
@@ -1663,13 +1648,13 @@ def test_mpmath_issues():
 def test_Catalan_EulerGamma_prec():
     n = GoldenRatio
     f = Float(n.n(), 5)
-    assert f._mpf_ == (0, long(212079), -17, 18)
+    assert f._mpf_ == (0, int(212079), -17, 18)
     assert f._prec == 20
     assert n._as_mpf_val(20) == f._mpf_
 
     n = EulerGamma
     f = Float(n.n(), 5)
-    assert f._mpf_ == (0, long(302627), -19, 19)
+    assert f._mpf_ == (0, int(302627), -19, 19)
     assert f._prec == 20
     assert n._as_mpf_val(20) == f._mpf_
 
@@ -1886,7 +1871,7 @@ def test_tribonacci_constant_rewrite_as_sqrt():
 
 
 def test_comparisons_with_unknown_type():
-    class Foo(object):
+    class Foo:
         """
         Class that is unaware of Basic, and relies on both classes returning
         the NotImplemented singleton for equivalence to evaluate to False.
@@ -1910,7 +1895,7 @@ def test_comparisons_with_unknown_type():
         raises(TypeError, lambda: n >= foo)
         raises(TypeError, lambda: foo <= n)
 
-    class Bar(object):
+    class Bar:
         """
         Class that considers itself equal to any instance of Number except
         infinities and nans, and relies on sympy types returning the
@@ -1965,7 +1950,7 @@ def test_Integer_precision():
     assert sympify(srepr(Float('1.0', precision=15))) == Float('1.0', precision=15)
 
 def test_numpy_to_float():
-    from sympy.utilities.pytest import skip
+    from sympy.testing.pytest import skip
     from sympy.external import import_module
     np = import_module('numpy')
     if not np:
