@@ -1,14 +1,14 @@
 from sympy import (sin, cos, tan, sec, csc, cot, log, exp, atan, asin, acos,
                    Symbol, Integral, integrate, pi, Dummy, Derivative,
-                   diff, I, sqrt, erf, Piecewise, Eq, Ne, symbols, Rational,
+                   diff, I, sqrt, erf, Piecewise, Ne, symbols, Rational,
                    And, Heaviside, S, asinh, acosh, atanh, acoth, expand,
                    Function, jacobi, gegenbauer, chebyshevt, chebyshevu,
                    legendre, hermite, laguerre, assoc_laguerre, uppergamma, li,
-                   Ei, Ci, Si, Chi, Shi, fresnels, fresnelc, polylog, erf, erfi,
+                   Ei, Ci, Si, Chi, Shi, fresnels, fresnelc, polylog, erfi,
                    sinh, cosh, elliptic_f, elliptic_e)
 from sympy.integrals.manualintegrate import (manualintegrate, find_substitutions,
     _parts_rule, integral_steps, contains_dont_know, manual_subs)
-from sympy.utilities.pytest import slow
+from sympy.testing.pytest import raises, slow
 
 x, y, z, u, n, a, b, c = symbols('x y z u n a b c')
 f = Function('f')
@@ -18,7 +18,7 @@ def test_find_substitutions():
         [(cot(x), 1, -u**6 - 2*u**4 - u**2)]
     assert find_substitutions((sec(x)**2 + tan(x) * sec(x)) / (sec(x) + tan(x)),
                               x, u) == [(sec(x) + tan(x), 1, 1/u)]
-    assert find_substitutions(x * exp(-x**2), x, u) == [(-x**2, -S.Half, exp(u))]
+    assert find_substitutions(x * exp(-x**2), x, u) == [(-x**2, Rational(-1, 2), exp(u))]
 
 
 def test_manualintegrate_polynomials():
@@ -108,25 +108,29 @@ def test_manualintegrate_inversetrig():
     assert manualintegrate(1 / (16 + 16 * x**2), x) == atan(x) / 16
     assert manualintegrate(1 / (4 + x**2), x) == atan(x / 2) / 2
     assert manualintegrate(1 / (1 + 4 * x**2), x) == atan(2*x) / 2
-    assert manualintegrate(1/(a + b*x**2), x) == \
-        Piecewise((atan(x/sqrt(a/b))/(b*sqrt(a/b)), a/b > 0), \
-                  (-acoth(x/sqrt(-a/b))/(b*sqrt(-a/b)), And(a/b < 0, x**2 > -a/b)), \
-                  (-atanh(x/sqrt(-a/b))/(b*sqrt(-a/b)), And(a/b < 0, x**2 < -a/b)))
-    assert manualintegrate(1/(4 + b*x**2), x) == \
-        Piecewise((atan(x/(2*sqrt(1/b)))/(2*b*sqrt(1/b)), 4/b > 0), \
-                  (-acoth(x/(2*sqrt(-1/b)))/(2*b*sqrt(-1/b)), And(4/b < 0, x**2 > -4/b)), \
-                  (-atanh(x/(2*sqrt(-1/b)))/(2*b*sqrt(-1/b)), And(4/b < 0, x**2 < -4/b)))
-    assert manualintegrate(1/(a + 4*x**2), x) == \
-        Piecewise((atan(2*x/sqrt(a))/(2*sqrt(a)), a/4 > 0), \
-                  (-acoth(2*x/sqrt(-a))/(2*sqrt(-a)), And(a/4 < 0, x**2 > -a/4)), \
-                  (-atanh(2*x/sqrt(-a))/(2*sqrt(-a)), And(a/4 < 0, x**2 < -a/4)))
+    ra = Symbol('a', real=True)
+    rb = Symbol('b', real=True)
+    assert manualintegrate(1/(ra + rb*x**2), x) == \
+        Piecewise((atan(x/sqrt(ra/rb))/(rb*sqrt(ra/rb)), ra/rb > 0),
+                  (-acoth(x/sqrt(-ra/rb))/(rb*sqrt(-ra/rb)), And(ra/rb < 0, x**2 > -ra/rb)),
+                  (-atanh(x/sqrt(-ra/rb))/(rb*sqrt(-ra/rb)), And(ra/rb < 0, x**2 < -ra/rb)))
+    assert manualintegrate(1/(4 + rb*x**2), x) == \
+        Piecewise((atan(x/(2*sqrt(1/rb)))/(2*rb*sqrt(1/rb)), 4/rb > 0),
+                  (-acoth(x/(2*sqrt(-1/rb)))/(2*rb*sqrt(-1/rb)), And(4/rb < 0, x**2 > -4/rb)),
+                  (-atanh(x/(2*sqrt(-1/rb)))/(2*rb*sqrt(-1/rb)), And(4/rb < 0, x**2 < -4/rb)))
+    assert manualintegrate(1/(ra + 4*x**2), x) == \
+        Piecewise((atan(2*x/sqrt(ra))/(2*sqrt(ra)), ra/4 > 0),
+                  (-acoth(2*x/sqrt(-ra))/(2*sqrt(-ra)), And(ra/4 < 0, x**2 > -ra/4)),
+                  (-atanh(2*x/sqrt(-ra))/(2*sqrt(-ra)), And(ra/4 < 0, x**2 < -ra/4)))
     assert manualintegrate(1/(4 + 4*x**2), x) == atan(x) / 4
 
+    assert manualintegrate(1/(a + b*x**2), x) == atan(x/sqrt(a/b))/(b*sqrt(a/b))
+
     # asin
     assert manualintegrate(1/sqrt(1-x**2), x) == asin(x)
     assert manualintegrate(1/sqrt(4-4*x**2), x) == asin(x)/2
     assert manualintegrate(3/sqrt(1-9*x**2), x) == asin(3*x)
-    assert manualintegrate(1/sqrt(4-9*x**2), x) == asin(3*x/2)/3
+    assert manualintegrate(1/sqrt(4-9*x**2), x) == asin(x*Rational(3, 2))/3
 
     # asinh
     assert manualintegrate(1/sqrt(x**2 + 1), x) == \
@@ -170,7 +174,7 @@ def test_manualintegrate_inversetrig():
 def test_manualintegrate_trig_substitution():
     assert manualintegrate(sqrt(16*x**2 - 9)/x, x) == \
         Piecewise((sqrt(16*x**2 - 9) - 3*acos(3/(4*x)),
-                   And(x < 3*S.One/4, x > -3*S.One/4)))
+                   And(x < Rational(3, 4), x > Rational(-3, 4))))
     assert manualintegrate(1/(x**4 * sqrt(25-x**2)), x) == \
         Piecewise((-sqrt(-x**2/25 + 1)/(125*x) -
                    (-x**2/25 + 1)**(3*S.Half)/(15*x**3), And(x < 5, x > -5)))
@@ -194,7 +198,7 @@ def test_manualintegrate_special():
     assert manualintegrate(f, x) == F and F.diff(x).equals(f)
     f, F = 3*exp(4*x**2), 3*sqrt(pi)*erfi(2*x)/4
     assert manualintegrate(f, x) == F and F.diff(x).equals(f)
-    f, F = x**(S(1)/3)*exp(-x/8), -16*uppergamma(S(4)/3, x/8)
+    f, F = x**Rational(1, 3)*exp(-x/8), -16*uppergamma(Rational(4, 3), x/8)
     assert manualintegrate(f, x) == F and F.diff(x).equals(f)
     f, F = exp(2*x)/x, Ei(2*x)
     assert manualintegrate(f, x) == F and F.diff(x).equals(f)
@@ -220,15 +224,15 @@ def test_manualintegrate_special():
     assert manualintegrate(f, x) == F and F.diff(x).equals(f)
     f, F = polylog(2, 5*x)/x, polylog(3, 5*x)
     assert manualintegrate(f, x) == F and F.diff(x).equals(f)
-    f, F = 5/sqrt(3 - 2*sin(x)**2), 5*sqrt(3)*elliptic_f(x, S(2)/3)/3
+    f, F = 5/sqrt(3 - 2*sin(x)**2), 5*sqrt(3)*elliptic_f(x, Rational(2, 3))/3
     assert manualintegrate(f, x) == F and F.diff(x).equals(f)
-    f, F = sqrt(4 + 9*sin(x)**2), 2*elliptic_e(x, -S(9)/4)
+    f, F = sqrt(4 + 9*sin(x)**2), 2*elliptic_e(x, Rational(-9, 4))
     assert manualintegrate(f, x) == F and F.diff(x).equals(f)
 
 
 def test_manualintegrate_derivative():
     assert manualintegrate(pi * Derivative(x**2 + 2*x + 3), x) == \
-        pi * ((x**2 + 2*x + 3))
+        pi * (x**2 + 2*x + 3)
     assert manualintegrate(Derivative(x**2 + 2*x + 3, y), x) == \
         Integral(Derivative(x**2 + 2*x + 3, y))
     assert manualintegrate(Derivative(sin(x), x, x, x, y), x) == \
@@ -244,19 +248,19 @@ def test_manualintegrate_Heaviside():
     assert manualintegrate(Heaviside(2*x + 4), x) == (x+2)*Heaviside(2*x + 4)
     assert manualintegrate(x*Heaviside(x), x) == x**2*Heaviside(x)/2
     assert manualintegrate(Heaviside(x + 1)*Heaviside(1 - x)*x**2, x) == \
-        ((x**3/3 + S(1)/3)*Heaviside(x + 1) - S(2)/3)*Heaviside(-x + 1)
+        ((x**3/3 + Rational(1, 3))*Heaviside(x + 1) - Rational(2, 3))*Heaviside(-x + 1)
 
     y = Symbol('y')
     assert manualintegrate(sin(7 + x)*Heaviside(3*x - 7), x) == \
-            (- cos(x + 7) + cos(S(28)/3))*Heaviside(3*x - S(7))
+            (- cos(x + 7) + cos(Rational(28, 3)))*Heaviside(3*x - S(7))
 
     assert manualintegrate(sin(y + x)*Heaviside(3*x - y), x) == \
-            (cos(4*y/3) - cos(x + y))*Heaviside(3*x - y)
+            (cos(y*Rational(4, 3)) - cos(x + y))*Heaviside(3*x - y)
 
 
 def test_manualintegrate_orthogonal_poly():
     n = symbols('n')
-    a, b = 7, S(5)/3
+    a, b = 7, Rational(5, 3)
     polys = [jacobi(n, a, b, x), gegenbauer(n, a, x), chebyshevt(n, x),
         chebyshevu(n, x), legendre(n, x), hermite(n, x), laguerre(n, x),
         assoc_laguerre(n, a, x)]
@@ -339,10 +343,7 @@ def test_issue_6746():
         (y + 1)**(n*x)/(n*log(y + 1))
     a = Symbol('a', negative=True)
     b = Symbol('b')
-    assert manualintegrate(1/(a + b*x**2), x) == \
-        Piecewise((atan(x/sqrt(a/b))/(b*sqrt(a/b)), a/b > 0), \
-        (-acoth(x/sqrt(-a/b))/(b*sqrt(-a/b)), And(a/b < 0, x**2 > -a/b)), \
-        (-atanh(x/sqrt(-a/b))/(b*sqrt(-a/b)), And(a/b < 0, x**2 < -a/b)))
+    assert manualintegrate(1/(a + b*x**2), x) == atan(x/sqrt(a/b))/(b*sqrt(a/b))
     b = Symbol('b', negative=True)
     assert manualintegrate(1/(a + b*x**2), x) == \
         atan(x/(sqrt(-a)*sqrt(-1/b)))/(b*sqrt(-a)*sqrt(-1/b))
@@ -364,6 +365,7 @@ def test_issue_2850():
     assert manualintegrate(atan(x)*log(x), x) == -x*atan(x) + (x*atan(x) - \
             log(x**2 + 1)/2)*log(x) + log(x**2 + 1)/2 + Integral(log(x**2 + 1)/x, x)/2
 
+
 def test_issue_9462():
     assert manualintegrate(sin(2*x)*exp(x), x) == exp(x)*sin(2*x)/5 - 2*exp(x)*cos(2*x)/5
     assert not contains_dont_know(integral_steps(sin(2*x)*exp(x), x))
@@ -383,41 +385,60 @@ def test_cyclic_parts():
 
 
 @slow
-def test_issue_10847():
-    assert manualintegrate(x**2 / (x**2 - c), x) == c*Piecewise((atan(x/sqrt(-c))/sqrt(-c), -c > 0), \
-                                                                (-acoth(x/sqrt(c))/sqrt(c), And(-c < 0, x**2 > c)), \
-                                                                (-atanh(x/sqrt(c))/sqrt(c), And(-c < 0, x**2 < c))) + x
-    assert manualintegrate(sqrt(x - y) * log(z / x), x) == 4*y**2*Piecewise((atan(sqrt(x - y)/sqrt(y))/sqrt(y), y > 0), \
-                                                                            (-acoth(sqrt(x - y)/sqrt(-y))/sqrt(-y), \
-                                                                             And(x - y > -y, y < 0)), \
-                                                                            (-atanh(sqrt(x - y)/sqrt(-y))/sqrt(-y), \
-                                                                             And(x - y < -y, y < 0)))/3 \
-                                                                             - 4*y*sqrt(x - y)/3 + 2*(x - y)**(S(3)/2)*log(z/x)/3 \
-                                                                             + 4*(x - y)**(S(3)/2)/9
-
-    assert manualintegrate(sqrt(x) * log(x), x) == 2*x**(S(3)/2)*log(x)/3 - 4*x**(S(3)/2)/9
-    assert manualintegrate(sqrt(a*x + b) / x, x) == -2*b*Piecewise((-atan(sqrt(a*x + b)/sqrt(-b))/sqrt(-b), -b > 0), \
-                                                               (acoth(sqrt(a*x + b)/sqrt(b))/sqrt(b), And(-b < 0, a*x + b > b)), \
-                                                               (atanh(sqrt(a*x + b)/sqrt(b))/sqrt(b), And(-b < 0, a*x + b < b))) \
-                                                               + 2*sqrt(a*x + b)
-
-    assert expand(manualintegrate(sqrt(a*x + b) / (x + c), x)) == -2*a*c*Piecewise((atan(sqrt(a*x + b)/sqrt(a*c - b))/sqrt(a*c - b), \
-        a*c - b > 0), (-acoth(sqrt(a*x + b)/sqrt(-a*c + b))/sqrt(-a*c + b), And(a*c - b < 0, a*x + b > -a*c + b)), \
-        (-atanh(sqrt(a*x + b)/sqrt(-a*c + b))/sqrt(-a*c + b), And(a*c - b < 0, a*x + b < -a*c + b))) \
-        + 2*b*Piecewise((atan(sqrt(a*x + b)/sqrt(a*c - b))/sqrt(a*c - b), a*c - b > 0), \
-        (-acoth(sqrt(a*x + b)/sqrt(-a*c + b))/sqrt(-a*c + b), And(a*c - b < 0, a*x + b > -a*c + b)), \
-        (-atanh(sqrt(a*x + b)/sqrt(-a*c + b))/sqrt(-a*c + b), And(a*c - b < 0, a*x + b < -a*c + b))) + 2*sqrt(a*x + b)
-
-    assert manualintegrate((4*x**4 + 4*x**3 + 16*x**2 + 12*x + 8) \
+def test_issue_10847_slow():
+    assert manualintegrate((4*x**4 + 4*x**3 + 16*x**2 + 12*x + 8)
                            / (x**6 + 2*x**5 + 3*x**4 + 4*x**3 + 3*x**2 + 2*x + 1), x) == \
                            2*x/(x**2 + 1) + 3*atan(x) - 1/(x**2 + 1) - 3/(x + 1)
+
+
+def test_issue_10847():
+
+    assert manualintegrate(x**2 / (x**2 - c), x) == c*atan(x/sqrt(-c))/sqrt(-c) + x
+
+    rc = Symbol('c', real=True)
+    assert manualintegrate(x**2 / (x**2 - rc), x) == \
+        rc*Piecewise((atan(x/sqrt(-rc))/sqrt(-rc), -rc > 0),
+                     (-acoth(x/sqrt(rc))/sqrt(rc), And(-rc < 0, x**2 > rc)),
+                     (-atanh(x/sqrt(rc))/sqrt(rc), And(-rc < 0, x**2 < rc))) + x
+
+    assert manualintegrate(sqrt(x - y) * log(z / x), x) == \
+        4*y**Rational(3, 2)*atan(sqrt(x - y)/sqrt(y))/3 - 4*y*sqrt(x - y)/3 +\
+        2*(x - y)**Rational(3, 2)*log(z/x)/3 + 4*(x - y)**Rational(3, 2)/9
+    ry = Symbol('y', real=True)
+    rz = Symbol('z', real=True)
+    assert manualintegrate(sqrt(x - ry) * log(rz / x), x) == \
+        4*ry**2*Piecewise((atan(sqrt(x - ry)/sqrt(ry))/sqrt(ry), ry > 0),
+                         (-acoth(sqrt(x - ry)/sqrt(-ry))/sqrt(-ry), And(x - ry > -ry, ry < 0)),
+                         (-atanh(sqrt(x - ry)/sqrt(-ry))/sqrt(-ry), And(x - ry < -ry, ry < 0)))/3 \
+                         - 4*ry*sqrt(x - ry)/3 + 2*(x - ry)**Rational(3, 2)*log(rz/x)/3 \
+                         + 4*(x - ry)**Rational(3, 2)/9
+
+    assert manualintegrate(sqrt(x) * log(x), x) == 2*x**Rational(3, 2)*log(x)/3 - 4*x**Rational(3, 2)/9
+    assert manualintegrate(sqrt(a*x + b) / x, x) == \
+        2*b*atan(sqrt(a*x + b)/sqrt(-b))/sqrt(-b) + 2*sqrt(a*x + b)
+    ra = Symbol('a', real=True)
+    rb = Symbol('b', real=True)
+    assert manualintegrate(sqrt(ra*x + rb) / x, x) == \
+        -2*rb*Piecewise((-atan(sqrt(ra*x + rb)/sqrt(-rb))/sqrt(-rb), -rb > 0),
+                        (acoth(sqrt(ra*x + rb)/sqrt(rb))/sqrt(rb), And(-rb < 0, ra*x + rb > rb)),
+                        (atanh(sqrt(ra*x + rb)/sqrt(rb))/sqrt(rb), And(-rb < 0, ra*x + rb < rb))) \
+                        + 2*sqrt(ra*x + rb)
+
+    assert expand(manualintegrate(sqrt(ra*x + rb) / (x + rc), x)) == -2*ra*rc*Piecewise((atan(sqrt(ra*x + rb)/sqrt(ra*rc - rb))/sqrt(ra*rc - rb), \
+        ra*rc - rb > 0), (-acoth(sqrt(ra*x + rb)/sqrt(-ra*rc + rb))/sqrt(-ra*rc + rb), And(ra*rc - rb < 0, ra*x + rb > -ra*rc + rb)), \
+        (-atanh(sqrt(ra*x + rb)/sqrt(-ra*rc + rb))/sqrt(-ra*rc + rb), And(ra*rc - rb < 0, ra*x + rb < -ra*rc + rb))) \
+        + 2*rb*Piecewise((atan(sqrt(ra*x + rb)/sqrt(ra*rc - rb))/sqrt(ra*rc - rb), ra*rc - rb > 0), \
+        (-acoth(sqrt(ra*x + rb)/sqrt(-ra*rc + rb))/sqrt(-ra*rc + rb), And(ra*rc - rb < 0, ra*x + rb > -ra*rc + rb)), \
+        (-atanh(sqrt(ra*x + rb)/sqrt(-ra*rc + rb))/sqrt(-ra*rc + rb), And(ra*rc - rb < 0, ra*x + rb < -ra*rc + rb))) + 2*sqrt(ra*x + rb)
+
     assert manualintegrate(sqrt(2*x + 3) / (x + 1), x) == 2*sqrt(2*x + 3) - log(sqrt(2*x + 3) + 1) + log(sqrt(2*x + 3) - 1)
-    assert manualintegrate(sqrt(2*x + 3) / 2 * x, x) == (2*x + 3)**(S(5)/2)/20 - (2*x + 3)**(S(3)/2)/4
+    assert manualintegrate(sqrt(2*x + 3) / 2 * x, x) == (2*x + 3)**Rational(5, 2)/20 - (2*x + 3)**Rational(3, 2)/4
     assert manualintegrate(x**Rational(3,2) * log(x), x) == 2*x**Rational(5,2)*log(x)/5 - 4*x**Rational(5,2)/25
     assert manualintegrate(x**(-3) * log(x), x) == -log(x)/(2*x**2) - 1/(4*x**2)
     assert manualintegrate(log(y)/(y**2*(1 - 1/y)), y) == \
         log(y)*log(-1 + 1/y) - Integral(log(-1 + 1/y)/y, y)
 
+
 def test_issue_12899():
     assert manualintegrate(f(x,y).diff(x),y) == Integral(Derivative(f(x,y),x),y)
     assert manualintegrate(f(x,y).diff(y).diff(x),y) == Derivative(f(x,y),x)
@@ -438,6 +459,7 @@ def test_issue_12641():
 def test_issue_13297():
     assert manualintegrate(sin(x) * cos(x)**5, x) == -cos(x)**6 / 6
 
+
 def test_issue_14470():
     assert manualintegrate(1/(x*sqrt(x + 1)), x) == \
         log(-1 + 1/sqrt(x + 1)) - log(1 + 1/sqrt(x + 1))
@@ -473,8 +495,17 @@ def test_manual_subs():
     # if exp(x) is y, then log(y) need not be x
     assert manual_subs(expr, exp(x), y) == log(x) + y
 
+    raises(ValueError, lambda: manual_subs(expr, x))
+    raises(ValueError, lambda: manual_subs(expr, exp(x), x, y))
+
 
 def test_issue_15471():
-    f = log(x)*cos(log(x))/x**(S(3)/4)
-    F = -128*x**(S(1)/4)*sin(log(x))/289 + 240*x**(S(1)/4)*cos(log(x))/289 + (16*x**(S(1)/4)*sin(log(x))/17 + 4*x**(S(1)/4)*cos(log(x))/17)*log(x)
+    f = log(x)*cos(log(x))/x**Rational(3, 4)
+    F = -128*x**Rational(1, 4)*sin(log(x))/289 + 240*x**Rational(1, 4)*cos(log(x))/289 + (16*x**Rational(1, 4)*sin(log(x))/17 + 4*x**Rational(1, 4)*cos(log(x))/17)*log(x)
     assert manualintegrate(f, x) == F and F.diff(x).equals(f)
+
+def test_quadratic_denom():
+    f = (5*x + 2)/(3*x**2 - 2*x + 8)
+    assert manualintegrate(f, x) == 5*log(3*x**2 - 2*x + 8)/6 + 11*sqrt(23)*atan(3*sqrt(23)*(x - Rational(1, 3))/23)/69
+    g = 3/(2*x**2 + 3*x + 1)
+    assert manualintegrate(g, x) == 3*log(4*x + 2) - 3*log(4*x + 4)
