@@ -232,8 +232,8 @@ def _berkowitz_toeplitz_matrix(M):
     # compute -R * A**n * C.
     diags = [C]
     for i in range(M.rows - 2):
-        diags.append(A.multiply(diags[i], dotprodsimp=True))
-    diags = [(-R).multiply(d, dotprodsimp=True)[0, 0] for d in diags]
+        diags.append(A.multiply(diags[i], dotprodsimp=None))
+    diags = [(-R).multiply(d, dotprodsimp=None)[0, 0] for d in diags]
     diags = [M.one, -a] + diags
 
     def entry(i,j):
@@ -289,7 +289,7 @@ def _berkowitz_vector(M):
 
     submat, toeplitz = _berkowitz_toeplitz_matrix(M)
 
-    return toeplitz.multiply(_berkowitz_vector(submat), dotprodsimp=True)
+    return toeplitz.multiply(_berkowitz_vector(submat), dotprodsimp=None)
 
 
 def _adjugate(M, method="berkowitz"):
