@@ -1,18 +1,21 @@
-from __future__ import print_function, division
-
 from sympy import Number
-from sympy.core import Mul, Basic, sympify, Add
-from sympy.core.compatibility import range
+from sympy.core import Mul, Basic, sympify, S
 from sympy.functions import adjoint
-from sympy.matrices.expressions.transpose import transpose
 from sympy.strategies import (rm_id, unpack, typed, flatten, exhaust,
         do_one, new)
-from sympy.matrices.expressions.matexpr import (MatrixExpr, ShapeError,
-        Identity, ZeroMatrix)
+from sympy.matrices.common import ShapeError, NonInvertibleMatrixError
 from sympy.matrices.matrices import MatrixBase
 
+from .inverse import Inverse
+from .matexpr import \
+    MatrixExpr, Identity, ZeroMatrix, OneMatrix, GenericIdentity
+from .matpow import MatPow
+from .transpose import transpose
+from .permutation import PermutationMatrix
 
-class MatMul(MatrixExpr):
+
+# XXX: MatMul should perhaps not subclass directly from Mul
+class MatMul(MatrixExpr, Mul):
     """
     A product of matrix expressions
 
@@ -28,16 +31,33 @@ class MatMul(MatrixExpr):
     """
     is_MatMul = True
 
-    def __new__(cls, *args, **kwargs):
+    identity = GenericIdentity()
+
+    def __new__(cls, *args, evaluate=False, **kwargs):
         check = kwargs.get('check', True)
 
+        if not args:
+            return cls.identity
+
+        # This must be removed aggressively in the constructor to avoid
+        # TypeErrors from GenericIdentity().shape
+        args = filter(lambda i: cls.identity != i, args)
         args = list(map(sympify, args))
         obj = Basic.__new__(cls, *args)
         factor, matrices = obj.as_coeff_matrices()
+
         if check:
             validate(*matrices)
+
         if not matrices:
+            # Should it be
+            #
+            # return Basic.__neq__(cls, factor, GenericIdentity()) ?
             return factor
+
+        if evaluate:
+            return canonicalize(obj)
+
         return obj
 
     @property
@@ -45,37 +65,52 @@ class MatMul(MatrixExpr):
         matrices = [arg for arg in self.args if arg.is_Matrix]
         return (matrices[0].rows, matrices[-1].cols)
 
-    def _entry(self, i, j, expand=True):
+    def _entry(self, i, j, expand=True, **kwargs):
+        from sympy import Dummy, Sum, Mul, ImmutableMatrix, Integer
+
         coeff, matrices = self.as_coeff_matrices()
 
         if len(matrices) == 1:  # situation like 2*X, matmul is just X
             return coeff * matrices[0][i, j]
 
-        head, tail = matrices[0], matrices[1:]
-        if len(tail) == 0:
-            raise ValueError("lenth of tail cannot be 0")
-        X = head
-        Y = MatMul(*tail)
+        indices = [None]*(len(matrices) + 1)
+        ind_ranges = [None]*(len(matrices) - 1)
+        indices[0] = i
+        indices[-1] = j
 
-        from sympy.core.symbol import Dummy
-        from sympy.concrete.summations import Sum
-        from sympy.matrices import ImmutableMatrix
-        k = Dummy('k', integer=True)
-        if X.has(ImmutableMatrix) or Y.has(ImmutableMatrix):
-            return coeff*Add(*[X[i, k]*Y[k, j] for k in range(X.cols)])
-        result = Sum(coeff*X[i, k]*Y[k, j], (k, 0, X.cols - 1))
-        try:
-            if not X.cols.is_number:
-                # Don't waste time in result.doit() if the sum bounds are symbolic
-                expand = False
-        except AttributeError:
-            pass
+        def f():
+            counter = 1
+            while True:
+                yield Dummy("i_%i" % counter)
+                counter += 1
+
+        dummy_generator = kwargs.get("dummy_generator", f())
+
+        for i in range(1, len(matrices)):
+            indices[i] = next(dummy_generator)
+
+        for i, arg in enumerate(matrices[:-1]):
+            ind_ranges[i] = arg.shape[1] - 1
+        matrices = [arg._entry(indices[i], indices[i+1], dummy_generator=dummy_generator) for i, arg in enumerate(matrices)]
+        expr_in_sum = Mul.fromiter(matrices)
+        if any(v.has(ImmutableMatrix) for v in matrices):
+            expand = True
+        result = coeff*Sum(
+                expr_in_sum,
+                *zip(indices[1:-1], [0]*len(ind_ranges), ind_ranges)
+            )
+
+        # Don't waste time in result.doit() if the sum bounds are symbolic
+        if not any(isinstance(v, (Integer, int)) for v in ind_ranges):
+            expand = False
         return result.doit() if expand else result
 
     def as_coeff_matrices(self):
         scalars = [x for x in self.args if not x.is_Matrix]
         matrices = [x for x in self.args if x.is_Matrix]
         coeff = Mul(*scalars)
+        if coeff.is_commutative is False:
+            raise NotImplementedError("noncommutative scalars in MatMul are not supported.")
 
         return coeff, matrices
 
@@ -84,7 +119,27 @@ class MatMul(MatrixExpr):
         return coeff, MatMul(*matrices)
 
     def _eval_transpose(self):
-        return MatMul(*[transpose(arg) for arg in self.args[::-1]]).doit()
+        """Transposition of matrix multiplication.
+
+        Notes
+        =====
+
+        The following rules are applied.
+
+        Transposition for matrix multiplied with another matrix:
+        `\\left(A B\\right)^{T} = B^{T} A^{T}`
+
+        Transposition for matrix multiplied with scalar:
+        `\\left(c A\\right)^{T} = c A^{T}`
+
+        References
+        ==========
+
+        .. [1] https://en.wikipedia.org/wiki/Transpose
+        """
+        coeff, matrices = self.as_coeff_matrices()
+        return MatMul(
+            coeff, *[transpose(arg) for arg in matrices[::-1]]).doit()
 
     def _eval_adjoint(self):
         return MatMul(*[adjoint(arg) for arg in self.args[::-1]]).doit()
@@ -109,7 +164,6 @@ class MatMul(MatrixExpr):
                 arg.inverse() if isinstance(arg, MatrixExpr) else arg**-1
                     for arg in self.args[::-1]]).doit()
         except ShapeError:
-            from sympy.matrices.expressions.inverse import Inverse
             return Inverse(self)
 
     def doit(self, **kwargs):
@@ -118,16 +172,41 @@ class MatMul(MatrixExpr):
             args = [arg.doit(**kwargs) for arg in self.args]
         else:
             args = self.args
-        return canonicalize(MatMul(*args))
+        # treat scalar*MatrixSymbol or scalar*MatPow separately
+        expr = canonicalize(MatMul(*args))
+        return expr
 
     # Needed for partial compatibility with Mul
     def args_cnc(self, **kwargs):
-        coeff, matrices = self.as_coeff_matrices()
-        # I don't know how coeff could have noncommutative factors, but this
-        # handles it.
-        coeff_c, coeff_nc = coeff.args_cnc(**kwargs)
+        coeff_c = [x for x in self.args if x.is_commutative]
+        coeff_nc = [x for x in self.args if not x.is_commutative]
+        return [coeff_c, coeff_nc]
+
+    def _eval_derivative_matrix_lines(self, x):
+        from .transpose import Transpose
+        with_x_ind = [i for i, arg in enumerate(self.args) if arg.has(x)]
+        lines = []
+        for ind in with_x_ind:
+            left_args = self.args[:ind]
+            right_args = self.args[ind+1:]
+
+            if right_args:
+                right_mat = MatMul.fromiter(right_args)
+            else:
+                right_mat = Identity(self.shape[1])
+            if left_args:
+                left_rev = MatMul.fromiter([Transpose(i).doit() if i.is_Matrix else i for i in reversed(left_args)])
+            else:
+                left_rev = Identity(self.shape[0])
+
+            d = self.args[ind]._eval_derivative_matrix_lines(x)
+            for i in d:
+                i.append_first(left_rev)
+                i.append_second(right_mat)
+                lines.append(i)
+
+        return lines
 
-        return coeff_c, coeff_nc + matrices
 
 def validate(*matrices):
     """ Checks for valid shapes for args of MatMul """
@@ -193,19 +272,6 @@ def merge_explicit(matmul):
 
     return MatMul(*newargs)
 
-def xxinv(mul):
-    """ Y * X * X.I -> Y """
-    factor, matrices = mul.as_coeff_matrices()
-    for i, (X, Y) in enumerate(zip(matrices[:-1], matrices[1:])):
-        try:
-            if X.is_square and Y.is_square and X == Y.inverse():
-                I = Identity(X.rows)
-                return newmul(factor, *(matrices[:i] + [I] + matrices[i+2:]))
-        except ValueError:  # Y might not be invertible
-            pass
-
-    return mul
-
 def remove_ids(mul):
     """ Remove Identities from a MatMul
 
@@ -214,8 +280,9 @@ def remove_ids(mul):
     as args.
 
     See Also
-    --------
-        sympy.strategies.rm_id
+    ========
+
+    sympy.strategies.rm_id
     """
     # Separate Exprs from MatrixExprs in args
     factor, mmul = mul.as_coeff_mmul()
@@ -232,13 +299,102 @@ def factor_in_front(mul):
         return newmul(factor, *matrices)
     return mul
 
-rules = (any_zeros, remove_ids, xxinv, unpack, rm_id(lambda x: x == 1),
-         merge_explicit, factor_in_front, flatten)
+def combine_powers(mul):
+    """Combine consecutive powers with the same base into one
+
+    e.g. A*A**2 -> A**3
+
+    This also cancels out the possible matrix inverses using the
+    knowledgebase of ``Inverse``.
+
+    e.g. Y * X * X.I -> Y
+    """
+    factor, args = mul.as_coeff_matrices()
+    new_args = [args[0]]
+
+    for B in args[1:]:
+        A = new_args[-1]
+        if A.is_square == False or B.is_square == False:
+            new_args.append(B)
+            continue
+
+        if isinstance(A, MatPow):
+            A_base, A_exp = A.args
+        else:
+            A_base, A_exp = A, S.One
+
+        if isinstance(B, MatPow):
+            B_base, B_exp = B.args
+        else:
+            B_base, B_exp = B, S.One
+
+        if A_base == B_base:
+            new_exp = A_exp + B_exp
+            new_args[-1] = MatPow(A_base, new_exp).doit(deep=False)
+            continue
+        elif not isinstance(B_base, MatrixBase):
+            try:
+                B_base_inv = B_base.inverse()
+            except NonInvertibleMatrixError:
+                B_base_inv = None
+            if B_base_inv is not None and A_base == B_base_inv:
+                new_exp = A_exp - B_exp
+                new_args[-1] = MatPow(A_base, new_exp).doit(deep=False)
+                continue
+        new_args.append(B)
+
+    return newmul(factor, *new_args)
+
+def combine_permutations(mul):
+    """Refine products of permutation matrices as the products of cycles.
+    """
+    args = mul.args
+    l = len(args)
+    if l < 2:
+        return mul
+
+    result = [args[0]]
+    for i in range(1, l):
+        A = result[-1]
+        B = args[i]
+        if isinstance(A, PermutationMatrix) and \
+            isinstance(B, PermutationMatrix):
+            cycle_1 = A.args[0]
+            cycle_2 = B.args[0]
+            result[-1] = PermutationMatrix(cycle_1 * cycle_2)
+        else:
+            result.append(B)
+
+    return MatMul(*result)
+
+def combine_one_matrices(mul):
+    """
+    Combine products of OneMatrix
+
+    e.g. OneMatrix(2, 3) * OneMatrix(3, 4) -> 3 * OneMatrix(2, 4)
+    """
+    factor, args = mul.as_coeff_matrices()
+    new_args = [args[0]]
+
+    for B in args[1:]:
+        A = new_args[-1]
+        if not isinstance(A, OneMatrix) or not isinstance(B, OneMatrix):
+            new_args.append(B)
+            continue
+        new_args.pop()
+        new_args.append(OneMatrix(A.shape[0], B.shape[1]))
+        factor *= A.shape[1]
+
+    return newmul(factor, *new_args)
+
+rules = (
+    any_zeros, remove_ids, combine_one_matrices, combine_powers, unpack, rm_id(lambda x: x == 1),
+    merge_explicit, factor_in_front, flatten, combine_permutations)
 
 canonicalize = exhaust(typed({MatMul: do_one(*rules)}))
 
 def only_squares(*matrices):
-    """ factor matrices only if they are square """
+    """factor matrices only if they are square"""
     if matrices[0].rows != matrices[-1].cols:
         raise RuntimeError("Invalid matrices being multiplied")
     out = []
