@@ -2,12 +2,11 @@
 
 from __future__ import print_function, division
 
+from sympy.core.symbol import Dummy
 from sympy.polys.monomials import monomial_mul, monomial_lcm, monomial_divides, term_div
 from sympy.polys.orderings import lex
 from sympy.polys.polyerrors import DomainError
 from sympy.polys.polyconfig import query
-from sympy.core.symbol import Dummy
-from sympy.core.compatibility import range
 
 def groebner(seq, ring, method=None):
     """
@@ -15,8 +14,8 @@ def groebner(seq, ring, method=None):
 
     Wrapper around the (default) improved Buchberger and the other algorithms
     for computing Groebner bases. The choice of algorithm can be changed via
-    ``method`` argument or :func:`setup` from :mod:`sympy.polys.polyconfig`,
-    where ``method`` can be either ``buchberger`` or ``f5b``.
+    ``method`` argument or :func:`sympy.polys.polyconfig.setup`, where
+    ``method`` can be either ``buchberger`` or ``f5b``.
 
     """
     if method is None:
@@ -34,7 +33,7 @@ def groebner(seq, ring, method=None):
 
     domain, orig = ring.domain, None
 
-    if not domain.has_Field or not domain.has_assoc_Field:
+    if not domain.is_Field or not domain.has_assoc_Field:
         try:
             orig, ring = ring, ring.clone(domain=domain.get_field())
         except DomainError:
@@ -73,22 +72,24 @@ def _buchberger(f, ring):
     variable at a time, provided that the ideal is zero-dimensional
     (finite number of solutions).
 
-    References
-    ==========
-
-    1. [Bose03]_
-    2. [Giovini91]_
-    3. [Ajwa95]_
-    4. [Cox97]_
+    Notes
+    =====
 
     Algorithm used: an improved version of Buchberger's algorithm
     as presented in T. Becker, V. Weispfenning, Groebner Bases: A
     Computational Approach to Commutative Algebra, Springer, 1993,
     page 232.
 
+    References
+    ==========
+
+    .. [1] [Bose03]_
+    .. [2] [Giovini91]_
+    .. [3] [Ajwa95]_
+    .. [4] [Cox97]_
+
     """
     order = ring.order
-    domain = ring.domain
 
     monomial_mul = ring.monomial_mul
     monomial_div = ring.monomial_div
@@ -521,7 +522,7 @@ def f5_reduce(f, B):
     """
     F5-reduce a labeled polynomial f by B.
 
-    Continously searches for non-zero labeled polynomial h in B, such
+    Continuously searches for non-zero labeled polynomial h in B, such
     that the leading term lt_h of h divides the leading term lt_f of
     f and Sign(lt_h * h) < Sign(f). If such a labeled polynomial h is
     found, f gets replaced by f - lt_f / lt_h * h. If no such h can be
@@ -592,16 +593,17 @@ def _f5b(F, ring):
 
     Once a Groebner basis has been found, it gets reduced.
 
-    ** References **
-    Yao Sun, Dingkang Wang: "A New Proof for the Correctness of F5
-    (F5-Like) Algorithm", http://arxiv.org/abs/1004.0084 (specifically
-    v4)
+    References
+    ==========
 
-    Thomas Becker, Volker Weispfenning, Groebner bases: A computational
-    approach to commutative algebra, 1993, p. 203, 216
+    .. [1] Yao Sun, Dingkang Wang: "A New Proof for the Correctness of F5
+           (F5-Like) Algorithm", http://arxiv.org/abs/1004.0084 (specifically
+           v4)
+
+    .. [2] Thomas Becker, Volker Weispfenning, Groebner bases: A computational
+           approach to commutative algebra, 1993, p. 203, 216
     """
     order = ring.order
-    domain = ring.domain
 
     # reduce polynomials (like in Mario Pernici's implementation) (Becker, Weispfenning, p. 203)
     B = F
@@ -733,7 +735,7 @@ def is_groebner(G, ring):
     """
     for i in range(len(G)):
         for j in range(i + 1, len(G)):
-            s = spoly(G[i], G[j])
+            s = spoly(G[i], G[j], ring)
             s = s.rem(G)
             if s:
                 return False
@@ -774,7 +776,7 @@ def is_reduced(G, ring):
         if g.LC != domain.one:
             return False
 
-        for term in g:
+        for term in g.terms():
             for h in G[:i] + G[i + 1:]:
                 if monomial_divides(h.LM, term[0]):
                     return False
@@ -785,7 +787,7 @@ def groebner_lcm(f, g):
     """
     Computes LCM of two polynomials using Groebner bases.
 
-    The LCM is computed as the unique generater of the intersection
+    The LCM is computed as the unique generator of the intersection
     of the two ideals generated by `f` and `g`. The approach is to
     compute a Groebner basis with respect to lexicographic ordering
     of `t*f` and `(1 - t)*g`, where `t` is an unrelated variable and
@@ -794,7 +796,7 @@ def groebner_lcm(f, g):
     References
     ==========
 
-    1. [Cox97]_
+    .. [1] [Cox97]_
 
     """
     if f.ring != g.ring:
@@ -844,7 +846,7 @@ def groebner_gcd(f, g):
         raise ValueError("Values should be equal")
     domain = f.ring.domain
 
-    if not domain.has_Field:
+    if not domain.is_Field:
         fc, f = f.primitive()
         gc, g = g.primitive()
         gcd = domain.gcd(fc, gc)
@@ -855,7 +857,7 @@ def groebner_gcd(f, g):
         raise ValueError("Length should be 1")
     h = H[0]
 
-    if not domain.has_Field:
+    if not domain.is_Field:
         return gcd*h
     else:
         return h.monic()
