@@ -6,30 +6,40 @@ from itertools import combinations_with_replacement, product
 from textwrap import dedent
 
 from sympy.core import Mul, S, Tuple, sympify
-from sympy.core.compatibility import exec_, iterable, range
-from sympy.polys.polyutils import PicklableWithSlots, dict_from_expr
+from sympy.core.compatibility import exec_, iterable
 from sympy.polys.polyerrors import ExactQuotientFailed
+from sympy.polys.polyutils import PicklableWithSlots, dict_from_expr
 from sympy.utilities import public
+from sympy.core.compatibility import is_sequence
 
 @public
-def itermonomials(variables, degree):
+def itermonomials(variables, max_degrees, min_degrees=None):
     r"""
-    Generate a set of monomials of the given total degree or less.
+    `max_degrees` and `min_degrees` are either both integers or both lists.
+    Unless otherwise specified, `min_degrees` is either 0 or [0,...,0].
 
-    Given a set of variables `V` and a total degree `N` generate
-    a set of monomials of degree at most `N`. The total number of
-    monomials in commutative variables is huge and is given by the
-    following formula:
+    A generator of all monomials `monom` is returned, such that
+    either
+    min_degree <= total_degree(monom) <= max_degree,
+    or
+    min_degrees[i] <= degree_list(monom)[i] <= max_degrees[i], for all i.
 
-    .. math::
+    Case I:: `max_degrees` and `min_degrees` are both integers.
+    ===========================================================
+    Given a set of variables `V` and a min_degree `N` and a max_degree `M`
+    generate a set of monomials of degree less than or equal to `N` and greater
+    than or equal to `M`. The total number of monomials in commutative
+    variables is huge and is given by the following formula if `M = 0`:
 
-        \frac{(\#V + N)!}{\#V! N!}
+        .. math::
+
+            \frac{(\#V + N)!}{\#V! N!}
 
     For example if we would like to generate a dense polynomial of
-    a total degree `N = 50` in 5 variables, assuming that exponents
-    and all of coefficients are 32-bit long and stored in an array we
-    would need almost 80 GiB of memory! Fortunately most polynomials,
-    that we will encounter, are sparse.
+    a total degree `N = 50` and `M = 0`, which is the worst case, in 5
+    variables, assuming that exponents and all of coefficients are 32-bit long
+    and stored in an array we would need almost 80 GiB of memory! Fortunately
+    most polynomials, that we will encounter, are sparse.
 
     Examples
     ========
@@ -49,21 +59,103 @@ def itermonomials(variables, degree):
         [1, x, y, x**2, x*y, y**2, x**3, x**2*y, x*y**2, y**3]
 
         >>> a, b = symbols('a, b', commutative=False)
-        >>> itermonomials([a, b, x], 2)
+        >>> set(itermonomials([a, b, x], 2))
         {1, a, a**2, b, b**2, x, x**2, a*b, b*a, x*a, x*b}
 
+        >>> sorted(itermonomials([x, y], 2, 1), key=monomial_key('grlex', [y, x]))
+        [x, y, x**2, x*y, y**2]
 
+
+    Case II:: `max_degrees` and `min_degrees` are both lists.
+    =========================================================
+    If max_degrees = [d_1, ..., d_n] and min_degrees = [e_1, ..., e_n],
+    the number of monomials generated is:
+
+        (d_1 - e_1 + 1) * ... * (d_n - e_n + 1)
+
+    Example
+    =======
+
+    Let us generate all monomials `monom` in variables `x`, and `y`
+    such that [1, 2][i] <= degree_list(monom)[i] <= [2, 4][i], i = 0, 1 ::
+
+        >>> from sympy import symbols
+        >>> from sympy.polys.monomials import itermonomials
+        >>> from sympy.polys.orderings import monomial_key
+        >>> from itertools import product
+        >>> from sympy.core import Mul
+        >>> from sympy.abc import x, y
+
+        >>> sorted(itermonomials([x, y], [2, 4], [1, 2]), reverse=True, key=monomial_key('lex', [x, y]))
+        [x**2*y**4, x**2*y**3, x**2*y**2, x*y**4, x*y**3, x*y**2]
     """
-    if degree < 0:
-        return set()
-    if not variables or degree == 0:
-        return {S(1)}
-    # Force to list in case of passed tuple or other incompatible collection
-    variables = list(variables) + [S(1)]
-    if all(variable.is_commutative for variable in variables):
-        return {Mul(*item) for item in combinations_with_replacement(variables, degree)}
+    n = len(variables)
+    if is_sequence(max_degrees):
+        if len(max_degrees) != n:
+            raise ValueError('Argument sizes do not match')
+        if min_degrees is None:
+            min_degrees = [0]*n
+        elif not is_sequence(min_degrees):
+            raise ValueError('min_degrees is not a list')
+        else:
+            if len(min_degrees) != n:
+                raise ValueError('Argument sizes do not match')
+            if any(i < 0 for i in min_degrees):
+                raise ValueError("min_degrees can't contain negative numbers")
+        total_degree = False
     else:
-        return {Mul(*item) for item in product(variables, repeat=degree)}
+        max_degree = max_degrees
+        if max_degree < 0:
+            raise ValueError("max_degrees can't be negative")
+        if min_degrees is None:
+            min_degree = 0
+        else:
+            if min_degrees < 0:
+                raise ValueError("min_degrees can't be negative")
+            min_degree = min_degrees
+        total_degree = True
+    if total_degree:
+        if min_degree > max_degree:
+            return
+        if not variables or max_degree == 0:
+            yield S.One
+            return
+        # Force to list in case of passed tuple or other incompatible collection
+        variables = list(variables) + [S.One]
+        if all(variable.is_commutative for variable in variables):
+            monomials_list_comm = []
+            for item in combinations_with_replacement(variables, max_degree):
+                powers = dict()
+                for variable in variables:
+                    powers[variable] = 0
+                for variable in item:
+                    if variable != 1:
+                        powers[variable] += 1
+                if max(powers.values()) >= min_degree:
+                    monomials_list_comm.append(Mul(*item))
+            for mon in set(monomials_list_comm):
+                yield mon
+        else:
+            monomials_list_non_comm = []
+            for item in product(variables, repeat=max_degree):
+                powers = dict()
+                for variable in variables:
+                    powers[variable] = 0
+                for variable in item:
+                    if variable != 1:
+                        powers[variable] += 1
+                if max(powers.values()) >= min_degree:
+                    monomials_list_non_comm.append(Mul(*item))
+            for mon in set(monomials_list_non_comm):
+                yield mon
+    else:
+        if any(min_degrees[i] > max_degrees[i] for i in range(n)):
+            raise ValueError('min_degrees[i] must be <= max_degrees[i] for all i')
+        power_lists = []
+        for var, min_d, max_d in zip(variables, min_degrees, max_degrees):
+            power_lists.append([var**i for i in range(min_d, max_d + 1)])
+        for powers in product(*power_lists):
+            yield Mul(*powers)
 
 def monomial_count(V, N):
     r"""
@@ -87,7 +179,7 @@ def monomial_count(V, N):
     >>> monomial_count(2, 2)
     6
 
-    >>> M = itermonomials([x, y], 2)
+    >>> M = list(itermonomials([x, y], 2))
 
     >>> sorted(M, key=monomial_key('grlex', [y, x]))
     [1, x, y, x**2, x*y, y**2]
@@ -102,6 +194,9 @@ def monomial_mul(A, B):
     """
     Multiplication of tuples representing monomials.
 
+    Examples
+    ========
+
     Lets multiply `x**3*y**4*z` with `x*y**2`::
 
         >>> from sympy.polys.monomials import monomial_mul
@@ -118,6 +213,9 @@ def monomial_div(A, B):
     """
     Division of tuples representing monomials.
 
+    Examples
+    ========
+
     Lets divide `x**3*y**4*z` by `x*y**2`::
 
         >>> from sympy.polys.monomials import monomial_div
@@ -144,6 +242,9 @@ def monomial_ldiv(A, B):
     """
     Division of tuples representing monomials.
 
+    Examples
+    ========
+
     Lets divide `x**3*y**4*z` by `x*y**2`::
 
         >>> from sympy.polys.monomials import monomial_ldiv
@@ -169,6 +270,9 @@ def monomial_gcd(A, B):
     """
     Greatest common divisor of tuples representing monomials.
 
+    Examples
+    ========
+
     Lets compute GCD of `x*y**4*z` and `x**3*y**2`::
 
         >>> from sympy.polys.monomials import monomial_gcd
@@ -185,6 +289,9 @@ def monomial_lcm(A, B):
     """
     Least common multiple of tuples representing monomials.
 
+    Examples
+    ========
+
     Lets compute LCM of `x*y**4*z` and `x**3*y**2`::
 
         >>> from sympy.polys.monomials import monomial_lcm
@@ -201,6 +308,9 @@ def monomial_divides(A, B):
     """
     Does there exist a monomial X such that XA == B?
 
+    Examples
+    ========
+
     >>> from sympy.polys.monomials import monomial_divides
     >>> monomial_divides((1, 2), (3, 4))
     True
@@ -213,6 +323,9 @@ def monomial_max(*monoms):
     """
     Returns maximal degree for each variable in a set of monomials.
 
+    Examples
+    ========
+
     Consider monomials `x**3*y**4*z**5`, `y**5*z` and `x**6*y**3*z**9`.
     We wish to find out what is the maximal degree for each of `x`, `y`
     and `z` variables::
@@ -235,6 +348,9 @@ def monomial_min(*monoms):
     """
     Returns minimal degree for each variable in a set of monomials.
 
+    Examples
+    ========
+
     Consider monomials `x**3*y**4*z**5`, `y**5*z` and `x**6*y**3*z**9`.
     We wish to find out what is the minimal degree for each of `x`, `y`
     and `z` variables::
@@ -257,7 +373,10 @@ def monomial_deg(M):
     """
     Returns the total degree of a monomial.
 
-    For example, the total degree of `xy^2` is 3:
+    Examples
+    ========
+
+    The total degree of `xy^2` is 3:
 
     >>> from sympy.polys.monomials import monomial_deg
     >>> monomial_deg((1, 2))
@@ -399,7 +518,7 @@ class MonomialOps(object):
 class Monomial(PicklableWithSlots):
     """Class representing a monomial, i.e. a product of powers. """
 
-    __slots__ = ['exponents', 'gens']
+    __slots__ = ('exponents', 'gens')
 
     def __init__(self, monom, gens=None):
         if not iterable(monom):
@@ -407,7 +526,7 @@ class Monomial(PicklableWithSlots):
             if len(rep) == 1 and list(rep.values())[0] == 1:
                 monom = list(rep.keys())[0]
             else:
-                raise ValueError("Expected a monomial got %s" % monom)
+                raise ValueError("Expected a monomial got {}".format(monom))
 
         self.exponents = tuple(map(int, monom))
         self.gens = gens
@@ -462,7 +581,7 @@ class Monomial(PicklableWithSlots):
         elif isinstance(other, (tuple, Tuple)):
             exponents = other
         else:
-            return NotImplementedError
+            raise NotImplementedError
 
         return self.rebuild(monomial_mul(self.exponents, exponents))
 
@@ -472,7 +591,7 @@ class Monomial(PicklableWithSlots):
         elif isinstance(other, (tuple, Tuple)):
             exponents = other
         else:
-            return NotImplementedError
+            raise NotImplementedError
 
         result = monomial_div(self.exponents, exponents)
 
