@@ -1,11 +1,12 @@
 """Tests for computational algebraic number field theory. """
 
 from sympy import (S, Rational, Symbol, Poly, sqrt, I, oo, Tuple, expand,
-    pi, cos, sin, exp)
-
-from sympy.utilities.pytest import raises, slow
-from sympy.core.compatibility import range
-
+    pi, cos, sin, exp, GoldenRatio, TribonacciConstant, cbrt)
+from sympy.solvers.solveset import nonlinsolve
+from sympy.geometry import Circle, intersection
+from sympy.testing.pytest import raises, slow
+from sympy.sets.sets import FiniteSet
+from sympy import Point2D
 from sympy.polys.numberfields import (
     minimal_polynomial,
     primitive_element,
@@ -80,8 +81,8 @@ def test_minimal_polynomial():
     assert minimal_polynomial(sqrt(2), x) == x**2 - 2
 
     assert minimal_polynomial(sqrt(2), polys=True) == Poly(x**2 - 2)
-    assert minimal_polynomial(sqrt(2), x, polys=True) == Poly(x**2 - 2)
-    assert minimal_polynomial(sqrt(2), x, polys=True, compose=False) == Poly(x**2 - 2)
+    assert minimal_polynomial(sqrt(2), x, polys=True) == Poly(x**2 - 2, domain='QQ')
+    assert minimal_polynomial(sqrt(2), x, polys=True, compose=False) == Poly(x**2 - 2, domain='QQ')
 
     a = AlgebraicNumber(sqrt(2))
     b = AlgebraicNumber(sqrt(3))
@@ -89,8 +90,8 @@ def test_minimal_polynomial():
     assert minimal_polynomial(a, x) == x**2 - 2
     assert minimal_polynomial(b, x) == x**2 - 3
 
-    assert minimal_polynomial(a, x, polys=True) == Poly(x**2 - 2)
-    assert minimal_polynomial(b, x, polys=True) == Poly(x**2 - 3)
+    assert minimal_polynomial(a, x, polys=True) == Poly(x**2 - 2, domain='QQ')
+    assert minimal_polynomial(b, x, polys=True) == Poly(x**2 - 3, domain='QQ')
 
     assert minimal_polynomial(sqrt(a/2 + 17), x) == 2*x**4 - 68*x**2 + 577
     assert minimal_polynomial(sqrt(b/2 + 17), x) == 4*x**4 - 136*x**2 + 1153
@@ -150,9 +151,17 @@ def test_minimal_polynomial():
     assert minimal_polynomial(I, x, domain=QQ) == x**2 + 1
     assert minimal_polynomial(I, x, domain='QQ(y)') == x**2 + 1
 
+    #issue 11553
+    assert minimal_polynomial(GoldenRatio, x) == x**2 - x - 1
+    assert minimal_polynomial(TribonacciConstant + 3, x) == x**3 - 10*x**2 + 32*x - 34
+    assert minimal_polynomial(GoldenRatio, x, domain=QQ.algebraic_field(sqrt(5))) == \
+            2*x - sqrt(5) - 1
+    assert minimal_polynomial(TribonacciConstant, x, domain=QQ.algebraic_field(cbrt(19 - 3*sqrt(33)))) == \
+    48*x - 19*(19 - 3*sqrt(33))**Rational(2, 3) - 3*sqrt(33)*(19 - 3*sqrt(33))**Rational(2, 3) \
+    - 16*(19 - 3*sqrt(33))**Rational(1, 3) - 16
 
 def test_minimal_polynomial_hi_prec():
-    p = 1/sqrt(1 - 9*sqrt(2) + 7*sqrt(3) + S(1)/10**30)
+    p = 1/sqrt(1 - 9*sqrt(2) + 7*sqrt(3) + Rational(1, 10)**30)
     mp = minimal_polynomial(p, x)
     # checked with Wolfram Alpha
     assert mp.coeff(x**6) == -1232000000000000000000000000001223999999999999999999999999999987999999999999999999999999999996000000000000000000000000000000
@@ -201,20 +210,20 @@ def test_minpoly_compose():
     mp = minimal_polynomial(exp(I*pi/7) + sqrt(2), x)
     assert mp == x**12 - 2*x**11 - 9*x**10 + 16*x**9 + 43*x**8 - 70*x**7 - 97*x**6 + 126*x**5 + 211*x**4 - 212*x**3 - 37*x**2 + 142*x + 127
 
-    mp = minimal_polynomial(exp(2*I*pi/7), x)
+    mp = minimal_polynomial(exp(I*pi*Rational(2, 7)), x)
     assert mp == x**6 + x**5 + x**4 + x**3 + x**2 + x + 1
-    mp = minimal_polynomial(exp(2*I*pi/15), x)
+    mp = minimal_polynomial(exp(I*pi*Rational(2, 15)), x)
     assert mp == x**8 - x**7 + x**5 - x**4 + x**3 - x + 1
-    mp = minimal_polynomial(cos(2*pi/7), x)
+    mp = minimal_polynomial(cos(pi*Rational(2, 7)), x)
     assert mp == 8*x**3 + 4*x**2 - 4*x - 1
-    mp = minimal_polynomial(sin(2*pi/7), x)
-    ex = (5*cos(2*pi/7) - 7)/(9*cos(pi/7) - 5*cos(3*pi/7))
+    mp = minimal_polynomial(sin(pi*Rational(2, 7)), x)
+    ex = (5*cos(pi*Rational(2, 7)) - 7)/(9*cos(pi/7) - 5*cos(pi*Rational(3, 7)))
     mp = minimal_polynomial(ex, x)
     assert mp == x**3 + 2*x**2 - x - 1
     assert minimal_polynomial(-1/(2*cos(pi/7)), x) == x**3 + 2*x**2 - x - 1
-    assert minimal_polynomial(sin(2*pi/15), x) == \
+    assert minimal_polynomial(sin(pi*Rational(2, 15)), x) == \
             256*x**8 - 448*x**6 + 224*x**4 - 32*x**2 + 1
-    assert minimal_polynomial(sin(5*pi/14), x) == 8*x**3 - 4*x**2 - 4*x + 1
+    assert minimal_polynomial(sin(pi*Rational(5, 14)), x) == 8*x**3 - 4*x**2 - 4*x + 1
     assert minimal_polynomial(cos(pi/15), x) == 16*x**4 + 8*x**3 - 16*x**2 - 8*x + 1
 
     ex = rootof(x**3 +x*4 + 1, 0)
@@ -267,9 +276,9 @@ def test_primitive_element():
     assert primitive_element(
         [sqrt(2), sqrt(3)], x) == (x**4 - 10*x**2 + 1, [1, 1])
 
-    assert primitive_element([sqrt(2)], x, polys=True) == (Poly(x**2 - 2), [1])
+    assert primitive_element([sqrt(2)], x, polys=True) == (Poly(x**2 - 2, domain='QQ'), [1])
     assert primitive_element([sqrt(
-        2), sqrt(3)], x, polys=True) == (Poly(x**4 - 10*x**2 + 1), [1, 1])
+        2), sqrt(3)], x, polys=True) == (Poly(x**4 - 10*x**2 + 1, domain='QQ'), [1, 1])
 
     assert primitive_element(
         [sqrt(2)], x, ex=True) == (x**2 - 2, [1], [[1, 0]])
@@ -278,9 +287,9 @@ def test_primitive_element():
          Q(1, 2), 0, Q(11, 2), 0]])
 
     assert primitive_element(
-        [sqrt(2)], x, ex=True, polys=True) == (Poly(x**2 - 2), [1], [[1, 0]])
+        [sqrt(2)], x, ex=True, polys=True) == (Poly(x**2 - 2, domain='QQ'), [1], [[1, 0]])
     assert primitive_element([sqrt(2), sqrt(3)], x, ex=True, polys=True) == \
-        (Poly(x**4 - 10*x**2 + 1), [1, 1], [[Q(1, 2), 0, -Q(9, 2),
+        (Poly(x**4 - 10*x**2 + 1, domain='QQ'), [1, 1], [[Q(1, 2), 0, -Q(9, 2),
          0], [-Q(1, 2), 0, Q(11, 2), 0]])
 
     assert primitive_element([sqrt(2)], polys=True) == (Poly(x**2 - 2), [1])
@@ -353,29 +362,29 @@ def test_field_isomorphism():
     assert field_isomorphism( I*sqrt(3), -I*sqrt(3)/2) == [-2, 0]
     assert field_isomorphism(-I*sqrt(3), -I*sqrt(3)/2) == [ 2, 0]
 
-    assert field_isomorphism( 2*I*sqrt(3)/7, 5*I*sqrt(3)/3) == [ S(6)/35, 0]
-    assert field_isomorphism(-2*I*sqrt(3)/7, 5*I*sqrt(3)/3) == [-S(6)/35, 0]
+    assert field_isomorphism( 2*I*sqrt(3)/7, 5*I*sqrt(3)/3) == [ Rational(6, 35), 0]
+    assert field_isomorphism(-2*I*sqrt(3)/7, 5*I*sqrt(3)/3) == [Rational(-6, 35), 0]
 
-    assert field_isomorphism( 2*I*sqrt(3)/7, -5*I*sqrt(3)/3) == [-S(6)/35, 0]
-    assert field_isomorphism(-2*I*sqrt(3)/7, -5*I*sqrt(3)/3) == [ S(6)/35, 0]
+    assert field_isomorphism( 2*I*sqrt(3)/7, -5*I*sqrt(3)/3) == [Rational(-6, 35), 0]
+    assert field_isomorphism(-2*I*sqrt(3)/7, -5*I*sqrt(3)/3) == [ Rational(6, 35), 0]
 
     assert field_isomorphism(
-        2*I*sqrt(3)/7 + 27, 5*I*sqrt(3)/3) == [ S(6)/35, 27]
+        2*I*sqrt(3)/7 + 27, 5*I*sqrt(3)/3) == [ Rational(6, 35), 27]
     assert field_isomorphism(
-        -2*I*sqrt(3)/7 + 27, 5*I*sqrt(3)/3) == [-S(6)/35, 27]
+        -2*I*sqrt(3)/7 + 27, 5*I*sqrt(3)/3) == [Rational(-6, 35), 27]
 
     assert field_isomorphism(
-        2*I*sqrt(3)/7 + 27, -5*I*sqrt(3)/3) == [-S(6)/35, 27]
+        2*I*sqrt(3)/7 + 27, -5*I*sqrt(3)/3) == [Rational(-6, 35), 27]
     assert field_isomorphism(
-        -2*I*sqrt(3)/7 + 27, -5*I*sqrt(3)/3) == [ S(6)/35, 27]
+        -2*I*sqrt(3)/7 + 27, -5*I*sqrt(3)/3) == [ Rational(6, 35), 27]
 
     p = AlgebraicNumber( sqrt(2) + sqrt(3))
     q = AlgebraicNumber(-sqrt(2) + sqrt(3))
     r = AlgebraicNumber( sqrt(2) - sqrt(3))
     s = AlgebraicNumber(-sqrt(2) - sqrt(3))
 
-    pos_coeffs = [ S(1)/2, S(0), -S(9)/2, S(0)]
-    neg_coeffs = [-S(1)/2, S(0), S(9)/2, S(0)]
+    pos_coeffs = [ S.Half, S.Zero, Rational(-9, 2), S.Zero]
+    neg_coeffs = [Rational(-1, 2), S.Zero, Rational(9, 2), S.Zero]
 
     a = AlgebraicNumber(sqrt(2))
 
@@ -411,8 +420,8 @@ def test_field_isomorphism():
     assert field_isomorphism(a, r, fast=False) == neg_coeffs
     assert field_isomorphism(a, s, fast=False) == pos_coeffs
 
-    pos_coeffs = [ S(1)/2, S(0), -S(11)/2, S(0)]
-    neg_coeffs = [-S(1)/2, S(0), S(11)/2, S(0)]
+    pos_coeffs = [ S.Half, S.Zero, Rational(-11, 2), S.Zero]
+    neg_coeffs = [Rational(-1, 2), S.Zero, Rational(11, 2), S.Zero]
 
     a = AlgebraicNumber(sqrt(3))
 
@@ -448,8 +457,8 @@ def test_field_isomorphism():
     assert field_isomorphism(a, r, fast=False) == neg_coeffs
     assert field_isomorphism(a, s, fast=False) == neg_coeffs
 
-    pos_coeffs = [ S(3)/2, S(0), -S(33)/2, -S(8)]
-    neg_coeffs = [-S(3)/2, S(0), S(33)/2, -S(8)]
+    pos_coeffs = [ Rational(3, 2), S.Zero, Rational(-33, 2), -S(8)]
+    neg_coeffs = [Rational(-3, 2), S.Zero, Rational(33, 2), -S(8)]
 
     a = AlgebraicNumber(3*sqrt(3) - 8)
 
@@ -470,10 +479,10 @@ def test_field_isomorphism():
 
     a = AlgebraicNumber(3*sqrt(2) + 2*sqrt(3) + 1)
 
-    pos_1_coeffs = [ S(1)/2, S(0), -S(5)/2, S(1)]
-    neg_5_coeffs = [-S(5)/2, S(0), S(49)/2, S(1)]
-    pos_5_coeffs = [ S(5)/2, S(0), -S(49)/2, S(1)]
-    neg_1_coeffs = [-S(1)/2, S(0), S(5)/2, S(1)]
+    pos_1_coeffs = [ S.Half, S.Zero, Rational(-5, 2), S.One]
+    neg_5_coeffs = [Rational(-5, 2), S.Zero, Rational(49, 2), S.One]
+    pos_5_coeffs = [ Rational(5, 2), S.Zero, Rational(-49, 2), S.One]
+    neg_1_coeffs = [Rational(-1, 2), S.Zero, Rational(5, 2), S.One]
 
     assert is_isomorphism_possible(a, p) is True
     assert is_isomorphism_possible(a, q) is True
@@ -511,7 +520,7 @@ def test_to_number_field():
     assert to_number_field(
         [sqrt(2), sqrt(3)]) == AlgebraicNumber(sqrt(2) + sqrt(3))
 
-    a = AlgebraicNumber(sqrt(2) + sqrt(3), [S(1)/2, S(0), -S(9)/2, S(0)])
+    a = AlgebraicNumber(sqrt(2) + sqrt(3), [S.Half, S.Zero, Rational(-9, 2), S.Zero])
 
     assert to_number_field(sqrt(2), sqrt(2) + sqrt(3)) == a
     assert to_number_field(sqrt(2), AlgebraicNumber(sqrt(2) + sqrt(3))) == a
@@ -532,7 +541,7 @@ def test_AlgebraicNumber():
 
     assert a.is_aliased is False
 
-    assert a.coeffs() == [S(1), S(0)]
+    assert a.coeffs() == [S.One, S.Zero]
     assert a.native_coeffs() == [QQ(1), QQ(0)]
 
     a = AlgebraicNumber(root, gen=x, alias='y')
@@ -560,11 +569,11 @@ def test_AlgebraicNumber():
     assert AlgebraicNumber(sqrt(2), (0, 0)).rep == DMP([], QQ)
 
     assert AlgebraicNumber(sqrt(2), [8]).rep == DMP([QQ(8)], QQ)
-    assert AlgebraicNumber(sqrt(2), [S(8)/3]).rep == DMP([QQ(8, 3)], QQ)
+    assert AlgebraicNumber(sqrt(2), [Rational(8, 3)]).rep == DMP([QQ(8, 3)], QQ)
 
     assert AlgebraicNumber(sqrt(2), [7, 3]).rep == DMP([QQ(7), QQ(3)], QQ)
     assert AlgebraicNumber(
-        sqrt(2), [S(7)/9, S(3)/2]).rep == DMP([QQ(7, 9), QQ(3, 2)], QQ)
+        sqrt(2), [Rational(7, 9), Rational(3, 2)]).rep == DMP([QQ(7, 9), QQ(3, 2)], QQ)
 
     assert AlgebraicNumber(sqrt(2), [1, 2, 3]).rep == DMP([QQ(2), QQ(5)], QQ)
 
@@ -578,7 +587,7 @@ def test_AlgebraicNumber():
 
     assert a.is_aliased is False
 
-    assert a.coeffs() == [S(1), S(2)]
+    assert a.coeffs() == [S.One, S(2)]
     assert a.native_coeffs() == [QQ(1), QQ(2)]
 
     a = AlgebraicNumber((minpoly, root), [1, 2])
@@ -610,7 +619,6 @@ def test_AlgebraicNumber():
     assert a == b
 
     c = AlgebraicNumber(sqrt(2), gen=x)
-    d = AlgebraicNumber(sqrt(2), gen=x)
 
     assert a == b
     assert a == c
@@ -625,8 +633,8 @@ def test_AlgebraicNumber():
     a = AlgebraicNumber(sqrt(2), [1, 0])
     b = AlgebraicNumber(sqrt(2), [1, 0], alias=y)
 
-    assert a.as_poly(x) == Poly(x)
-    assert b.as_poly() == Poly(y)
+    assert a.as_poly(x) == Poly(x, domain='QQ')
+    assert b.as_poly() == Poly(y, domain='QQ')
 
     assert a.as_expr() == sqrt(2)
     assert a.as_expr(x) == x
@@ -640,8 +648,8 @@ def test_AlgebraicNumber():
 
     assert p == Poly(2*p.gen + 3)
 
-    assert a.as_poly(x) == Poly(2*x + 3)
-    assert b.as_poly() == Poly(2*y + 3)
+    assert a.as_poly(x) == Poly(2*x + 3, domain='QQ')
+    assert b.as_poly() == Poly(2*y + 3, domain='QQ')
 
     assert a.as_expr() == 2*sqrt(2) + 3
     assert a.as_expr(x) == 2*x + 3
@@ -676,7 +684,7 @@ def test_to_algebraic_integer():
     assert a.root == 2*sqrt(3)
     assert a.rep == DMP([QQ(1), QQ(0)], QQ)
 
-    a = AlgebraicNumber(sqrt(3)/2, [S(7)/19, 3], gen=x).to_algebraic_integer()
+    a = AlgebraicNumber(sqrt(3)/2, [Rational(7, 19), 3], gen=x).to_algebraic_integer()
 
     assert a.minpoly == x**2 - 12
     assert a.root == 2*sqrt(3)
@@ -691,13 +699,13 @@ def test_IntervalPrinter():
 
 def test_isolate():
     assert isolate(1) == (1, 1)
-    assert isolate(S(1)/2) == (S(1)/2, S(1)/2)
+    assert isolate(S.Half) == (S.Half, S.Half)
 
     assert isolate(sqrt(2)) == (1, 2)
     assert isolate(-sqrt(2)) == (-2, -1)
 
-    assert isolate(sqrt(2), eps=S(1)/100) == (S(24)/17, S(17)/12)
-    assert isolate(-sqrt(2), eps=S(1)/100) == (-S(17)/12, -S(24)/17)
+    assert isolate(sqrt(2), eps=Rational(1, 100)) == (Rational(24, 17), Rational(17, 12))
+    assert isolate(-sqrt(2), eps=Rational(1, 100)) == (Rational(-17, 12), Rational(-24, 17))
 
     raises(NotImplementedError, lambda: isolate(I))
 
@@ -719,16 +727,16 @@ def test_minpoly_fraction_field():
     assert minimal_polynomial(sqrt(x) / z, y) == z**2*y**2 - x
     assert minimal_polynomial(sqrt(x) / (z + 1), y) == (z**2 + 2*z + 1)*y**2 - x
 
-    assert minimal_polynomial(1/x, y, polys=True) == Poly(-x*y + 1, y)
+    assert minimal_polynomial(1/x, y, polys=True) == Poly(-x*y + 1, y, domain='ZZ(x)')
     assert minimal_polynomial(1 / (x + 1), y, polys=True) == \
-        Poly((x + 1)*y - 1, y)
-    assert minimal_polynomial(sqrt(x), y, polys=True) == Poly(y**2 - x, y)
+        Poly((x + 1)*y - 1, y, domain='ZZ(x)')
+    assert minimal_polynomial(sqrt(x), y, polys=True) == Poly(y**2 - x, y, domain='ZZ(x)')
     assert minimal_polynomial(sqrt(x) / z, y, polys=True) == \
-        Poly(z**2*y**2 - x, y)
+        Poly(z**2*y**2 - x, y, domain='ZZ(x, z)')
 
     # this is (sqrt(1 + x**3)/x).integrate(x).diff(x) - sqrt(1 + x**3)/x
-    a = sqrt(x)/sqrt(1 + x**(-3)) - sqrt(x**3 + 1)/x + 1/(x**(S(5)/2)* \
-        (1 + x**(-3))**(S(3)/2)) + 1/(x**(S(11)/2)*(1 + x**(-3))**(S(3)/2))
+    a = sqrt(x)/sqrt(1 + x**(-3)) - sqrt(x**3 + 1)/x + 1/(x**Rational(5, 2)* \
+        (1 + x**(-3))**Rational(3, 2)) + 1/(x**Rational(11, 2)*(1 + x**(-3))**Rational(3, 2))
 
     assert minimal_polynomial(a, y) == y
 
@@ -751,3 +759,26 @@ def test_minpoly_domain():
         domain=QQ.algebraic_field(sqrt(2))) == 2*x**2 - 3
 
     raises(NotAlgebraic, lambda: minimal_polynomial(y, x, domain=QQ))
+
+
+def test_issue_14831():
+    a = -2*sqrt(2)*sqrt(12*sqrt(2) + 17)
+    assert minimal_polynomial(a, x) == x**2 + 16*x - 8
+    e = (-3*sqrt(12*sqrt(2) + 17) + 12*sqrt(2) +
+         17 - 2*sqrt(2)*sqrt(12*sqrt(2) + 17))
+    assert minimal_polynomial(e, x) == x
+
+
+def test_issue_18248():
+    assert nonlinsolve([x*y**3-sqrt(2)/3, x*y**6-4/(9*(sqrt(3)))],x,y) == \
+            FiniteSet((sqrt(3)/2, sqrt(6)/3), (sqrt(3)/2, -sqrt(6)/6 - sqrt(2)*I/2),
+            (sqrt(3)/2, -sqrt(6)/6 + sqrt(2)*I/2))
+
+
+def test_issue_13230():
+    c1 = Circle(Point2D(3, sqrt(5)), 5)
+    c2 = Circle(Point2D(4, sqrt(7)), 6)
+    assert intersection(c1, c2) == [Point2D(-1 + (-sqrt(7) + sqrt(5))*(-2*sqrt(7)/29
+    + 9*sqrt(5)/29 + sqrt(196*sqrt(35) + 1941)/29), -2*sqrt(7)/29 + 9*sqrt(5)/29
+    + sqrt(196*sqrt(35) + 1941)/29), Point2D(-1 + (-sqrt(7) + sqrt(5))*(-sqrt(196*sqrt(35)
+    + 1941)/29 - 2*sqrt(7)/29 + 9*sqrt(5)/29), -sqrt(196*sqrt(35) + 1941)/29 - 2*sqrt(7)/29 + 9*sqrt(5)/29)]
