@@ -9,7 +9,7 @@ from sympy.polys.partfrac import (
 
 from sympy import (S, Poly, E, pi, I, Matrix, Eq, RootSum, Lambda,
                    Symbol, Dummy, factor, together, sqrt, Expr, Rational)
-from sympy.testing.pytest import raises, XFAIL
+from sympy.testing.pytest import raises, ON_TRAVIS, skip, XFAIL
 from sympy.abc import x, y, a, b, c
 
 
@@ -74,6 +74,23 @@ def test_apart_symbolic():
         1/((a - b)*(a - c)*(a + x))
 
 
+def _make_extension_example():
+    # https://github.com/sympy/sympy/issues/18531
+    from sympy.core import Mul
+    def mul2(expr):
+        # 2-arg mul hack...
+        return Mul(2, expr, evaluate=False)
+
+    f = ((x**2 + 1)**3/((x - 1)**2*(x + 1)**2*(-x**2 + 2*x + 1)*(x**2 + 2*x - 1)))
+    g = (1/mul2(x - sqrt(2) + 1)
+       - 1/mul2(x - sqrt(2) - 1)
+       + 1/mul2(x + 1 + sqrt(2))
+       - 1/mul2(x - 1 + sqrt(2))
+       + 1/mul2((x + 1)**2)
+       + 1/mul2((x - 1)**2))
+    return f, g
+
+
 def test_apart_extension():
     f = 2/(x**2 + 1)
     g = I/(x + I) - I/(x - I)
@@ -85,6 +102,22 @@ def test_apart_extension():
 
     assert factor(together(apart(f)).expand()) == f
 
+    f, g = _make_extension_example()
+
+    # XXX: Only works with dotprodsimp. See test_apart_extension_xfail below
+    from sympy.matrices import dotprodsimp
+    with dotprodsimp(True):
+        assert apart(f, x, extension={sqrt(2)}) == g
+
+
+# XXX: This is XFAIL just because it is slow
+@XFAIL
+def test_apart_extension_xfail():
+    if ON_TRAVIS:
+        skip('Too slow for Travis')
+    f, g = _make_extension_example()
+    assert apart(f, x, extension={sqrt(2)}) == g
+
 
 def test_apart_full():
     f = 1/(x**2 + 1)
