@@ -11,12 +11,12 @@ from sympy.polys.polyroots import (root_factors, roots_linear,
     roots_binomial, preprocess_roots, roots)
 
 from sympy.polys.orthopolys import legendre_poly
+from sympy.polys.polyerrors import PolynomialError
 from sympy.polys.polyutils import _nsort
 
 from sympy.utilities.iterables import cartes
-from sympy.utilities.pytest import raises, slow
-from sympy.utilities.randtest import verify_numerically
-from sympy.core.compatibility import range
+from sympy.testing.pytest import raises, slow
+from sympy.testing.randtest import verify_numerically
 import mpmath
 
 
@@ -38,12 +38,12 @@ def _check(roots):
 
 
 def test_roots_linear():
-    assert roots_linear(Poly(2*x + 1, x)) == [-Rational(1, 2)]
+    assert roots_linear(Poly(2*x + 1, x)) == [Rational(-1, 2)]
 
 
 def test_roots_quadratic():
     assert roots_quadratic(Poly(2*x**2, x)) == [0, 0]
-    assert roots_quadratic(Poly(2*x**2 + 3*x, x)) == [-Rational(3, 2), 0]
+    assert roots_quadratic(Poly(2*x**2 + 3*x, x)) == [Rational(-3, 2), 0]
     assert roots_quadratic(Poly(2*x**2 + 3, x)) == [-I*sqrt(6)/2, I*sqrt(6)/2]
     assert roots_quadratic(Poly(2*x**2 + 4*x + 3, x)) == [-1 - I*sqrt(2)/2, -1 + I*sqrt(2)/2]
     _check(Poly(2*x**2 + 4*x + 3, x).all_roots())
@@ -74,10 +74,20 @@ def test_roots_quadratic():
         roots = roots_quadratic(f)
         assert roots == _nsort(roots)
 
+
+def test_issue_7724():
+    eq = Poly(x**4*I + x**2 + I, x)
+    assert roots(eq) == {
+        sqrt(I/2 + sqrt(5)*I/2): 1,
+        sqrt(-sqrt(5)*I/2 + I/2): 1,
+        -sqrt(I/2 + sqrt(5)*I/2): 1,
+        -sqrt(-sqrt(5)*I/2 + I/2): 1}
+
+
 def test_issue_8438():
     p = Poly([1, y, -2, -3], x).as_expr()
     roots = roots_cubic(Poly(p, x), x)
-    z = -S(3)/2 - 7*I/2  # this will fail in code given in commit msg
+    z = Rational(-3, 2) - I*Rational(7, 2)  # this will fail in code given in commit msg
     post = [r.subs(y, z) for r in roots]
     assert set(post) == \
     set(roots_cubic(Poly(p.subs(y, z), x)))
@@ -133,6 +143,16 @@ def test_issue_14522():
     assert all(eq(r) == 0 for r in roots_eq)
 
 
+def test_issue_15076():
+    sol = roots_quartic(Poly(t**4 - 6*t**2 + t/x - 3, t))
+    assert sol[0].has(x)
+
+
+def test_issue_16589():
+    eq = Poly(x**4 - 8*sqrt(2)*x**3 + 4*x**3 - 64*sqrt(2)*x**2 + 1024*x, x)
+    roots_eq = roots(eq)
+    assert 0 in roots_eq
+
 def test_roots_cubic():
     assert roots_cubic(Poly(2*x**3, x)) == [0, 0, 0]
     assert roots_cubic(Poly(x**3 - 3*x**2 + 3*x - 1, x)) == [1, 1, 1]
@@ -144,9 +164,9 @@ def test_roots_cubic():
     eq = -x**3 + 2*x**2 + 3*x - 2
     assert roots(eq, trig=True, multiple=True) == \
            roots_cubic(Poly(eq, x), trig=True) == [
-        S(2)/3 + 2*sqrt(13)*cos(acos(8*sqrt(13)/169)/3)/3,
-        -2*sqrt(13)*sin(-acos(8*sqrt(13)/169)/3 + pi/6)/3 + S(2)/3,
-        -2*sqrt(13)*cos(-acos(8*sqrt(13)/169)/3 + pi/3)/3 + S(2)/3,
+        Rational(2, 3) + 2*sqrt(13)*cos(acos(8*sqrt(13)/169)/3)/3,
+        -2*sqrt(13)*sin(-acos(8*sqrt(13)/169)/3 + pi/6)/3 + Rational(2, 3),
+        -2*sqrt(13)*cos(-acos(8*sqrt(13)/169)/3 + pi/3)/3 + Rational(2, 3),
         ]
 
 
@@ -181,7 +201,7 @@ def test_roots_quartic():
         if i == 2:
             c = -a*(a**2/S(8) - b/S(2))
         elif i == 3:
-            d = a*(a*(3*a**2/S(256) - b/S(16)) + c/S(4))
+            d = a*(a*(a**2*Rational(3, 256) - b/S(16)) + c/S(4))
         eq = x**4 + a*x**3 + b*x**2 + c*x + d
         ans = roots_quartic(Poly(eq, x))
         assert all(eq.subs(x, ai).n(chop=True) == 0 for ai in ans)
@@ -200,9 +220,9 @@ def test_roots_quartic():
     ans = roots_quartic(eq)
     assert all(type(i) == Piecewise for i in ans)
     reps = (
-        dict(y=-Rational(1, 3), z=-Rational(1, 4)),  # 4 real
-        dict(y=-Rational(1, 3), z=-Rational(1, 2)),  # 2 real
-        dict(y=-Rational(1, 3), z=-2))  # 0 real
+        dict(y=Rational(-1, 3), z=Rational(-1, 4)),  # 4 real
+        dict(y=Rational(-1, 3), z=Rational(-1, 2)),  # 2 real
+        dict(y=Rational(-1, 3), z=-2))  # 0 real
     for rep in reps:
         sol = roots_quartic(Poly(eq.subs(rep), x))
         assert all([verify_numerically(w.subs(rep) - s, 0) for w, s in zip(ans, sol)])
@@ -212,18 +232,18 @@ def test_roots_cyclotomic():
     assert roots_cyclotomic(cyclotomic_poly(1, x, polys=True)) == [1]
     assert roots_cyclotomic(cyclotomic_poly(2, x, polys=True)) == [-1]
     assert roots_cyclotomic(cyclotomic_poly(
-        3, x, polys=True)) == [-S(1)/2 - I*sqrt(3)/2, -S(1)/2 + I*sqrt(3)/2]
+        3, x, polys=True)) == [Rational(-1, 2) - I*sqrt(3)/2, Rational(-1, 2) + I*sqrt(3)/2]
     assert roots_cyclotomic(cyclotomic_poly(4, x, polys=True)) == [-I, I]
     assert roots_cyclotomic(cyclotomic_poly(
-        6, x, polys=True)) == [S(1)/2 - I*sqrt(3)/2, S(1)/2 + I*sqrt(3)/2]
+        6, x, polys=True)) == [S.Half - I*sqrt(3)/2, S.Half + I*sqrt(3)/2]
 
     assert roots_cyclotomic(cyclotomic_poly(7, x, polys=True)) == [
         -cos(pi/7) - I*sin(pi/7),
         -cos(pi/7) + I*sin(pi/7),
-        -cos(3*pi/7) - I*sin(3*pi/7),
-        -cos(3*pi/7) + I*sin(3*pi/7),
-        cos(2*pi/7) - I*sin(2*pi/7),
-        cos(2*pi/7) + I*sin(2*pi/7),
+        -cos(pi*Rational(3, 7)) - I*sin(pi*Rational(3, 7)),
+        -cos(pi*Rational(3, 7)) + I*sin(pi*Rational(3, 7)),
+        cos(pi*Rational(2, 7)) - I*sin(pi*Rational(2, 7)),
+        cos(pi*Rational(2, 7)) + I*sin(pi*Rational(2, 7)),
     ]
 
     assert roots_cyclotomic(cyclotomic_poly(8, x, polys=True)) == [
@@ -259,7 +279,7 @@ def test_roots_cyclotomic():
 def test_roots_binomial():
     assert roots_binomial(Poly(5*x, x)) == [0]
     assert roots_binomial(Poly(5*x**4, x)) == [0, 0, 0, 0]
-    assert roots_binomial(Poly(5*x + 2, x)) == [-Rational(2, 5)]
+    assert roots_binomial(Poly(5*x + 2, x)) == [Rational(-2, 5)]
 
     A = 10**Rational(3, 4)/10
 
@@ -284,9 +304,9 @@ def test_roots_binomial():
 
     # issue 8813
     assert roots(Poly(2*x**3 - 16*y**3, x)) == {
-        2*y*(-S(1)/2 - sqrt(3)*I/2): 1,
+        2*y*(Rational(-1, 2) - sqrt(3)*I/2): 1,
         2*y: 1,
-        2*y*(-S(1)/2 + sqrt(3)*I/2): 1}
+        2*y*(Rational(-1, 2) + sqrt(3)*I/2): 1}
 
 
 def test_roots_preprocessing():
@@ -355,29 +375,29 @@ def test_roots0():
     assert roots(x**9, x) == {S.Zero: 9}
     assert roots(((x - 2)*(x + 3)*(x - 4)).expand(), x) == {-S(3): 1, S(2): 1, S(4): 1}
 
-    assert roots(2*x + 1, x) == {-S.Half: 1}
-    assert roots((2*x + 1)**2, x) == {-S.Half: 2}
-    assert roots((2*x + 1)**5, x) == {-S.Half: 5}
-    assert roots((2*x + 1)**10, x) == {-S.Half: 10}
+    assert roots(2*x + 1, x) == {Rational(-1, 2): 1}
+    assert roots((2*x + 1)**2, x) == {Rational(-1, 2): 2}
+    assert roots((2*x + 1)**5, x) == {Rational(-1, 2): 5}
+    assert roots((2*x + 1)**10, x) == {Rational(-1, 2): 10}
 
     assert roots(x**4 - 1, x) == {I: 1, S.One: 1, -S.One: 1, -I: 1}
     assert roots((x**4 - 1)**2, x) == {I: 2, S.One: 2, -S.One: 2, -I: 2}
 
-    assert roots(((2*x - 3)**2).expand(), x) == { Rational(3, 2): 2}
-    assert roots(((2*x + 3)**2).expand(), x) == {-Rational(3, 2): 2}
+    assert roots(((2*x - 3)**2).expand(), x) == {Rational( 3, 2): 2}
+    assert roots(((2*x + 3)**2).expand(), x) == {Rational(-3, 2): 2}
 
-    assert roots(((2*x - 3)**3).expand(), x) == { Rational(3, 2): 3}
-    assert roots(((2*x + 3)**3).expand(), x) == {-Rational(3, 2): 3}
+    assert roots(((2*x - 3)**3).expand(), x) == {Rational( 3, 2): 3}
+    assert roots(((2*x + 3)**3).expand(), x) == {Rational(-3, 2): 3}
 
-    assert roots(((2*x - 3)**5).expand(), x) == { Rational(3, 2): 5}
-    assert roots(((2*x + 3)**5).expand(), x) == {-Rational(3, 2): 5}
+    assert roots(((2*x - 3)**5).expand(), x) == {Rational( 3, 2): 5}
+    assert roots(((2*x + 3)**5).expand(), x) == {Rational(-3, 2): 5}
 
     assert roots(((a*x - b)**5).expand(), x) == { b/a: 5}
     assert roots(((a*x + b)**5).expand(), x) == {-b/a: 5}
 
     assert roots(x**2 + (-a - 1)*x + a, x) == {a: 1, S.One: 1}
 
-    assert roots(x**4 - 2*x**2 + 1, x) == {S.One: 2, -S.One: 2}
+    assert roots(x**4 - 2*x**2 + 1, x) == {S.One: 2, S.NegativeOne: 2}
 
     assert roots(x**6 - 4*x**4 + 4*x**3 - x**2, x) == \
         {S.One: 2, -1 - sqrt(2): 1, S.Zero: 2, -1 + sqrt(2): 1}
@@ -393,7 +413,8 @@ def test_roots0():
     f = -2016*x**2 - 5616*x**3 - 2056*x**4 + 3324*x**5 + 2176*x**6 - \
         224*x**7 - 384*x**8 - 64*x**9
 
-    assert roots(f) == {S(0): 2, -S(2): 2, S(2): 1, -S(7)/2: 1, -S(3)/2: 1, -S(1)/2: 1, S(3)/2: 1}
+    assert roots(f) == {S.Zero: 2, -S(2): 2, S(2): 1, Rational(-7, 2): 1,
+                Rational(-3, 2): 1, Rational(-1, 2): 1, Rational(3, 2): 1}
 
     assert roots((a + b + c)*x - (a + b + c + d), x) == {(a + b + c + d)/(a + b + c): 1}
 
@@ -406,9 +427,9 @@ def test_roots0():
     assert roots(x**3 + 2*x**2 + 4*x + 8, x, cubics=True) == \
         {-2*I: 1, 2*I: 1, -S(2): 1}
     assert roots((x**2 - x)*(x**3 + 2*x**2 + 4*x + 8), x ) == \
-        {S(1): 1, S(0): 1, -S(2): 1, -2*I: 1, 2*I: 1}
+        {S.One: 1, S.Zero: 1, -S(2): 1, -2*I: 1, 2*I: 1}
 
-    r1_2, r1_3 = Rational(1, 2), Rational(1, 3)
+    r1_2, r1_3 = S.Half, Rational(1, 3)
 
     x0 = (3*sqrt(33) + 19)**r1_3
     x1 = 4/x0/3
@@ -467,6 +488,10 @@ def test_roots0():
     assert roots(x**4 - 1, x, filter='Z', multiple=True) == [-S.One, S.One]
     assert roots(x**4 - 1, x, filter='I', multiple=True) == [I, -I]
 
+    ar, br = symbols('a, b', real=True)
+    p = x**2*(ar-br)**2 + 2*x*(br-ar) + 1
+    assert roots(p, x, filter='R') == {1/(ar - br): 2}
+
     assert roots(x**3, x, multiple=True) == [S.Zero, S.Zero, S.Zero]
     assert roots(1234, x, multiple=True) == []
 
@@ -474,22 +499,22 @@ def test_roots0():
 
     assert roots(f) == {
         -I*sin(pi/7) + cos(pi/7): 1,
-        -I*sin(2*pi/7) - cos(2*pi/7): 1,
-        -I*sin(3*pi/7) + cos(3*pi/7): 1,
+        -I*sin(pi*Rational(2, 7)) - cos(pi*Rational(2, 7)): 1,
+        -I*sin(pi*Rational(3, 7)) + cos(pi*Rational(3, 7)): 1,
         I*sin(pi/7) + cos(pi/7): 1,
-        I*sin(2*pi/7) - cos(2*pi/7): 1,
-        I*sin(3*pi/7) + cos(3*pi/7): 1,
+        I*sin(pi*Rational(2, 7)) - cos(pi*Rational(2, 7)): 1,
+        I*sin(pi*Rational(3, 7)) + cos(pi*Rational(3, 7)): 1,
     }
 
     g = ((x**2 + 1)*f**2).expand()
 
     assert roots(g) == {
         -I*sin(pi/7) + cos(pi/7): 2,
-        -I*sin(2*pi/7) - cos(2*pi/7): 2,
-        -I*sin(3*pi/7) + cos(3*pi/7): 2,
+        -I*sin(pi*Rational(2, 7)) - cos(pi*Rational(2, 7)): 2,
+        -I*sin(pi*Rational(3, 7)) + cos(pi*Rational(3, 7)): 2,
         I*sin(pi/7) + cos(pi/7): 2,
-        I*sin(2*pi/7) - cos(2*pi/7): 2,
-        I*sin(3*pi/7) + cos(3*pi/7): 2,
+        I*sin(pi*Rational(2, 7)) - cos(pi*Rational(2, 7)): 2,
+        I*sin(pi*Rational(3, 7)) + cos(pi*Rational(3, 7)): 2,
         -I: 1, I: 1,
     }
 
@@ -668,3 +693,8 @@ def test_nroots2():
 
 def test_roots_composite():
     assert len(roots(Poly(y**3 + y**2*sqrt(x) + y + x, y, composite=True))) == 3
+
+
+def test_issue_19113():
+    eq = cos(x)**3 - cos(x) + 1
+    raises(PolynomialError, lambda: roots(eq))
