@@ -5,7 +5,7 @@ The FCodePrinter converts single sympy expressions into single Fortran
 expressions, using the functions defined in the Fortran 77 standard where
 possible. Some useful pointers to Fortran can be found on wikipedia:
 
-http://en.wikipedia.org/wiki/Fortran
+https://en.wikipedia.org/wiki/Fortran
 
 Most of the code below is based on the "Professional Programmer\'s Guide to
 Fortran77" by Clive G. Page:
@@ -19,17 +19,14 @@ it is necessary to make them different for Fortran.
 
 from __future__ import print_function, division
 
+from typing import Dict, Any
+
 from collections import defaultdict
 from itertools import chain
 import string
 
-from sympy.core import S, Add, N, Float, Symbol
-from sympy.core.compatibility import string_types, range
-from sympy.core.function import Function
-from sympy.core.relational import Eq
-from sympy.sets import Range
 from sympy.codegen.ast import (
-    Assignment, Attribute, Declaration, Pointer, Type, value_const,
+    Assignment, Declaration, Pointer, value_const,
     float32, float64, float80, complex64, complex128, int8, int16, int32,
     int64, intc, real, integer,  bool_, complex_
 )
@@ -37,9 +34,13 @@ from sympy.codegen.fnodes import (
     allocatable, isign, dsign, cmplx, merge, literal_dp, elemental, pure,
     intent_in, intent_out, intent_inout
 )
-from sympy.printing.printer import printer_context
-from sympy.printing.codeprinter import CodePrinter, requires
+from sympy.core import S, Add, N, Float, Symbol
+from sympy.core.function import Function
+from sympy.core.relational import Eq
+from sympy.sets import Range
+from sympy.printing.codeprinter import CodePrinter
 from sympy.printing.precedence import precedence, PRECEDENCE
+from sympy.printing.printer import printer_context
 
 
 known_functions = {
@@ -59,7 +60,7 @@ known_functions = {
     "Abs": "abs",
     "conjugate": "conjg",
     "Max": "max",
-    "Min": "min"
+    "Min": "min",
 }
 
 
@@ -103,7 +104,7 @@ class FCodePrinter(CodePrinter):
         'contract': True,
         'standard': 77,
         'name_mangling' : True,
-    }
+    }  # type: Dict[str, Any]
 
     _operators = {
         'and': '.and.',
@@ -117,9 +118,11 @@ class FCodePrinter(CodePrinter):
         '!=': '/=',
     }
 
-    def __init__(self, settings={}):
-        self.mangled_symbols = {}         ## Dict showing mapping of all words
-        self.used_name= []
+    def __init__(self, settings=None):
+        if not settings:
+            settings = {}
+        self.mangled_symbols = {}         # Dict showing mapping of all words
+        self.used_name = []
         self.type_aliases = dict(chain(self.type_aliases.items(),
                                        settings.pop('type_aliases', {}).items()))
         self.type_mappings = dict(chain(self.type_mappings.items(),
@@ -202,7 +205,7 @@ class FCodePrinter(CodePrinter):
         arg, = expr.args
         if arg.is_integer:
             new_expr = merge(0, isign(1, arg), Eq(arg, 0))
-        elif arg.is_complex:
+        elif (arg.is_complex or arg.is_infinite):
             new_expr = merge(cmplx(literal_dp(0), literal_dp(0)), arg/Abs(arg), Eq(Abs(arg), literal_dp(0)))
         else:
             new_expr = merge(literal_dp(0), dsign(literal_dp(1), arg), Eq(arg, literal_dp(0)))
@@ -268,8 +271,8 @@ class FCodePrinter(CodePrinter):
                 pure_imaginary.append(arg)
             else:
                 mixed.append(arg)
-        if len(pure_imaginary) > 0:
-            if len(mixed) > 0:
+        if pure_imaginary:
+            if mixed:
                 PREC = precedence(expr)
                 term = Add(*mixed)
                 t = self._print(term)
@@ -304,6 +307,19 @@ class FCodePrinter(CodePrinter):
         else:
             return CodePrinter._print_Function(self, expr.func(*args))
 
+    def _print_Mod(self, expr):
+        # NOTE : Fortran has the functions mod() and modulo(). modulo() behaves
+        # the same wrt to the sign of the arguments as Python and SymPy's
+        # modulus computations (% and Mod()) but is not available in Fortran 66
+        # or Fortran 77, thus we raise an error.
+        if self._settings['standard'] in [66, 77]:
+            msg = ("Python % operator and SymPy's Mod() function are not "
+                   "supported by Fortran 66 or 77 standards.")
+            raise NotImplementedError(msg)
+        else:
+            x, y = expr.args
+            return "      modulo({}, {})".format(self._print(x), self._print(y))
+
     def _print_ImaginaryUnit(self, expr):
         # purpose: print complex numbers nicely in Fortran.
         return "cmplx(0,1)"
@@ -350,6 +366,13 @@ class FCodePrinter(CodePrinter):
             return "%sd%s" % (printed[:e], printed[e + 1:])
         return "%sd0" % printed
 
+    def _print_Relational(self, expr):
+        lhs_code = self._print(expr.lhs)
+        rhs_code = self._print(expr.rhs)
+        op = expr.rel_op
+        op = op if op not in self._relationals else self._relationals[op]
+        return "{0} {1} {2}".format(lhs_code, op, rhs_code)
+
     def _print_Indexed(self, expr):
         inds = [ self._print(i) for i in expr.indices ]
         return "%s(%s)" % (self._print(expr.base.label), ", ".join(inds))
@@ -366,9 +389,9 @@ class FCodePrinter(CodePrinter):
 
     def _print_sum_(self, sm):
         params = self._print(sm.array)
-        if sm.dim != None:
+        if sm.dim != None: # Must use '!= None', cannot use 'is not None'
             params += ', ' + self._print(sm.dim)
-        if sm.mask != None:
+        if sm.mask != None: # Must use '!= None', cannot use 'is not None'
             params += ', mask=' + self._print(sm.mask)
         return '%s(%s)' % (sm.__class__.__name__.rstrip('_'), params)
 
@@ -410,14 +433,6 @@ class FCodePrinter(CodePrinter):
                 'end do').format(target=target, start=start, stop=stop,
                         step=step, body=body)
 
-    def _print_Equality(self, expr):
-        lhs, rhs = expr.args
-        return ' == '.join(map(lambda arg: self._print(arg), (lhs, rhs)))
-
-    def _print_Unequality(self, expr):
-        lhs, rhs = expr.args
-        return ' /= '.join(map(lambda arg: self._print(arg), (lhs, rhs)))
-
     def _print_Type(self, type_):
         type_ = self.type_aliases.get(type_, type_)
         type_str = self.type_mappings.get(type_, type_.name)
@@ -459,7 +474,7 @@ class FCodePrinter(CodePrinter):
                 alloc=', allocatable' if allocatable in var.attrs else '',
                 s=self._print(var.symbol)
             )
-            if val != None:
+            if val != None: # Must be "!= None", cannot be "is not None"
                 result += ' = %s' % self._print(val)
         else:
             if value_const in var.attrs or val:
@@ -534,7 +549,7 @@ class FCodePrinter(CodePrinter):
                     hunk = line[:pos]
                     line = line[pos:].lstrip()
                     result.append(hunk)
-                    while len(line) > 0:
+                    while line:
                         pos = line.rfind(" ", 0, 66)
                         if pos == -1 or len(line) < 66:
                             pos = 66
@@ -551,7 +566,7 @@ class FCodePrinter(CodePrinter):
                 if line:
                     hunk += trailing
                 result.append(hunk)
-                while len(line) > 0:
+                while line:
                     pos = split_pos_code(line, 65)
                     hunk = line[:pos].rstrip()
                     line = line[pos:].lstrip()
@@ -564,7 +579,7 @@ class FCodePrinter(CodePrinter):
 
     def indent_code(self, code):
         """Accepts a string of code or a list of code lines"""
-        if isinstance(code, string_types):
+        if isinstance(code, str):
             code_lines = self.indent_code(code.splitlines(True))
             return ''.join(code_lines)
 
@@ -652,7 +667,7 @@ class FCodePrinter(CodePrinter):
                 return strm.name
 
     def _print_Print(self, ps):
-        if ps.format_string != None:
+        if ps.format_string != None: # Must be '!= None', cannot be 'is not None'
             fmt = self._print(ps.format_string)
         else:
             fmt = "*"
@@ -699,7 +714,7 @@ class FCodePrinter(CodePrinter):
             "{function_head}\n"
             "end function\n"
             "end interface"
-        ).format(function_head=self._head(entity, fp, *args))
+        ).format(function_head=self._head(entity, fp))
 
     def _print_FunctionDefinition(self, fd):
         if elemental in fd.attrs:
@@ -742,9 +757,9 @@ class FCodePrinter(CodePrinter):
 
     def _print_use(self, use):
         result = 'use %s' % self._print(use.namespace)
-        if use.rename != None:
+        if use.rename != None: # Must be '!= None', cannot be 'is not None'
             result += ', ' + ', '.join([self._print(rnm) for rnm in use.rename])
-        if use.only != None:
+        if use.only != None: # Must be '!= None', cannot be 'is not None'
             result += ', only: ' + ', '.join([self._print(nly) for nly in use.only])
         return result
 
