@@ -10,8 +10,9 @@ using the functions defined in math.h where possible.
 
 from __future__ import print_function, division
 
+from typing import Any, Dict
+
 from sympy.codegen.ast import Assignment
-from sympy.core.compatibility import string_types, range
 from sympy.printing.codeprinter import CodePrinter
 from sympy.printing.precedence import precedence, PRECEDENCE
 from sympy.sets.fancysets import Range
@@ -89,7 +90,7 @@ class RCodePrinter(CodePrinter):
         'dereference': set(),
         'error_on_reserved': False,
         'reserved_word_suffix': '_',
-    }
+    }  # type: Dict[str, Any]
     _operators = {
        'and': '&',
         'or': '|',
@@ -97,7 +98,7 @@ class RCodePrinter(CodePrinter):
     }
 
     _relationals = {
-    }
+    }  # type: Dict[str, str]
 
     def __init__(self, settings={}):
         CodePrinter.__init__(self, settings)
@@ -246,7 +247,7 @@ class RCodePrinter(CodePrinter):
         lhs_code = self._print(expr.lhs)
         rhs_code = self._print(expr.rhs)
         op = expr.rel_op
-        return ("{0} {1} {2}").format(lhs_code, op, rhs_code)
+        return "{0} {1} {2}".format(lhs_code, op, rhs_code)
 
     def _print_sinc(self, expr):
         from sympy.functions.elementary.trigonometric import sin
@@ -277,7 +278,7 @@ class RCodePrinter(CodePrinter):
     def indent_code(self, code):
         """Accepts a string of code or a list of code lines"""
 
-        if isinstance(code, string_types):
+        if isinstance(code, str):
             code_lines = self.indent_code(code.splitlines(True))
             return ''.join(code_lines)
 
