@@ -3,7 +3,7 @@ from sympy import Symbol, exp, log, oo, Rational, I, sin, gamma, loggamma, S, \
 from sympy.functions.elementary.hyperbolic import cosh, coth, sinh, tanh
 from sympy.series.gruntz import compare, mrv, rewrite, mrv_leadterm, gruntz, \
     sign
-from sympy.utilities.pytest import XFAIL, skip, slow
+from sympy.testing.pytest import XFAIL, skip, slow
 
 """
 This test suite is testing the limit algorithm using the bottom up approach.
@@ -36,12 +36,12 @@ def test_gruntz_evaluation():
     assert gruntz(exp(x)*(exp(1/x + exp(-x) + exp(-x**2))
                   - exp(1/x - exp(-exp(x)))), x, oo) == 1
     # 8.3
-    assert gruntz(exp(exp(x - exp(-x))/(1 - 1/x)) - exp(exp(x)), x, oo) == oo
+    assert gruntz(exp(exp(x - exp(-x))/(1 - 1/x)) - exp(exp(x)), x, oo) is oo
     # 8.5
-    assert gruntz(exp(exp(exp(x + exp(-x)))) / exp(exp(exp(x))), x, oo) == oo
+    assert gruntz(exp(exp(exp(x + exp(-x)))) / exp(exp(exp(x))), x, oo) is oo
     # 8.6
     assert gruntz(exp(exp(exp(x))) / exp(exp(exp(x - exp(-exp(x))))),
-                  x, oo) == oo
+                  x, oo) is oo
     # 8.7
     assert gruntz(exp(exp(exp(x))) / exp(exp(exp(x - exp(-exp(exp(x)))))),
                   x, oo) == 1
@@ -53,21 +53,21 @@ def test_gruntz_evaluation():
                   x, oo) == 0
     # 8.10
     assert gruntz((x*log(x)*(log(x*exp(x) - x**2))**2)
-                  / (log(log(x**2 + 2*exp(exp(3*x**3*log(x)))))), x, oo) == S(1)/3
+                  / (log(log(x**2 + 2*exp(exp(3*x**3*log(x)))))), x, oo) == Rational(1, 3)
     # 8.11
     assert gruntz((exp(x*exp(-x)/(exp(-x) + exp(-2*x**2/(x + 1)))) - exp(x))/x,
                   x, oo) == -exp(2)
     # 8.12
     assert gruntz((3**x + 5**x)**(1/x), x, oo) == 5
     # 8.13
-    assert gruntz(x/log(x**(log(x**(log(2)/log(x))))), x, oo) == oo
+    assert gruntz(x/log(x**(log(x**(log(2)/log(x))))), x, oo) is oo
     # 8.14
     assert gruntz(exp(exp(2*log(x**5 + x)*log(log(x))))
-                  / exp(exp(10*log(x)*log(log(x)))), x, oo) == oo
+                  / exp(exp(10*log(x)*log(log(x)))), x, oo) is oo
     # 8.15
-    assert gruntz(exp(exp(S(5)/2*x**(-S(5)/7) + S(21)/8*x**(S(6)/11)
-                          + 2*x**(-8) + S(54)/17*x**(S(49)/45) ))**8
-                  / log(log(-log(S(4)/3*x**(-S(5)/14))))**(S(7)/6), x, oo) == oo
+    assert gruntz(exp(exp(Rational(5, 2)*x**Rational(-5, 7) + Rational(21, 8)*x**Rational(6, 11)
+                          + 2*x**(-8) + Rational(54, 17)*x**Rational(49, 45)))**8
+                  / log(log(-log(Rational(4, 3)*x**Rational(-5, 14))))**Rational(7, 6), x, oo) is oo
     # 8.16
     assert gruntz((exp(4*x*exp(-x)/(1/exp(x) + 1/exp(2*x**2/(x + 1)))) - exp(x))
                   / exp(x)**4, x, oo) == 1
@@ -81,14 +81,14 @@ def test_gruntz_evaluation():
     assert gruntz(exp((log(log(x + exp(log(x)*log(log(x))))))
                   / (log(log(log(exp(x) + x + log(x)))))), x, oo) == E
     # Another
-    assert gruntz(exp(exp(exp(x + exp(-x)))) / exp(exp(x)), x, oo) == oo
+    assert gruntz(exp(exp(exp(x + exp(-x)))) / exp(exp(x)), x, oo) is oo
 
 
 def test_gruntz_evaluation_slow():
     _sskip()
     # 8.4
     assert gruntz(exp(exp(exp(x)/(1 - 1/x)))
-                  - exp(exp(exp(x)/(1 - 1/x - log(x)**(-log(x))))), x, oo) == -oo
+                  - exp(exp(exp(x)/(1 - 1/x - log(x)**(-log(x))))), x, oo) is -oo
     # 8.18
     assert gruntz((exp(exp(-x/(1 + exp(-x))))*exp(-x/(1 + exp(-x/(1 + exp(-x)))))
                    *exp(exp(-x + exp(-x/(1 + exp(-x))))))
@@ -103,22 +103,22 @@ def test_gruntz_eval_special():
                   x, oo) == -2/sqrt(pi)
     assert gruntz(exp(exp(x)) * (exp(sin(1/x + exp(-exp(x)))) - exp(sin(1/x))),
                   x, oo) == 1
-    assert gruntz(exp(x)*(gamma(x + exp(-x)) - gamma(x)), x, oo) == oo
-    assert gruntz(exp(exp(digamma(digamma(x))))/x, x, oo) == exp(-S(1)/2)
-    assert gruntz(exp(exp(digamma(log(x))))/x, x, oo) == exp(-S(1)/2)
-    assert gruntz(digamma(digamma(digamma(x))), x, oo) == oo
-    assert gruntz(loggamma(loggamma(x)), x, oo) == oo
+    assert gruntz(exp(x)*(gamma(x + exp(-x)) - gamma(x)), x, oo) is oo
+    assert gruntz(exp(exp(digamma(digamma(x))))/x, x, oo) == exp(Rational(-1, 2))
+    assert gruntz(exp(exp(digamma(log(x))))/x, x, oo) == exp(Rational(-1, 2))
+    assert gruntz(digamma(digamma(digamma(x))), x, oo) is oo
+    assert gruntz(loggamma(loggamma(x)), x, oo) is oo
     assert gruntz(((gamma(x + 1/gamma(x)) - gamma(x))/log(x) - cos(1/x))
-                  * x*log(x), x, oo) == -S(1)/2
+                  * x*log(x), x, oo) == Rational(-1, 2)
     assert gruntz(x * (gamma(x - 1/gamma(x)) - gamma(x) + log(x)), x, oo) \
-        == S(1)/2
+        == S.Half
     assert gruntz((gamma(x + 1/gamma(x)) - gamma(x)) / log(x), x, oo) == 1
 
 
 def test_gruntz_eval_special_slow():
     _sskip()
     assert gruntz(gamma(x + 1)/sqrt(2*pi)
-                  - exp(-x)*(x**(x + S(1)/2) + x**(x - S(1)/2)/12), x, oo) == oo
+                  - exp(-x)*(x**(x + S.Half) + x**(x - S.Half)/12), x, oo) is oo
     assert gruntz(exp(exp(exp(digamma(digamma(digamma(x))))))/x, x, oo) == 0
 
 
@@ -126,7 +126,7 @@ def test_gruntz_eval_special_slow():
 def test_grunts_eval_special_slow_sometimes_fail():
     _sskip()
     # XXX This sometimes fails!!!
-    assert gruntz(exp(gamma(x - exp(-x))*exp(1/x)) - exp(gamma(x)), x, oo) == oo
+    assert gruntz(exp(gamma(x - exp(-x))*exp(1/x)) - exp(gamma(x)), x, oo) is oo
 
 
 @XFAIL
@@ -143,13 +143,13 @@ def test_gruntz_eval_special_fail():
 
 
 def test_gruntz_hyperbolic():
-    assert gruntz(cosh(x), x, oo) == oo
-    assert gruntz(cosh(x), x, -oo) == oo
-    assert gruntz(sinh(x), x, oo) == oo
-    assert gruntz(sinh(x), x, -oo) == -oo
-    assert gruntz(2*cosh(x)*exp(x), x, oo) == oo
+    assert gruntz(cosh(x), x, oo) is oo
+    assert gruntz(cosh(x), x, -oo) is oo
+    assert gruntz(sinh(x), x, oo) is oo
+    assert gruntz(sinh(x), x, -oo) is -oo
+    assert gruntz(2*cosh(x)*exp(x), x, oo) is oo
     assert gruntz(2*cosh(x)*exp(x), x, -oo) == 1
-    assert gruntz(2*sinh(x)*exp(x), x, oo) == oo
+    assert gruntz(2*sinh(x)*exp(x), x, oo) is oo
     assert gruntz(2*sinh(x)*exp(x), x, -oo) == -1
     assert gruntz(tanh(x), x, oo) == 1
     assert gruntz(tanh(x), x, -oo) == -1
@@ -218,6 +218,8 @@ def test_sign1():
     assert sign(-3 - 1/x, x) == -1
     assert sign(sin(1/x), x) == 1
     assert sign((x**Integer(2)), x) == 1
+    assert sign(x**2, x) == 1
+    assert sign(x**5, x) == 1
 
 
 def test_sign2():
@@ -340,18 +342,18 @@ def test_mrv_leadterm3():
 
 
 def test_limit1():
-    assert gruntz(x, x, oo) == oo
-    assert gruntz(x, x, -oo) == -oo
-    assert gruntz(-x, x, oo) == -oo
-    assert gruntz(x**2, x, -oo) == oo
-    assert gruntz(-x**2, x, oo) == -oo
+    assert gruntz(x, x, oo) is oo
+    assert gruntz(x, x, -oo) is -oo
+    assert gruntz(-x, x, oo) is -oo
+    assert gruntz(x**2, x, -oo) is oo
+    assert gruntz(-x**2, x, oo) is -oo
     assert gruntz(x*log(x), x, 0, dir="+") == 0
     assert gruntz(1/x, x, oo) == 0
-    assert gruntz(exp(x), x, oo) == oo
-    assert gruntz(-exp(x), x, oo) == -oo
-    assert gruntz(exp(x)/x, x, oo) == oo
+    assert gruntz(exp(x), x, oo) is oo
+    assert gruntz(-exp(x), x, oo) is -oo
+    assert gruntz(exp(x)/x, x, oo) is oo
     assert gruntz(1/x - exp(-x), x, oo) == 0
-    assert gruntz(x + 1/x, x, oo) == oo
+    assert gruntz(x + 1/x, x, oo) is oo
 
 
 def test_limit2():
@@ -359,9 +361,9 @@ def test_limit2():
     assert gruntz((exp(x) - 1)/x, x, 0) == 1
     assert gruntz(1 + 1/x, x, oo) == 1
     assert gruntz(-exp(1/x), x, oo) == -1
-    assert gruntz(x + exp(-x), x, oo) == oo
-    assert gruntz(x + exp(-x**2), x, oo) == oo
-    assert gruntz(x + exp(-exp(x)), x, oo) == oo
+    assert gruntz(x + exp(-x), x, oo) is oo
+    assert gruntz(x + exp(-x**2), x, oo) is oo
+    assert gruntz(x + exp(-exp(x)), x, oo) is oo
     assert gruntz(13 + 1/x - exp(-x), x, oo) == 13
 
 
@@ -405,22 +407,22 @@ def test_issue_4814():
 def test_intractable():
     assert gruntz(1/gamma(x), x, oo) == 0
     assert gruntz(1/loggamma(x), x, oo) == 0
-    assert gruntz(gamma(x)/loggamma(x), x, oo) == oo
-    assert gruntz(exp(gamma(x))/gamma(x), x, oo) == oo
+    assert gruntz(gamma(x)/loggamma(x), x, oo) is oo
+    assert gruntz(exp(gamma(x))/gamma(x), x, oo) is oo
     assert gruntz(gamma(x), x, 3) == 2
-    assert gruntz(gamma(S(1)/7 + 1/x), x, oo) == gamma(S(1)/7)
+    assert gruntz(gamma(Rational(1, 7) + 1/x), x, oo) == gamma(Rational(1, 7))
     assert gruntz(log(x**x)/log(gamma(x)), x, oo) == 1
-    assert gruntz(log(gamma(gamma(x)))/exp(x), x, oo) == oo
+    assert gruntz(log(gamma(gamma(x)))/exp(x), x, oo) is oo
 
 
 def test_aseries_trig():
     assert cancel(gruntz(1/log(atan(x)), x, oo)
-           - 1/(log(pi) + log(S(1)/2))) == 0
-    assert gruntz(1/acot(x), x, -oo) == -oo
+           - 1/(log(pi) + log(S.Half))) == 0
+    assert gruntz(1/acot(x), x, -oo) is -oo
 
 
 def test_exp_log_series():
-    assert gruntz(x/log(log(x*exp(x))), x, oo) == oo
+    assert gruntz(x/log(log(x*exp(x))), x, oo) is oo
 
 
 def test_issue_3644():
