@@ -11,21 +11,20 @@ from sympy.simplify.hyperexpand import (ShiftA, ShiftB, UnShiftA, UnShiftB,
                        build_hypergeometric_formula)
 from sympy import hyper, I, S, meijerg, Piecewise, Tuple, Sum, binomial, Expr
 from sympy.abc import z, a, b, c
-from sympy.utilities.pytest import XFAIL, raises, slow, ON_TRAVIS, skip
-from sympy.utilities.randtest import verify_numerically as tn
-from sympy.core.compatibility import range
+from sympy.testing.pytest import XFAIL, raises, slow, ON_TRAVIS, skip
+from sympy.testing.randtest import verify_numerically as tn
 
 from sympy import (cos, sin, log, exp, asin, lowergamma, atanh, besseli,
                    gamma, sqrt, pi, erf, exp_polar, Rational)
 
 
 def test_branch_bug():
-    assert hyperexpand(hyper((-S(1)/3, S(1)/2), (S(2)/3, S(3)/2), -z)) == \
+    assert hyperexpand(hyper((Rational(-1, 3), S.Half), (Rational(2, 3), Rational(3, 2)), -z)) == \
         -z**S('1/3')*lowergamma(exp_polar(I*pi)/3, z)/5 \
         + sqrt(pi)*erf(sqrt(z))/(5*sqrt(z))
-    assert hyperexpand(meijerg([S(7)/6, 1], [], [S(2)/3], [S(1)/6, 0], z)) == \
+    assert hyperexpand(meijerg([Rational(7, 6), 1], [], [Rational(2, 3)], [Rational(1, 6), 0], z)) == \
         2*z**S('2/3')*(2*sqrt(pi)*erf(sqrt(z))/sqrt(z) - 2*lowergamma(
-                       S(2)/3, z)/z**S('2/3'))*gamma(S(2)/3)/gamma(S(5)/3)
+                       Rational(2, 3), z)/z**S('2/3'))*gamma(Rational(2, 3))/gamma(Rational(5, 3))
 
 
 def test_hyperexpand():
@@ -52,8 +51,8 @@ def can_do(ap, bq, numerical=True, div=1, lowerplane=False):
     randsyms = r.free_symbols - {z}
     while randsyms:
         # Only randomly generated parameters are checked.
-        for n, a in enumerate(randsyms):
-            repl[a] = randcplx(n)/div
+        for n, ai in enumerate(randsyms):
+            repl[ai] = randcplx(n)/div
         if not any([b.is_Integer and b <= 0 for b in Tuple(*bq).subs(repl)]):
             break
     [a, b, c, d] = [2, -1, 3, 1]
@@ -68,30 +67,30 @@ def can_do(ap, bq, numerical=True, div=1, lowerplane=False):
 def test_roach():
     # Kelly B. Roach.  Meijer G Function Representations.
     # Section "Gallery"
-    assert can_do([S(1)/2], [S(9)/2])
-    assert can_do([], [1, S(5)/2, 4])
-    assert can_do([-S.Half, 1, 2], [3, 4])
-    assert can_do([S(1)/3], [-S(2)/3, -S(1)/2, S(1)/2, 1])
-    assert can_do([-S(3)/2, -S(1)/2], [-S(5)/2, 1])
-    assert can_do([-S(3)/2, ], [-S(1)/2, S(1)/2])  # shine-integral
-    assert can_do([-S(3)/2, -S(1)/2], [2])  # elliptic integrals
+    assert can_do([S.Half], [Rational(9, 2)])
+    assert can_do([], [1, Rational(5, 2), 4])
+    assert can_do([Rational(-1, 2), 1, 2], [3, 4])
+    assert can_do([Rational(1, 3)], [Rational(-2, 3), Rational(-1, 2), S.Half, 1])
+    assert can_do([Rational(-3, 2), Rational(-1, 2)], [Rational(-5, 2), 1])
+    assert can_do([Rational(-3, 2), ], [Rational(-1, 2), S.Half])  # shine-integral
+    assert can_do([Rational(-3, 2), Rational(-1, 2)], [2])  # elliptic integrals
 
 
 @XFAIL
 def test_roach_fail():
-    assert can_do([-S(1)/2, 1], [S(1)/4, S(1)/2, S(3)/4])  # PFDD
-    assert can_do([S(3)/2], [S(5)/2, 5])  # struve function
-    assert can_do([-S(1)/2, S(1)/2, 1], [S(3)/2, S(5)/2])  # polylog, pfdd
-    assert can_do([1, 2, 3], [S(1)/2, 4])  # XXX ?
-    assert can_do([S(1)/2], [-S(1)/3, -S(1)/2, -S(2)/3])  # PFDD ?
+    assert can_do([Rational(-1, 2), 1], [Rational(1, 4), S.Half, Rational(3, 4)])  # PFDD
+    assert can_do([Rational(3, 2)], [Rational(5, 2), 5])  # struve function
+    assert can_do([Rational(-1, 2), S.Half, 1], [Rational(3, 2), Rational(5, 2)])  # polylog, pfdd
+    assert can_do([1, 2, 3], [S.Half, 4])  # XXX ?
+    assert can_do([S.Half], [Rational(-1, 3), Rational(-1, 2), Rational(-2, 3)])  # PFDD ?
 
 # For the long table tests, see end of file
 
 
 def test_polynomial():
     from sympy import oo
-    assert hyperexpand(hyper([], [-1], z)) == oo
-    assert hyperexpand(hyper([-2], [-1], z)) == oo
+    assert hyperexpand(hyper([], [-1], z)) is oo
+    assert hyperexpand(hyper([-2], [-1], z)) is oo
     assert hyperexpand(hyper([0, 0], [-1], z)) == 1
     assert can_do([-5, -2, randcplx(), randcplx()], [-10, randcplx()])
     assert hyperexpand(hyper((-1, 1), (-2,), z)) == 1 + z/2
@@ -101,33 +100,33 @@ def test_hyperexpand_bases():
     assert hyperexpand(hyper([2], [a], z)) == \
         a + z**(-a + 1)*(-a**2 + 3*a + z*(a - 1) - 2)*exp(z)* \
         lowergamma(a - 1, z) - 1
-    # TODO [a+1, a-S.Half], [2*a]
+    # TODO [a+1, aRational(-1, 2)], [2*a]
     assert hyperexpand(hyper([1, 2], [3], z)) == -2/z - 2*log(-z + 1)/z**2
-    assert hyperexpand(hyper([S.Half, 2], [S(3)/2], z)) == \
+    assert hyperexpand(hyper([S.Half, 2], [Rational(3, 2)], z)) == \
         -1/(2*z - 2) + atanh(sqrt(z))/sqrt(z)/2
-    assert hyperexpand(hyper([S(1)/2, S(1)/2], [S(5)/2], z)) == \
+    assert hyperexpand(hyper([S.Half, S.Half], [Rational(5, 2)], z)) == \
         (-3*z + 3)/4/(z*sqrt(-z + 1)) \
-        + (6*z - 3)*asin(sqrt(z))/(4*z**(S(3)/2))
-    assert hyperexpand(hyper([1, 2], [S(3)/2], z)) == -1/(2*z - 2) \
+        + (6*z - 3)*asin(sqrt(z))/(4*z**Rational(3, 2))
+    assert hyperexpand(hyper([1, 2], [Rational(3, 2)], z)) == -1/(2*z - 2) \
         - asin(sqrt(z))/(sqrt(z)*(2*z - 2)*sqrt(-z + 1))
-    assert hyperexpand(hyper([-S.Half - 1, 1, 2], [S.Half, 3], z)) == \
-        sqrt(z)*(6*z/7 - S(6)/5)*atanh(sqrt(z)) \
+    assert hyperexpand(hyper([Rational(-1, 2) - 1, 1, 2], [S.Half, 3], z)) == \
+        sqrt(z)*(z*Rational(6, 7) - Rational(6, 5))*atanh(sqrt(z)) \
         + (-30*z**2 + 32*z - 6)/35/z - 6*log(-z + 1)/(35*z**2)
     assert hyperexpand(hyper([1 + S.Half, 1, 1], [2, 2], z)) == \
-        -4*log(sqrt(-z + 1)/2 + S(1)/2)/z
+        -4*log(sqrt(-z + 1)/2 + S.Half)/z
     # TODO hyperexpand(hyper([a], [2*a + 1], z))
-    # TODO [S.Half, a], [S(3)/2, a+1]
+    # TODO [S.Half, a], [Rational(3, 2), a+1]
     assert hyperexpand(hyper([2], [b, 1], z)) == \
-        z**(-b/2 + S(1)/2)*besseli(b - 1, 2*sqrt(z))*gamma(b) \
+        z**(-b/2 + S.Half)*besseli(b - 1, 2*sqrt(z))*gamma(b) \
         + z**(-b/2 + 1)*besseli(b, 2*sqrt(z))*gamma(b)
     # TODO [a], [a - S.Half, 2*a]
 
 
 def test_hyperexpand_parametric():
-    assert hyperexpand(hyper([a, S(1)/2 + a], [S(1)/2], z)) \
+    assert hyperexpand(hyper([a, S.Half + a], [S.Half], z)) \
         == (1 + sqrt(z))**(-2*a)/2 + (1 - sqrt(z))**(-2*a)/2
-    assert hyperexpand(hyper([a, -S(1)/2 + a], [2*a], z)) \
-        == 2**(2*a - 1)*((-z + 1)**(S(1)/2) + 1)**(-2*a + 1)
+    assert hyperexpand(hyper([a, Rational(-1, 2) + a], [2*a], z)) \
+        == 2**(2*a - 1)*((-z + 1)**S.Half + 1)**(-2*a + 1)
 
 
 def test_shifted_sum():
@@ -340,8 +339,8 @@ def can_do_meijer(a1, a2, b1, b2, numeric=True):
         return True
 
     repl = {}
-    for n, a in enumerate(meijerg(a1, a2, b1, b2, z).free_symbols - {z}):
-        repl[a] = randcplx(n)
+    for n, ai in enumerate(meijerg(a1, a2, b1, b2, z).free_symbols - {z}):
+        repl[ai] = randcplx(n)
     return tn(meijerg(a1, a2, b1, b2, z).subs(repl), r.subs(repl), z)
 
 
@@ -355,9 +354,9 @@ def test_meijerg_expand():
         log(z + 1)
     assert hyperexpand(meijerg([[1, 1], []], [[1], [1]], z)) == \
         z/(z + 1)
-    assert hyperexpand(meijerg([[], []], [[S(1)/2], [0]], (z/2)**2)) \
+    assert hyperexpand(meijerg([[], []], [[S.Half], [0]], (z/2)**2)) \
         == sin(z)/sqrt(pi)
-    assert hyperexpand(meijerg([[], []], [[0], [S(1)/2]], (z/2)**2)) \
+    assert hyperexpand(meijerg([[], []], [[0], [S.Half]], (z/2)**2)) \
         == cos(z)/sqrt(pi)
     assert can_do_meijer([], [a], [a - 1, a - S.Half], [])
     assert can_do_meijer([], [], [a/2], [-a/2], False)  # branches...
@@ -398,7 +397,7 @@ def test_meijerg_expand():
 
     # Test that the simplest possible answer is returned:
     assert gammasimp(simplify(hyperexpand(
-        meijerg([1], [1 - a], [-a/2, -a/2 + S(1)/2], [], 1/z)))) == \
+        meijerg([1], [1 - a], [-a/2, -a/2 + S.Half], [], 1/z)))) == \
         -2*sqrt(pi)*(sqrt(z + 1) + 1)**a/a
 
     # Test that hyper is returned
@@ -406,7 +405,7 @@ def test_meijerg_expand():
         (a,), (a + 1, a + 1), z*exp_polar(I*pi))*z**a*gamma(a)/gamma(a + 1)**2
 
     # Test place option
-    f = meijerg(((0, 1), ()), ((S(1)/2,), (0,)), z**2)
+    f = meijerg(((0, 1), ()), ((S.Half,), (0,)), z**2)
     assert hyperexpand(f) == sqrt(pi)/sqrt(1 + z**(-2))
     assert hyperexpand(f, place=0) == sqrt(pi)*z/sqrt(z**2 + 1)
 
@@ -421,12 +420,12 @@ def test_meijerg_lookup():
     assert can_do_meijer([a], [], [b + 2, a], [])
     assert can_do_meijer([a], [], [b - 2, a], [])
 
-    assert hyperexpand(meijerg([a], [], [a, a, a - S(1)/2], [], z)) == \
-        -sqrt(pi)*z**(a - S(1)/2)*(2*cos(2*sqrt(z))*(Si(2*sqrt(z)) - pi/2)
+    assert hyperexpand(meijerg([a], [], [a, a, a - S.Half], [], z)) == \
+        -sqrt(pi)*z**(a - S.Half)*(2*cos(2*sqrt(z))*(Si(2*sqrt(z)) - pi/2)
                                    - 2*sin(2*sqrt(z))*Ci(2*sqrt(z))) == \
-        hyperexpand(meijerg([a], [], [a, a - S(1)/2, a], [], z)) == \
-        hyperexpand(meijerg([a], [], [a - S(1)/2, a, a], [], z))
-    assert can_do_meijer([a - 1], [], [a + 2, a - S(3)/2, a + 1], [])
+        hyperexpand(meijerg([a], [], [a, a - S.Half, a], [], z)) == \
+        hyperexpand(meijerg([a], [], [a - S.Half, a, a], [], z))
+    assert can_do_meijer([a - 1], [], [a + 2, a - Rational(3, 2), a + 1], [])
 
 
 @XFAIL
@@ -439,8 +438,8 @@ def test_meijerg_expand_fail():
     assert can_do_meijer([], [], [0, S.Half], [a, -a])
     assert can_do_meijer([], [], [3*a - S.Half, a, -a - S.Half], [a - S.Half])
     assert can_do_meijer([], [], [0, a - S.Half, -a - S.Half], [S.Half])
-    assert can_do_meijer([], [], [a, b + S(1)/2, b], [2*b - a])
-    assert can_do_meijer([], [], [a, b + S(1)/2, b, 2*b - a])
+    assert can_do_meijer([], [], [a, b + S.Half, b], [2*b - a])
+    assert can_do_meijer([], [], [a, b + S.Half, b, 2*b - a])
     assert can_do_meijer([S.Half], [], [-a, a], [0])
 
 
@@ -534,14 +533,14 @@ def test_meijerg_confluence():
 
     assert t(meijerg([], [1, 1], [0, 0], [], z), -log(z), 0)
     assert t(meijerg(
-        [], [3, 1], [0, 0], [], z), -z**2/4 + z - log(z)/2 - S(3)/4, 0)
+        [], [3, 1], [0, 0], [], z), -z**2/4 + z - log(z)/2 - Rational(3, 4), 0)
     assert t(meijerg([], [3, 1], [-1, 0], [], z),
-             z**2/12 - z/2 + log(z)/2 + S(1)/4 + 1/(6*z), 0)
+             z**2/12 - z/2 + log(z)/2 + Rational(1, 4) + 1/(6*z), 0)
     assert t(meijerg([], [1, 1, 1, 1], [0, 0, 0, 0], [], z), -log(z)**3/6, 0)
     assert t(meijerg([1, 1], [], [], [0, 0], z), 0, -log(1/z))
     assert t(meijerg([1, 1], [2, 2], [1, 1], [0, 0], z),
              -z*log(z) + 2*z, -log(1/z) + 2)
-    assert t(meijerg([S(1)/2], [1, 1], [0, 0], [S(3)/2], z), log(z)/2 - 1, 0)
+    assert t(meijerg([S.Half], [1, 1], [0, 0], [Rational(3, 2)], z), log(z)/2 - 1, 0)
 
     def u(an, ap, bm, bq):
         m = meijerg(an, ap, bm, bq, z)
@@ -558,9 +557,9 @@ def test_meijerg_confluence():
 def test_meijerg_with_Floats():
     # see issue #10681
     from sympy import RR
-    f = meijerg(((3.0, 1), ()), ((S(3)/2,), (0,)), z)
+    f = meijerg(((3.0, 1), ()), ((Rational(3, 2),), (0,)), z)
     a = -2.3632718012073
-    g = a*z**(S(3)/2)*hyper((-0.5, S(3)/2), (S(5)/2,), z*exp_polar(I*pi))
+    g = a*z**Rational(3, 2)*hyper((-0.5, Rational(3, 2)), (Rational(5, 2),), z*exp_polar(I*pi))
     assert RR.almosteq((hyperexpand(f)/g).n(), 1.0, 1e-12)
 
 
@@ -584,7 +583,7 @@ def test_lerchphi():
     assert hyperexpand(z*hyper([1, 1, 1], [2, 2], z)) == polylog(2, z)
     assert hyperexpand(z*hyper([1, 1, 1, 1], [2, 2, 2], z)) == polylog(3, z)
 
-    assert hyperexpand(hyper([1, a, 1 + S(1)/2], [a + 1, S(1)/2], z)) == \
+    assert hyperexpand(hyper([1, a, 1 + S.Half], [a + 1, S.Half], z)) == \
         -2*a/(z - 1) + (-2*a**2 + a)*lerchphi(z, 1, a)
 
     # Now numerical tests. These make sure reductions etc are carried out
@@ -604,9 +603,9 @@ def test_lerchphi():
 
     # test a bug
     from sympy import Abs
-    assert hyperexpand(hyper([S(1)/2, S(1)/2, S(1)/2, 1],
-                             [S(3)/2, S(3)/2, S(3)/2], S(1)/4)) == \
-        Abs(-polylog(3, exp_polar(I*pi)/2) + polylog(3, S(1)/2))
+    assert hyperexpand(hyper([S.Half, S.Half, S.Half, 1],
+                             [Rational(3, 2), Rational(3, 2), Rational(3, 2)], Rational(1, 4))) == \
+        Abs(-polylog(3, exp_polar(I*pi)/2) + polylog(3, S.Half))
 
 
 def test_partial_simp():
@@ -661,8 +660,8 @@ def test_Mod1_behavior():
 
 @slow
 def test_prudnikov_misc():
-    assert can_do([1, (3 + I)/2, (3 - I)/2], [S(3)/2, 2])
-    assert can_do([S.Half, a - 1], [S(3)/2, a + 1], lowerplane=True)
+    assert can_do([1, (3 + I)/2, (3 - I)/2], [Rational(3, 2), 2])
+    assert can_do([S.Half, a - 1], [Rational(3, 2), a + 1], lowerplane=True)
     assert can_do([], [b + 1])
     assert can_do([a], [a - 1, b + 1])
 
@@ -681,7 +680,6 @@ def test_prudnikov_misc():
     assert can_do([a], [a + 1], lowerplane=True)  # lowergamma
 
 
-@slow
 def test_prudnikov_1():
     # A. P. Prudnikov, Yu. A. Brychkov and O. I. Marichev (1990).
     # Integrals and Series: More Special Functions, Vol. 3,.
@@ -690,15 +688,15 @@ def test_prudnikov_1():
     # 7.3.1
     assert can_do([a, -a], [S.Half])
     assert can_do([a, 1 - a], [S.Half])
-    assert can_do([a, 1 - a], [S(3)/2])
+    assert can_do([a, 1 - a], [Rational(3, 2)])
     assert can_do([a, 2 - a], [S.Half])
-    assert can_do([a, 2 - a], [S(3)/2])
-    assert can_do([a, 2 - a], [S(3)/2])
-    assert can_do([a, a + S(1)/2], [2*a - 1])
-    assert can_do([a, a + S(1)/2], [2*a])
-    assert can_do([a, a + S(1)/2], [2*a + 1])
-    assert can_do([a, a + S(1)/2], [S(1)/2])
-    assert can_do([a, a + S(1)/2], [S(3)/2])
+    assert can_do([a, 2 - a], [Rational(3, 2)])
+    assert can_do([a, 2 - a], [Rational(3, 2)])
+    assert can_do([a, a + S.Half], [2*a - 1])
+    assert can_do([a, a + S.Half], [2*a])
+    assert can_do([a, a + S.Half], [2*a + 1])
+    assert can_do([a, a + S.Half], [S.Half])
+    assert can_do([a, a + S.Half], [Rational(3, 2)])
     assert can_do([a, a/2 + 1], [a/2])
     assert can_do([1, b], [2])
     assert can_do([1, b], [b + 1], numerical=False)  # Lerch Phi
@@ -734,11 +732,11 @@ def test_prudnikov_3():
         skip("Too slow for travis.")
 
     h = S.Half
-    assert can_do([S(1)/4, S(3)/4], [h])
-    assert can_do([S(1)/4, S(3)/4], [3*h])
-    assert can_do([S(1)/3, S(2)/3], [3*h])
-    assert can_do([S(3)/4, S(5)/4], [h])
-    assert can_do([S(3)/4, S(5)/4], [3*h])
+    assert can_do([Rational(1, 4), Rational(3, 4)], [h])
+    assert can_do([Rational(1, 4), Rational(3, 4)], [3*h])
+    assert can_do([Rational(1, 3), Rational(2, 3)], [3*h])
+    assert can_do([Rational(3, 4), Rational(5, 4)], [h])
+    assert can_do([Rational(3, 4), Rational(5, 4)], [3*h])
 
     for p in [1, 2, 3, 4]:
         for n in [-h, h, 1, 3*h, 2, 5*h, 3, 7*h, 4, 9*h]:
@@ -825,26 +823,26 @@ def test_prudnikov_8():
     h = S.Half
 
     # 7.12.2
-    for a in [1, 2, 3]:
-        for b in [1, 2, 3]:
-            for c in range(1, a + 1):
-                for d in [h, 1, 3*h, 2, 5*h, 3]:
-                    assert can_do([a, b], [c, d])
-        for b in [3*h, 5*h]:
-            for c in [h, 1, 3*h, 2, 5*h, 3]:
-                for d in [1, 2, 3]:
-                    assert can_do([a, b], [c, d])
+    for ai in [1, 2, 3]:
+        for bi in [1, 2, 3]:
+            for ci in range(1, ai + 1):
+                for di in [h, 1, 3*h, 2, 5*h, 3]:
+                    assert can_do([ai, bi], [ci, di])
+        for bi in [3*h, 5*h]:
+            for ci in [h, 1, 3*h, 2, 5*h, 3]:
+                for di in [1, 2, 3]:
+                    assert can_do([ai, bi], [ci, di])
 
-    for a in [-h, h, 3*h, 5*h]:
-        for b in [1, 2, 3]:
-            for c in [h, 1, 3*h, 2, 5*h, 3]:
-                for d in [1, 2, 3]:
-                    assert can_do([a, b], [c, d])
-        for b in [h, 3*h, 5*h]:
-            for c in [h, 3*h, 5*h, 3]:
-                for d in [h, 1, 3*h, 2, 5*h, 3]:
-                    if c <= b:
-                        assert can_do([a, b], [c, d])
+    for ai in [-h, h, 3*h, 5*h]:
+        for bi in [1, 2, 3]:
+            for ci in [h, 1, 3*h, 2, 5*h, 3]:
+                for di in [1, 2, 3]:
+                    assert can_do([ai, bi], [ci, di])
+        for bi in [h, 3*h, 5*h]:
+            for ci in [h, 3*h, 5*h, 3]:
+                for di in [h, 1, 3*h, 2, 5*h, 3]:
+                    if ci <= bi:
+                        assert can_do([ai, bi], [ci, di])
 
 
 def test_prudnikov_9():
@@ -877,40 +875,39 @@ def test_prudnikov_10():
     for m in [h, 1, 2, 5*h, 3, 7*h, 4]:
         assert can_do([7*h], [5*h, m])
 
-    assert can_do([-S(1)/2], [S(1)/2, S(1)/2])  # shine-integral shi
+    assert can_do([Rational(-1, 2)], [S.Half, S.Half])  # shine-integral shi
 
 
 def test_prudnikov_11():
     # 7.15
     assert can_do([a, a + S.Half], [2*a, b, 2*a - b])
-    assert can_do([a, a + S.Half], [S(3)/2, 2*a, 2*a - S(1)/2])
+    assert can_do([a, a + S.Half], [Rational(3, 2), 2*a, 2*a - S.Half])
 
-    assert can_do([S(1)/4, S(3)/4], [S(1)/2, S(1)/2, 1])
-    assert can_do([S(5)/4, S(3)/4], [S(3)/2, S(1)/2, 2])
-    assert can_do([S(5)/4, S(3)/4], [S(3)/2, S(3)/2, 1])
-    assert can_do([S(5)/4, S(7)/4], [S(3)/2, S(5)/2, 2])
+    assert can_do([Rational(1, 4), Rational(3, 4)], [S.Half, S.Half, 1])
+    assert can_do([Rational(5, 4), Rational(3, 4)], [Rational(3, 2), S.Half, 2])
+    assert can_do([Rational(5, 4), Rational(3, 4)], [Rational(3, 2), Rational(3, 2), 1])
+    assert can_do([Rational(5, 4), Rational(7, 4)], [Rational(3, 2), Rational(5, 2), 2])
 
-    assert can_do([1, 1], [S(3)/2, 2, 2])  # cosh-integral chi
+    assert can_do([1, 1], [Rational(3, 2), 2, 2])  # cosh-integral chi
 
 
-@slow
 def test_prudnikov_12():
     # 7.16
     assert can_do(
         [], [a, a + S.Half, 2*a], False)  # branches only agree for some z!
     assert can_do([], [a, a + S.Half, 2*a + 1], False)  # dito
     assert can_do([], [S.Half, a, a + S.Half])
-    assert can_do([], [S(3)/2, a, a + S.Half])
+    assert can_do([], [Rational(3, 2), a, a + S.Half])
 
-    assert can_do([], [S(1)/4, S(1)/2, S(3)/4])
-    assert can_do([], [S(1)/2, S(1)/2, 1])
-    assert can_do([], [S(1)/2, S(3)/2, 1])
-    assert can_do([], [S(3)/4, S(3)/2, S(5)/4])
-    assert can_do([], [1, 1, S(3)/2])
-    assert can_do([], [1, 2, S(3)/2])
-    assert can_do([], [1, S(3)/2, S(3)/2])
-    assert can_do([], [S(5)/4, S(3)/2, S(7)/4])
-    assert can_do([], [2, S(3)/2, S(3)/2])
+    assert can_do([], [Rational(1, 4), S.Half, Rational(3, 4)])
+    assert can_do([], [S.Half, S.Half, 1])
+    assert can_do([], [S.Half, Rational(3, 2), 1])
+    assert can_do([], [Rational(3, 4), Rational(3, 2), Rational(5, 4)])
+    assert can_do([], [1, 1, Rational(3, 2)])
+    assert can_do([], [1, 2, Rational(3, 2)])
+    assert can_do([], [1, Rational(3, 2), Rational(3, 2)])
+    assert can_do([], [Rational(5, 4), Rational(3, 2), Rational(7, 4)])
+    assert can_do([], [2, Rational(3, 2), Rational(3, 2)])
 
 
 @slow
@@ -933,25 +930,25 @@ def test_prudnikov_fail_2F1():
     # Legendre functions:
     assert can_do([a, b], [a + b + S.Half])
     assert can_do([a, b], [a + b - S.Half])
-    assert can_do([a, b], [a + b + S(3)/2])
+    assert can_do([a, b], [a + b + Rational(3, 2)])
     assert can_do([a, b], [(a + b + 1)/2])
     assert can_do([a, b], [(a + b)/2 + 1])
     assert can_do([a, b], [a - b + 1])
     assert can_do([a, b], [a - b + 2])
     assert can_do([a, b], [2*b])
     assert can_do([a, b], [S.Half])
-    assert can_do([a, b], [S(3)/2])
+    assert can_do([a, b], [Rational(3, 2)])
     assert can_do([a, 1 - a], [c])
     assert can_do([a, 2 - a], [c])
     assert can_do([a, 3 - a], [c])
-    assert can_do([a, a + S(1)/2], [c])
+    assert can_do([a, a + S.Half], [c])
     assert can_do([1, b], [c])
-    assert can_do([1, b], [S(3)/2])
+    assert can_do([1, b], [Rational(3, 2)])
 
-    assert can_do([S(1)/4, S(3)/4], [1])
+    assert can_do([Rational(1, 4), Rational(3, 4)], [1])
 
     # PFDD
-    o = S(1)
+    o = S.One
     assert can_do([o/8, 1], [o/8*9])
     assert can_do([o/6, 1], [o/6*7])
     assert can_do([o/6, 1], [o/6*13])
@@ -980,26 +977,26 @@ def test_prudnikov_fail_2F1():
 
 @XFAIL
 def test_prudnikov_fail_3F2():
-    assert can_do([a, a + S(1)/3, a + S(2)/3], [S(1)/3, S(2)/3])
-    assert can_do([a, a + S(1)/3, a + S(2)/3], [S(2)/3, S(4)/3])
-    assert can_do([a, a + S(1)/3, a + S(2)/3], [S(4)/3, S(5)/3])
+    assert can_do([a, a + Rational(1, 3), a + Rational(2, 3)], [Rational(1, 3), Rational(2, 3)])
+    assert can_do([a, a + Rational(1, 3), a + Rational(2, 3)], [Rational(2, 3), Rational(4, 3)])
+    assert can_do([a, a + Rational(1, 3), a + Rational(2, 3)], [Rational(4, 3), Rational(5, 3)])
 
     # page 421
-    assert can_do([a, a + S(1)/3, a + S(2)/3], [3*a/2, (3*a + 1)/2])
+    assert can_do([a, a + Rational(1, 3), a + Rational(2, 3)], [a*Rational(3, 2), (3*a + 1)/2])
 
     # pages 422 ...
-    assert can_do([-S.Half, S.Half, S.Half], [1, 1])  # elliptic integrals
-    assert can_do([-S.Half, S.Half, 1], [S(3)/2, S(3)/2])
+    assert can_do([Rational(-1, 2), S.Half, S.Half], [1, 1])  # elliptic integrals
+    assert can_do([Rational(-1, 2), S.Half, 1], [Rational(3, 2), Rational(3, 2)])
     # TODO LOTS more
 
     # PFDD
-    assert can_do([S(1)/8, S(3)/8, 1], [S(9)/8, S(11)/8])
-    assert can_do([S(1)/8, S(5)/8, 1], [S(9)/8, S(13)/8])
-    assert can_do([S(1)/8, S(7)/8, 1], [S(9)/8, S(15)/8])
-    assert can_do([S(1)/6, S(1)/3, 1], [S(7)/6, S(4)/3])
-    assert can_do([S(1)/6, S(2)/3, 1], [S(7)/6, S(5)/3])
-    assert can_do([S(1)/6, S(2)/3, 1], [S(5)/3, S(13)/6])
-    assert can_do([S.Half, 1, 1], [S(1)/4, S(3)/4])
+    assert can_do([Rational(1, 8), Rational(3, 8), 1], [Rational(9, 8), Rational(11, 8)])
+    assert can_do([Rational(1, 8), Rational(5, 8), 1], [Rational(9, 8), Rational(13, 8)])
+    assert can_do([Rational(1, 8), Rational(7, 8), 1], [Rational(9, 8), Rational(15, 8)])
+    assert can_do([Rational(1, 6), Rational(1, 3), 1], [Rational(7, 6), Rational(4, 3)])
+    assert can_do([Rational(1, 6), Rational(2, 3), 1], [Rational(7, 6), Rational(5, 3)])
+    assert can_do([Rational(1, 6), Rational(2, 3), 1], [Rational(5, 3), Rational(13, 6)])
+    assert can_do([S.Half, 1, 1], [Rational(1, 4), Rational(3, 4)])
     # LOTS more
 
 
@@ -1011,23 +1008,23 @@ def test_prudnikov_fail_other():
     assert can_do([1, a], [b, 1 - 2*a + b])  # ???
 
     # 7.14.2
-    assert can_do([-S(1)/2], [S(1)/2, 1])  # struve
-    assert can_do([1], [S(1)/2, S(1)/2])  # struve
-    assert can_do([S(1)/4], [S(1)/2, S(5)/4])  # PFDD
-    assert can_do([S(3)/4], [S(3)/2, S(7)/4])  # PFDD
-    assert can_do([1], [S(1)/4, S(3)/4])  # PFDD
-    assert can_do([1], [S(3)/4, S(5)/4])  # PFDD
-    assert can_do([1], [S(5)/4, S(7)/4])  # PFDD
+    assert can_do([Rational(-1, 2)], [S.Half, 1])  # struve
+    assert can_do([1], [S.Half, S.Half])  # struve
+    assert can_do([Rational(1, 4)], [S.Half, Rational(5, 4)])  # PFDD
+    assert can_do([Rational(3, 4)], [Rational(3, 2), Rational(7, 4)])  # PFDD
+    assert can_do([1], [Rational(1, 4), Rational(3, 4)])  # PFDD
+    assert can_do([1], [Rational(3, 4), Rational(5, 4)])  # PFDD
+    assert can_do([1], [Rational(5, 4), Rational(7, 4)])  # PFDD
     # TODO LOTS more
 
     # 7.15.2
-    assert can_do([S(1)/2, 1], [S(3)/4, S(5)/4, S(3)/2])  # PFDD
-    assert can_do([S(1)/2, 1], [S(7)/4, S(5)/4, S(3)/2])  # PFDD
+    assert can_do([S.Half, 1], [Rational(3, 4), Rational(5, 4), Rational(3, 2)])  # PFDD
+    assert can_do([S.Half, 1], [Rational(7, 4), Rational(5, 4), Rational(3, 2)])  # PFDD
 
     # 7.16.1
-    assert can_do([], [S(1)/3, S(2/3)])  # PFDD
-    assert can_do([], [S(2)/3, S(4/3)])  # PFDD
-    assert can_do([], [S(5)/3, S(4/3)])  # PFDD
+    assert can_do([], [Rational(1, 3), S(2/3)])  # PFDD
+    assert can_do([], [Rational(2, 3), S(4/3)])  # PFDD
+    assert can_do([], [Rational(5, 3), S(4/3)])  # PFDD
 
     # XXX this does not *evaluate* right??
     assert can_do([], [a, a + S.Half, 2*a - 1])
@@ -1042,4 +1039,4 @@ def test_omgissue_203():
     h = hyper((-5, -3, -4), (-6, -6), 1)
     assert hyperexpand(h) == Rational(1, 30)
     h = hyper((-6, -7, -5), (-6, -6), 1)
-    assert hyperexpand(h) == -Rational(1, 6)
+    assert hyperexpand(h) == Rational(-1, 6)
