@@ -1,5 +1,6 @@
-from sympy import sqrt, root, S, Symbol, sqrtdenest, Integral, cos
+from sympy import sqrt, root, Symbol, sqrtdenest, Integral, cos, Rational, I
 from sympy.simplify.sqrtdenest import _subsets as subsets
+from sympy.testing.pytest import slow
 
 r2, r3, r5, r6, r7, r10, r15, r29 = [sqrt(x) for x in [2, 3, 5, 6, 7, 10,
                                           15, 29]]
@@ -12,9 +13,9 @@ def test_sqrtdenest():
         sqrt(r2): sqrt(r2),
         sqrt(5 + r7): sqrt(5 + r7),
         sqrt(3 + sqrt(5 + 2*r7)):
-         3*r2*(5 + 2*r7)**(S(1)/4)/(2*sqrt(6 + 3*r7)) +
-         r2*sqrt(6 + 3*r7)/(2*(5 + 2*r7)**(S(1)/4)),
-        sqrt(3 + 2*r3): 3**(S(3)/4)*(r6/2 + 3*r2/2)/3}
+         3*r2*(5 + 2*r7)**Rational(1, 4)/(2*sqrt(6 + 3*r7)) +
+         r2*sqrt(6 + 3*r7)/(2*(5 + 2*r7)**Rational(1, 4)),
+        sqrt(3 + 2*r3): 3**Rational(3, 4)*(r6/2 + 3*r2/2)/3}
     for i in d:
         assert sqrtdenest(i) == d[i]
 
@@ -70,7 +71,7 @@ def test_sqrtdenest_rec():
     assert sqrtdenest(sqrt(-8*r2 - 2*r5 + 18)) == -r10 + 1 + r2 + r5
     assert sqrtdenest(sqrt(8*r2 + 2*r5 - 18)) == \
         sqrt(-1)*(-r10 + 1 + r2 + r5)
-    assert sqrtdenest(sqrt(8*r2/3 + 14*r5/3 + S(154)/9)) == \
+    assert sqrtdenest(sqrt(8*r2/3 + 14*r5/3 + Rational(154, 9))) == \
         -r10/3 + r2 + r5 + 3
     assert sqrtdenest(sqrt(sqrt(2*r6 + 5) + sqrt(2*r7 + 8))) == \
         sqrt(1 + r2 + r3 + r7)
@@ -94,10 +95,6 @@ def test_sqrtdenest3():
     z = sqrt(13 - 2*r10 + 2*r2*sqrt(-2*r10 + 11))
     assert sqrtdenest(z) == -1 + r2 + r10
     assert sqrtdenest(z, max_iter=1) == -1 + sqrt(2) + sqrt(10)
-    n = sqrt(2*r6/7 + 2*r7/7 + 2*sqrt(42)/7 + 2)
-    d = sqrt(16 - 2*r29 + 2*sqrt(55 - 10*r29))
-    assert sqrtdenest(n/d).equals(
-        r7*(1 + r6 + r7)/(7*(sqrt(-2*r29 + 11) + r5)))
     z = sqrt(sqrt(r2 + 2) + 2)
     assert sqrtdenest(z) == z
     assert sqrtdenest(sqrt(-2*r10 + 4*r2*sqrt(-2*r10 + 11) + 20)) == \
@@ -109,6 +106,16 @@ def test_sqrtdenest3():
     assert sqrtdenest(z) == sqrt(r2*r + r3*r + r10 + r15 + 5)
 
 
+@slow
+def test_sqrtdenest3_slow():
+    # Slow because of the equals, not the sqrtdenest
+    # Using == does not work as 7*(sqrt(-2*r29 + 11) + r5) is expanded
+    # automatically
+    n = sqrt(2*r6/7 + 2*r7/7 + 2*sqrt(42)/7 + 2)
+    d = sqrt(16 - 2*r29 + 2*sqrt(55 - 10*r29))
+    assert sqrtdenest(n/d).equals(
+        r7*(1 + r6 + r7)/(7*(sqrt(-2*r29 + 11) + r5)))
+
 def test_sqrtdenest4():
     # see Denest_en.pdf in https://github.com/sympy/sympy/issues/3192
     z = sqrt(8 - r2*sqrt(5 - r5) - sqrt(3)*(1 + r5))
@@ -173,6 +180,14 @@ def test_issue_5653():
     assert sqrtdenest(
         sqrt(2 + sqrt(2 + sqrt(2)))) == sqrt(2 + sqrt(2 + sqrt(2)))
 
+def test_issue_12420():
+    assert sqrtdenest((3 - sqrt(2)*sqrt(4 + 3*I) + 3*I)/2) == I
+    e = 3 - sqrt(2)*sqrt(4 + I) + 3*I
+    assert sqrtdenest(e) == e
 
 def test_sqrt_ratcomb():
     assert sqrtdenest(sqrt(1 + r3) + sqrt(3 + 3*r3) - sqrt(10 + 6*r3)) == 0
+
+def test_issue_18041():
+    e = -sqrt(-2 + 2*sqrt(3)*I)
+    assert sqrtdenest(e) == -1 - sqrt(3)*I
