@@ -6,7 +6,7 @@ from sympy.core import S
 from sympy.polys import Poly, groebner, roots
 from sympy.polys.polytools import parallel_poly_from_expr
 from sympy.polys.polyerrors import (ComputationFailed,
-    PolificationFailed, CoercionFailed, PolynomialError)
+    PolificationFailed, CoercionFailed)
 from sympy.simplify import rcollect
 from sympy.utilities import default_sort_key, postfixes
 from sympy.utilities.misc import filldedent
@@ -20,6 +20,24 @@ def solve_poly_system(seq, *gens, **args):
     """
     Solve a system of polynomial equations.
 
+    Parameters
+    ==========
+
+    seq: a list/tuple/set
+        Listing all the equations that are needed to be solved
+    gens: generators
+        generators of the equations in seq for which we want the
+        solutions
+    args: Keyword arguments
+        Special options for solving the equations
+
+    Returns
+    =======
+
+    List[Tuple]
+        A List of tuples. Solutions for symbols that satisfy the
+        equations listed in seq
+
     Examples
     ========
 
@@ -50,6 +68,23 @@ def solve_poly_system(seq, *gens, **args):
 def solve_biquadratic(f, g, opt):
     """Solve a system of two bivariate quadratic polynomial equations.
 
+    Parameters
+    ==========
+
+    f: a single Expr or Poly
+        First equation
+    g: a single Expr or Poly
+        Second Equation
+    opt: an Options object
+        For specifying keyword arguments and generators
+
+    Returns
+    =======
+
+    List[Tuple]
+        A List of tuples. Solutions for symbols that satisfy the
+        equations listed in seq.
+
     Examples
     ========
 
@@ -66,7 +101,7 @@ def solve_biquadratic(f, g, opt):
     >>> a = Poly(y + x**2 - 3, y, x, domain='ZZ')
     >>> b = Poly(-y + x - 4, y, x, domain='ZZ')
     >>> solve_biquadratic(a, b, NewOption)
-    [(-sqrt(29)/2 + 7/2, -sqrt(29)/2 - 1/2), (sqrt(29)/2 + 7/2, -1/2 + \
+    [(7/2 - sqrt(29)/2, -sqrt(29)/2 - 1/2), (sqrt(29)/2 + 7/2, -1/2 + \
       sqrt(29)/2)]
     """
     G = groebner([f, g])
@@ -84,7 +119,7 @@ def solve_biquadratic(f, g, opt):
         raise SolveFailed
 
     p = Poly(p, x, expand=False)
-    p_roots = [ rcollect(expr, y) for expr in roots(p).keys() ]
+    p_roots = [rcollect(expr, y) for expr in roots(p).keys()]
 
     q = q.ltrim(-1)
     q_roots = list(roots(q).keys())
@@ -125,6 +160,21 @@ def solve_generic(polys, opt):
     means only that roots() failed, but the system is solvable. To
     overcome this difficulty use numerical algorithms instead.
 
+    Parameters
+    ==========
+
+    polys: a list/tuple/set
+        Listing all the polynomial equations that are needed to be solved
+    opt: an Options object
+        For specifying keyword arguments and generators
+
+    Returns
+    =======
+
+    List[Tuple]
+        A List of tuples. Solutions for symbols that satisfy the
+        equations listed in seq
+
     References
     ==========
 
@@ -162,7 +212,7 @@ def solve_generic(polys, opt):
     def _is_univariate(f):
         """Returns True if 'f' is univariate in its last variable. """
         for monom in f.monoms():
-            if any(m for m in monom[:-1]):
+            if any(monom[:-1]):
                 return False
 
         return True
@@ -180,7 +230,7 @@ def solve_generic(polys, opt):
         """Recursively solves reduced polynomial systems. """
         if len(system) == len(gens) == 1:
             zeros = list(roots(system[0], gens[-1]).keys())
-            return [ (zero,) for zero in zeros ]
+            return [(zero,) for zero in zeros]
 
         basis = groebner(system, gens, polys=True)
 
@@ -209,7 +259,7 @@ def solve_generic(polys, opt):
             return []
 
         if len(basis) == 1:
-            return [ (zero,) for zero in zeros ]
+            return [(zero,) for zero in zeros]
 
         solutions = []
 
@@ -252,6 +302,24 @@ def solve_triangulated(polys, *gens, **args):
     domain and then by iteratively computing polynomial factorizations in
     appropriately constructed algebraic extensions of the ground domain.
 
+    Parameters
+    ==========
+
+    polys: a list/tuple/set
+        Listing all the equations that are needed to be solved
+    gens: generators
+        generators of the equations in polys for which we want the
+        solutions
+    args: Keyword arguments
+        Special options for solving the equations
+
+    Returns
+    =======
+
+    List[Tuple]
+        A List of tuples. Solutions for symbols that satisfy the
+        equations listed in polys
+
     Examples
     ========
 
