@@ -2,7 +2,8 @@
 
 from sympy import (And, Eq, FiniteSet, Ge, Gt, Interval, Le, Lt, Ne, oo, I,
                    Or, S, sin, cos, tan, sqrt, Symbol, Union, Integral, Sum,
-                   Function, Poly, PurePoly, pi, root, log, exp, Dummy, Abs)
+                   Function, Poly, PurePoly, pi, root, log, exp, Dummy, Abs,
+                   Piecewise, Rational)
 from sympy.solvers.inequalities import (reduce_inequalities,
                                         solve_poly_inequality as psolve,
                                         reduce_rational_inequalities,
@@ -14,7 +15,7 @@ from sympy.solvers.solvers import solve
 from sympy.solvers.solveset import solveset
 from sympy.abc import x, y
 
-from sympy.utilities.pytest import raises, slow, XFAIL
+from sympy.testing.pytest import raises, XFAIL
 
 
 inf = oo.evalf()
@@ -117,7 +118,7 @@ def test_reduce_poly_inequalities_complex_relational():
         [[Ne(x**2, 0)]], x, relational=True) == \
         And(Gt(x, -oo), Lt(x, oo), Ne(x, 0))
 
-    for one in (S(1), S(1.0)):
+    for one in (S.One, S(1.0)):
         inf = one*oo
         assert reduce_rational_inequalities(
             [[Eq(x**2, one)]], x, relational=True) == \
@@ -166,7 +167,7 @@ def test_reduce_rational_inequalities_real_relational():
 
     assert reduce_rational_inequalities([[(3*x + 1)/(x + 4) >= 1]], x,
         relational=False) == \
-        Union(Interval.open(-oo, -4), Interval.Ropen(S(3)/2, oo))
+        Union(Interval.open(-oo, -4), Interval.Ropen(Rational(3, 2), oo))
 
     assert reduce_rational_inequalities([[(x - 8)/x <= 3 - x]], x,
         relational=False) == \
@@ -184,21 +185,21 @@ def test_reduce_abs_inequalities():
     assert reduce_inequalities(e, x) == ans
     assert reduce_inequalities(abs(x - 5)) == Eq(x, 5)
     assert reduce_inequalities(
-        abs(2*x + 3) >= 8) == Or(And(Le(S(5)/2, x), Lt(x, oo)),
-        And(Le(x, -S(11)/2), Lt(-oo, x)))
+        abs(2*x + 3) >= 8) == Or(And(Le(Rational(5, 2), x), Lt(x, oo)),
+        And(Le(x, Rational(-11, 2)), Lt(-oo, x)))
     assert reduce_inequalities(abs(x - 4) + abs(
-        3*x - 5) < 7) == And(Lt(S(1)/2, x), Lt(x, 4))
+        3*x - 5) < 7) == And(Lt(S.Half, x), Lt(x, 4))
     assert reduce_inequalities(abs(x - 4) + abs(3*abs(x) - 5) < 7) == \
-        Or(And(S(-2) < x, x < -1), And(S(1)/2 < x, x < 4))
+        Or(And(S(-2) < x, x < -1), And(S.Half < x, x < 4))
 
-    nr = Symbol('nr', real=False)
+    nr = Symbol('nr', extended_real=False)
     raises(TypeError, lambda: reduce_inequalities(abs(nr - 5) < 3))
     assert reduce_inequalities(x < 3, symbols=[x, nr]) == And(-oo < x, x < 3)
 
 
 def test_reduce_inequalities_general():
     assert reduce_inequalities(Ge(sqrt(2)*x, 1)) == And(sqrt(2)/2 <= x, x < oo)
-    assert reduce_inequalities(PurePoly(x + 1, x) > 0) == And(S(-1) < x, x < oo)
+    assert reduce_inequalities(x + 1 > 0) == And(S.NegativeOne < x, x < oo)
 
 
 def test_reduce_inequalities_boolean():
@@ -210,8 +211,8 @@ def test_reduce_inequalities_boolean():
 
 def test_reduce_inequalities_multivariate():
     assert reduce_inequalities([Ge(x**2, 1), Ge(y**2, 1)]) == And(
-        Or(And(Le(1, x), Lt(x, oo)), And(Le(x, -1), Lt(-oo, x))),
-        Or(And(Le(1, y), Lt(y, oo)), And(Le(y, -1), Lt(-oo, y))))
+        Or(And(Le(S.One, x), Lt(x, oo)), And(Le(x, -1), Lt(-oo, x))),
+        Or(And(Le(S.One, y), Lt(y, oo)), And(Le(y, -1), Lt(-oo, y))))
 
 
 def test_reduce_inequalities_errors():
@@ -227,20 +228,20 @@ def test__solve_inequalities():
 
 
 def test_issue_6343():
-    eq = -3*x**2/2 - 45*x/4 + S(33)/2 > 0
+    eq = -3*x**2/2 - x*Rational(45, 4) + Rational(33, 2) > 0
     assert reduce_inequalities(eq) == \
-        And(x < -S(15)/4 + sqrt(401)/4, -sqrt(401)/4 - S(15)/4 < x)
+        And(x < Rational(-15, 4) + sqrt(401)/4, -sqrt(401)/4 - Rational(15, 4) < x)
 
 
 def test_issue_8235():
     assert reduce_inequalities(x**2 - 1 < 0) == \
-        And(S(-1) < x, x < S(1))
+        And(S.NegativeOne < x, x < 1)
     assert reduce_inequalities(x**2 - 1 <= 0) == \
-        And(S(-1) <= x, x <= 1)
+        And(S.NegativeOne <= x, x <= 1)
     assert reduce_inequalities(x**2 - 1 > 0) == \
-        Or(And(-oo < x, x < -1), And(x < oo, S(1) < x))
+        Or(And(-oo < x, x < -1), And(x < oo, S.One < x))
     assert reduce_inequalities(x**2 - 1 >= 0) == \
-        Or(And(-oo < x, x <= S(-1)), And(S(1) <= x, x < oo))
+        Or(And(-oo < x, x <= -1), And(S.One <= x, x < oo))
 
     eq = x**8 + x - 9  # we want CRootOf solns here
     sol = solve(eq >= 0)
@@ -248,16 +249,16 @@ def test_issue_8235():
     assert sol == tru
 
     # recast vanilla as real
-    assert solve(sqrt((-x + 1)**2) < 1) == And(S(0) < x, x < 2)
+    assert solve(sqrt((-x + 1)**2) < 1) == And(S.Zero < x, x < 2)
 
 
 def test_issue_5526():
-    assert reduce_inequalities(S(0) <=
+    assert reduce_inequalities(0 <=
         x + Integral(y**2, (y, 1, 3)) - 1, [x]) == \
         (x >= -Integral(y**2, (y, 1, 3)) + 1)
     f = Function('f')
     e = Sum(f(x), (x, 1, 3))
-    assert reduce_inequalities(S(0) <= x + e + y**2, [x]) == \
+    assert reduce_inequalities(0 <= x + e + y**2, [x]) == \
         (x >= -y**2 - Sum(f(x), (x, 1, 3)))
 
 
@@ -274,8 +275,8 @@ def test_solve_univariate_inequality():
         Or(Eq(x, 0), Eq(x, 3))
     # issue 2785:
     assert isolve(x**3 - 2*x - 1 > 0, x, relational=False) == \
-        Union(Interval(-1, -sqrt(5)/2 + S(1)/2, True, True),
-              Interval(S(1)/2 + sqrt(5)/2, oo, True, True))
+        Union(Interval(-1, -sqrt(5)/2 + S.Half, True, True),
+              Interval(S.Half + sqrt(5)/2, oo, True, True))
     # issue 2794:
     assert isolve(x**3 - x**2 + x - 1 > 0, x, relational=False) == \
         Interval(1, oo, True)
@@ -295,7 +296,7 @@ def test_solve_univariate_inequality():
     assert isolve(1/(x - 2) > 0, x) == And(S(2) < x, x < oo)
     den = ((x - 1)*(x - 2)).expand()
     assert isolve((x - 1)/den <= 0, x) == \
-        Or(And(-oo < x, x < 1), And(S(1) < x, x < 2))
+        Or(And(-oo < x, x < 1), And(S.One < x, x < 2))
 
     n = Dummy('n')
     raises(NotImplementedError, lambda: isolve(Abs(x) <= n, x, relational=False))
@@ -326,24 +327,24 @@ def test_solve_univariate_inequality():
 def test_trig_inequalities():
     # all the inequalities are solved in a periodic interval.
     assert isolve(sin(x) < S.Half, x, relational=False) == \
-        Union(Interval(0, pi/6, False, True), Interval(5*pi/6, 2*pi, True, False))
+        Union(Interval(0, pi/6, False, True), Interval(pi*Rational(5, 6), 2*pi, True, False))
     assert isolve(sin(x) > S.Half, x, relational=False) == \
-        Interval(pi/6, 5*pi/6, True, True)
+        Interval(pi/6, pi*Rational(5, 6), True, True)
     assert isolve(cos(x) < S.Zero, x, relational=False) == \
-        Interval(pi/2, 3*pi/2, True, True)
+        Interval(pi/2, pi*Rational(3, 2), True, True)
     assert isolve(cos(x) >= S.Zero, x, relational=False) == \
-        Union(Interval(0, pi/2), Interval(3*pi/2, 2*pi))
+        Union(Interval(0, pi/2), Interval(pi*Rational(3, 2), 2*pi))
 
     assert isolve(tan(x) < S.One, x, relational=False) == \
         Union(Interval.Ropen(0, pi/4), Interval.Lopen(pi/2, pi))
 
     assert isolve(sin(x) <= S.Zero, x, relational=False) == \
-        Union(FiniteSet(S(0)), Interval(pi, 2*pi))
+        Union(FiniteSet(S.Zero), Interval(pi, 2*pi))
 
-    assert isolve(sin(x) <= S(1), x, relational=False) == S.Reals
+    assert isolve(sin(x) <= S.One, x, relational=False) == S.Reals
     assert isolve(cos(x) < S(-2), x, relational=False) == S.EmptySet
-    assert isolve(sin(x) >= S(-1), x, relational=False) == S.Reals
-    assert isolve(cos(x) > S(1), x, relational=False) == S.EmptySet
+    assert isolve(sin(x) >= S.NegativeOne, x, relational=False) == S.Reals
+    assert isolve(cos(x) > S.One, x, relational=False) == S.EmptySet
 
 
 def test_issue_9954():
@@ -357,7 +358,7 @@ def test_issue_9954():
 def test_slow_general_univariate():
     r = rootof(x**5 - x**2 + 1, 0)
     assert solve(sqrt(x) + 1/root(x, 3) > 1) == \
-        Or(And(S(0) < x, x < r**6), And(r**6 < x, x < oo))
+        Or(And(0 < x, x < r**6), And(r**6 < x, x < oo))
 
 
 def test_issue_8545():
@@ -376,24 +377,29 @@ def test_issue_8974():
 def test_issue_10198():
     assert reduce_inequalities(
         -1 + 1/abs(1/x - 1) < 0) == Or(
-        And(-oo < x, x < 0), And(S(0) < x, x < S(1)/2)
+        And(-oo < x, x < 0), And(S.Zero < x, x < S.Half)
         )
     assert reduce_inequalities(abs(1/sqrt(x)) - 1, x) == Eq(x, 1)
     assert reduce_abs_inequality(-3 + 1/abs(1 - 1/x), '<', x) == \
         Or(And(-oo < x, x < 0),
-        And(S(0) < x, x < S(3)/4), And(S(3)/2 < x, x < oo))
+        And(S.Zero < x, x < Rational(3, 4)), And(Rational(3, 2) < x, x < oo))
     raises(ValueError,lambda: reduce_abs_inequality(-3 + 1/abs(
         1 - 1/sqrt(x)), '<', x))
 
 
 def test_issue_10047():
-    # this must remain an inequality, not True, since if x
+    # issue 10047: this must remain an inequality, not True, since if x
     # is not real the inequality is invalid
-    assert solve(sin(x) < 2) == (x <= oo)
+    # assert solve(sin(x) < 2) == (x <= oo)
+
+    # with PR 16956, (x <= oo) autoevaluates when x is extended_real
+    # which is assumed in the current implementation of inequality solvers
+    assert solve(sin(x) < 2) == True
+    assert solveset(sin(x) < 2, domain=S.Reals) == S.Reals
 
 
 def test_issue_10268():
-    assert solve(log(x) < 1000) == And(S(0) < x, x < exp(1000))
+    assert solve(log(x) < 1000) == And(S.Zero < x, x < exp(1000))
 
 
 @XFAIL
@@ -442,11 +448,11 @@ def test__solve_inequality():
 def test__pt():
     from sympy.solvers.inequalities import _pt
     assert _pt(-oo, oo) == 0
-    assert _pt(S(1), S(3)) == 2
-    assert _pt(S(1), oo) == _pt(oo, S(1)) == 2
-    assert _pt(S(1), -oo) == _pt(-oo, S(1)) == S.Half
-    assert _pt(S(-1), oo) == _pt(oo, S(-1)) == -S.Half
-    assert _pt(S(-1), -oo) == _pt(-oo, S(-1)) == -2
+    assert _pt(S.One, S(3)) == 2
+    assert _pt(S.One, oo) == _pt(oo, S.One) == 2
+    assert _pt(S.One, -oo) == _pt(-oo, S.One) == S.Half
+    assert _pt(S.NegativeOne, oo) == _pt(oo, S.NegativeOne) == Rational(-1, 2)
+    assert _pt(S.NegativeOne, -oo) == _pt(-oo, S.NegativeOne) == -2
     assert _pt(x, oo) == _pt(oo, x) == x + 1
     assert _pt(x, -oo) == _pt(-oo, x) == x - 1
-    raises(ValueError, lambda: _pt(Dummy('i', infinite=True), S(1)))
+    raises(ValueError, lambda: _pt(Dummy('i', infinite=True), S.One))
