@@ -1,24 +1,35 @@
 from copy import copy
 
 from sympy.tensor.array.dense_ndim_array import ImmutableDenseNDimArray
-from sympy import Symbol, Rational, SparseMatrix, Dict, diff, symbols, Indexed, IndexedBase
-from sympy.core.compatibility import long
+from sympy import Symbol, Rational, SparseMatrix, Dict, diff, symbols, Indexed, IndexedBase, S
 from sympy.matrices import Matrix
 from sympy.tensor.array.sparse_ndim_array import ImmutableSparseNDimArray
-from sympy.utilities.pytest import raises
+from sympy.testing.pytest import raises
 
 
 def test_ndim_array_initiation():
+    arr_with_no_elements = ImmutableDenseNDimArray([], shape=(0,))
+    assert len(arr_with_no_elements) == 0
+    assert arr_with_no_elements.rank() == 1
+
+    raises(ValueError, lambda: ImmutableDenseNDimArray([0], shape=(0,)))
+    raises(ValueError, lambda: ImmutableDenseNDimArray([1, 2, 3], shape=(0,)))
+    raises(ValueError, lambda: ImmutableDenseNDimArray([], shape=()))
+
+    raises(ValueError, lambda: ImmutableSparseNDimArray([0], shape=(0,)))
+    raises(ValueError, lambda: ImmutableSparseNDimArray([1, 2, 3], shape=(0,)))
+    raises(ValueError, lambda: ImmutableSparseNDimArray([], shape=()))
+
     arr_with_one_element = ImmutableDenseNDimArray([23])
     assert len(arr_with_one_element) == 1
     assert arr_with_one_element[0] == 23
-    assert arr_with_one_element[:] == [23]
+    assert arr_with_one_element[:] == ImmutableDenseNDimArray([23])
     assert arr_with_one_element.rank() == 1
 
     arr_with_symbol_element = ImmutableDenseNDimArray([Symbol('x')])
     assert len(arr_with_symbol_element) == 1
     assert arr_with_symbol_element[0] == Symbol('x')
-    assert arr_with_symbol_element[:] == [Symbol('x')]
+    assert arr_with_symbol_element[:] == ImmutableDenseNDimArray([Symbol('x')])
     assert arr_with_symbol_element.rank() == 1
 
     number5 = 5
@@ -58,26 +69,27 @@ def test_ndim_array_initiation():
     assert array_with_many_args[0, 0] == 0
     assert array_with_many_args.rank() == 2
 
-    shape = (long(3), long(3))
+    shape = (int(3), int(3))
     array_with_long_shape = ImmutableSparseNDimArray.zeros(*shape)
     assert len(array_with_long_shape) == 3 * 3
     assert array_with_long_shape.shape == shape
-    assert array_with_long_shape[long(0), long(0)] == 0
+    assert array_with_long_shape[int(0), int(0)] == 0
     assert array_with_long_shape.rank() == 2
 
-    vector_with_long_shape = ImmutableDenseNDimArray(range(5), long(5))
+    vector_with_long_shape = ImmutableDenseNDimArray(range(5), int(5))
     assert len(vector_with_long_shape) == 5
-    assert vector_with_long_shape.shape == (long(5),)
+    assert vector_with_long_shape.shape == (int(5),)
     assert vector_with_long_shape.rank() == 1
-    raises(ValueError, lambda: vector_with_long_shape[long(5)])
+    raises(ValueError, lambda: vector_with_long_shape[int(5)])
 
     from sympy.abc import x
-    rank_zero_array = ImmutableDenseNDimArray(x)
-    assert len(rank_zero_array) == 0
-    assert rank_zero_array.shape == ()
-    assert rank_zero_array.rank() == 0
-    assert rank_zero_array[()] == x
-    raises(ValueError, lambda: rank_zero_array[0])
+    for ArrayType in [ImmutableDenseNDimArray, ImmutableSparseNDimArray]:
+        rank_zero_array = ArrayType(x)
+        assert len(rank_zero_array) == 1
+        assert rank_zero_array.shape == ()
+        assert rank_zero_array.rank() == 0
+        assert rank_zero_array[()] == x
+        raises(ValueError, lambda: rank_zero_array[0])
 
 
 def test_reshape():
@@ -91,12 +103,28 @@ def test_reshape():
     assert len(array) == 50
 
 
+def test_getitem():
+    for ArrayType in [ImmutableDenseNDimArray, ImmutableSparseNDimArray]:
+        array = ArrayType(range(24)).reshape(2, 3, 4)
+        assert array.tolist() == [[[0, 1, 2, 3], [4, 5, 6, 7], [8, 9, 10, 11]], [[12, 13, 14, 15], [16, 17, 18, 19], [20, 21, 22, 23]]]
+        assert array[0] == ArrayType([[0, 1, 2, 3], [4, 5, 6, 7], [8, 9, 10, 11]])
+        assert array[0, 0] == ArrayType([0, 1, 2, 3])
+        value = 0
+        for i in range(2):
+            for j in range(3):
+                for k in range(4):
+                    assert array[i, j, k] == value
+                    value += 1
+
+    raises(ValueError, lambda: array[3, 4, 5])
+    raises(ValueError, lambda: array[3, 4, 5, 6])
+    raises(ValueError, lambda: array[3, 4, 5, 3:4])
+
+
 def test_iterator():
     array = ImmutableDenseNDimArray(range(4), (2, 2))
-    j = 0
-    for i in array:
-        assert i == j
-        j += 1
+    array[0] == ImmutableDenseNDimArray([0, 1])
+    array[1] == ImmutableDenseNDimArray([2, 3])
 
     array = array.reshape(4)
     j = 0
@@ -111,10 +139,10 @@ def test_sparse():
     # dictionary where all data is, only non-zero entries are actually stored:
     assert len(sparse_array._sparse_array) == 1
 
-    assert list(sparse_array) == [0, 0, 0, 1]
+    assert sparse_array.tolist() == [[0, 0], [0, 1]]
 
-    for i, j in zip(sparse_array, [0, 0, 0, 1]):
-        assert i == j
+    for i, j in zip(sparse_array, [[0, 0], [0, 1]]):
+        assert i == ImmutableSparseNDimArray(j)
 
     def sparse_assignment():
         sparse_array[0, 0] = 123
@@ -123,6 +151,24 @@ def test_sparse():
     raises(TypeError, sparse_assignment)
     assert len(sparse_array._sparse_array) == 1
     assert sparse_array[0, 0] == 0
+    assert sparse_array/0 == ImmutableSparseNDimArray([[S.NaN, S.NaN], [S.NaN, S.ComplexInfinity]], (2, 2))
+
+    # test for large scale sparse array
+    # equality test
+    assert ImmutableSparseNDimArray.zeros(100000, 200000) == ImmutableSparseNDimArray.zeros(100000, 200000)
+
+    # __mul__ and __rmul__
+    a = ImmutableSparseNDimArray({200001: 1}, (100000, 200000))
+    assert a * 3 == ImmutableSparseNDimArray({200001: 3}, (100000, 200000))
+    assert 3 * a == ImmutableSparseNDimArray({200001: 3}, (100000, 200000))
+    assert a * 0 == ImmutableSparseNDimArray({}, (100000, 200000))
+    assert 0 * a == ImmutableSparseNDimArray({}, (100000, 200000))
+
+    # __div__
+    assert a/3 == ImmutableSparseNDimArray({200001: Rational(1, 3)}, (100000, 200000))
+
+    # __neg__
+    assert -a == ImmutableSparseNDimArray({200001: -1}, (100000, 200000))
 
 
 def test_calculation():
@@ -132,14 +178,14 @@ def test_calculation():
 
     c = a + b
     for i in c:
-        assert i == 10
+        assert i == ImmutableDenseNDimArray([10, 10, 10])
 
     assert c == ImmutableDenseNDimArray([10]*9, (3, 3))
     assert c == ImmutableSparseNDimArray([10]*9, (3, 3))
 
     c = b - a
     for i in c:
-        assert i == 8
+        assert i == ImmutableDenseNDimArray([8, 8, 8])
 
     assert c == ImmutableDenseNDimArray([8]*9, (3, 3))
     assert c == ImmutableSparseNDimArray([8]*9, (3, 3))
@@ -155,7 +201,7 @@ def test_ndim_array_converting():
     assert (isinstance(matrix, Matrix))
 
     for i in range(len(dense_array)):
-        assert dense_array[i] == matrix[i]
+        assert dense_array[dense_array._get_tuple_index(i)] == matrix[i]
     assert matrix.shape == dense_array.shape
 
     assert ImmutableDenseNDimArray(matrix) == dense_array
@@ -171,7 +217,7 @@ def test_ndim_array_converting():
     assert(isinstance(matrix, SparseMatrix))
 
     for i in range(len(sparse_array)):
-        assert sparse_array[i] == matrix[i]
+        assert sparse_array[sparse_array._get_tuple_index(i)] == matrix[i]
     assert matrix.shape == sparse_array.shape
 
     assert ImmutableSparseNDimArray(matrix) == sparse_array
@@ -285,7 +331,7 @@ def test_rebuild_immutable_arrays():
 def test_slices():
     md = ImmutableDenseNDimArray(range(10, 34), (2, 3, 4))
 
-    assert md[:] == md._array
+    assert md[:] == ImmutableDenseNDimArray(range(10, 34), (2, 3, 4))
     assert md[:, :, 0].tomatrix() == Matrix([[10, 14, 18], [22, 26, 30]])
     assert md[0, 1:2, :].tomatrix() == Matrix([[14, 15, 16, 17]])
     assert md[0, 1:3, :].tomatrix() == Matrix([[14, 15, 16, 17], [18, 19, 20, 21]])
@@ -294,8 +340,7 @@ def test_slices():
     sd = ImmutableSparseNDimArray(range(10, 34), (2, 3, 4))
     assert sd == ImmutableSparseNDimArray(md)
 
-    assert sd[:] == md._array
-    assert sd[:] == list(sd)
+    assert sd[:] == ImmutableSparseNDimArray(range(10, 34), (2, 3, 4))
     assert sd[:, :, 0].tomatrix() == Matrix([[10, 14, 18], [22, 26, 30]])
     assert sd[0, 1:2, :].tomatrix() == Matrix([[14, 15, 16, 17]])
     assert sd[0, 1:3, :].tomatrix() == Matrix([[14, 15, 16, 17], [18, 19, 20, 21]])
@@ -321,9 +366,13 @@ def test_diff_and_applyfunc():
     assert sdn == ImmutableSparseNDimArray([[x/2, y/2], [x*z/2, x*y*z/2]])
     assert sd != sdn
 
+    sdp = sd.applyfunc(lambda x: x+1)
+    assert sdp == ImmutableSparseNDimArray([[x + 1, y + 1], [x*z + 1, x*y*z + 1]])
+    assert sd != sdp
+
 
 def test_op_priority():
-    from sympy.abc import x, y, z
+    from sympy.abc import x
     md = ImmutableDenseNDimArray([1, 2, 3])
     e1 = (1+x)*md
     e2 = md*(1+x)
@@ -331,8 +380,8 @@ def test_op_priority():
     assert e1 == e2
 
     sd = ImmutableSparseNDimArray([1, 2, 3])
-    e3 = (1+x)*md
-    e4 = md*(1+x)
+    e3 = (1+x)*sd
+    e4 = sd*(1+x)
     assert e3 == ImmutableDenseNDimArray([1+x, 2+2*x, 3+3*x])
     assert e3 == e4
 
@@ -384,3 +433,8 @@ def test_issue_12665():
     arr = ImmutableDenseNDimArray([1, 2, 3])
     # This should NOT raise an exception:
     hash(arr)
+
+
+def test_zeros_without_shape():
+    arr = ImmutableDenseNDimArray.zeros()
+    assert arr == ImmutableDenseNDimArray(0)
