@@ -76,7 +76,7 @@ from string import Template
 from warnings import warn
 
 from sympy.core.cache import cacheit
-from sympy.core.compatibility import range, iterable
+from sympy.core.compatibility import iterable
 from sympy.core.function import Lambda
 from sympy.core.relational import Eq
 from sympy.core.symbol import Dummy, Symbol
@@ -133,7 +133,8 @@ class CodeWrapper(object):
             routines, self.filename, True, self.include_header,
             self.include_empty)
 
-    def wrap_code(self, routine, helpers=[]):
+    def wrap_code(self, routine, helpers=None):
+        helpers = helpers or []
         if self.filepath:
             workdir = os.path.abspath(self.filepath)
         else:
