@@ -2,14 +2,14 @@
 
 from __future__ import print_function, division
 
+from typing import List
+
 import sys
 import os
 import re as _re
 import struct
 from textwrap import fill, dedent
-from sympy.core.compatibility import (get_function_name, range, as_int,
-    string_types)
-
+from sympy.core.compatibility import get_function_name, as_int
 
 
 class Undecidable(ValueError):
@@ -63,7 +63,7 @@ def strlines(s, c=64, short=False):
     ========
     filldedent, rawlines
     """
-    if type(s) not in string_types:
+    if type(s) is not str:
         raise ValueError('expecting string input')
     if '\n' in s:
         return rawlines(s)
@@ -174,7 +174,7 @@ ARCH = str(struct.calcsize('P') * 8) + "-bit"
 # XXX: PyPy doesn't support hash randomization
 HASH_RANDOMIZATION = getattr(sys.flags, 'hash_randomization', False)
 
-_debug_tmp = []
+_debug_tmp = []  # type: List[str]
 _debug_iter = 0
 
 def debug_decorator(func):
@@ -412,7 +412,6 @@ def translate(s, a, b=None, c=None):
     ========
 
     >>> from sympy.utilities.misc import translate
-    >>> from sympy.core.compatibility import unichr
     >>> abc = 'abc'
     >>> translate(abc, None, 'a')
     'bc'
@@ -432,7 +431,6 @@ def translate(s, a, b=None, c=None):
     >>> translate(abc, {'ab': 'x', 'bc': 'y'}) in ('xc', 'ay')
     True
     """
-    from sympy.core.compatibility import maketrans, PY3
 
     mr = {}
     if a is None:
@@ -456,40 +454,13 @@ def translate(s, a, b=None, c=None):
                 a = b = ''
         elif len(a) != len(b):
             raise ValueError('oldchars and newchars have different lengths')
-    if PY3:
-        if c:
-            s = s.translate(maketrans('', '', c))
-        s = replace(s, mr)
-        return s.translate(maketrans(a, b))
-    else:
-        # when support for Python 2 is dropped, this if-else-block
-        # can be replaced with the if-clause
-        if c:
-            c = list(c)
-            rem = {}
-            for i in range(-1, -1 - len(c), -1):
-                if ord(c[i]) > 255:
-                    rem[c[i]] = ''
-                    c.pop(i)
-            s = s.translate(None, ''.join(c))
-            s = replace(s, rem)
-            if a:
-                a = list(a)
-                b = list(b)
-                for i in range(-1, -1 - len(a), -1):
-                    if ord(a[i]) > 255 or ord(b[i]) > 255:
-                        mr[a.pop(i)] = b.pop(i)
-                a = ''.join(a)
-                b = ''.join(b)
-        s = replace(s, mr)
-        table = maketrans(a, b)
-        # s may have become unicode which uses the py3 syntax for translate
-        if isinstance(table, str) and isinstance(s, str):
-            s = s.translate(table)
-        else:
-            s = s.translate(dict(
-                [(i, ord(c)) for i, c in enumerate(table)]))
-        return s
+
+    if c:
+        val = str.maketrans('', '', c)
+        s = s.translate(val)
+    s = replace(s, mr)
+    n = str.maketrans(a, b)
+    return s.translate(n)
 
 
 def ordinal(num):
